@@ -1,5 +1,7 @@
 /*
-* Copyright (C) {1999}, International Business Machines Corporation and others. All Rights Reserved.
+**********************************************************************
+* Copyright (C) 1999-2007, International Business Machines Corporation
+* and others. All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   11/17/99    aliu        Creation.
@@ -8,15 +10,20 @@
 #ifndef RBT_DATA_H
 #define RBT_DATA_H
 
+#include "unicode/utypes.h"
+#include "unicode/uclean.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
+#include "unicode/uobject.h"
 #include "rbt_set.h"
+#include "hash.h"
 
 U_NAMESPACE_BEGIN
 
 class UnicodeFunctor;
-class UnicodeString;
 class UnicodeMatcher;
 class UnicodeReplacer;
-class Hashtable;
 
 /**
  * The rule data for a RuleBasedTransliterators.  RBT objects hold
@@ -35,7 +42,7 @@ class Hashtable;
  * data structure handles this.  See the parsing code for more
  * details.
  */
-class U_I18N_API TransliterationRuleData {
+class TransliterationRuleData : public UMemory {
 
 public:
 
@@ -55,7 +62,7 @@ public:
      * data.variables.  The stand-in also represents the UnicodeMatcher in
      * the stored rules.
      */
-    Hashtable* variableNames;
+    Hashtable variableNames;
 
     /**
      * Map category variable (UChar) to set (UnicodeFunctor).
@@ -68,6 +75,15 @@ public:
      */
     UnicodeFunctor** variables;
 
+    /**
+     * Flag that indicates whether the variables are owned (if a single
+     * call to Transliterator::createFromRules() produces a CompoundTransliterator
+     * with more than one RuleBasedTransliterator as children, they all share
+     * the same variables list, so only the first one is considered to own
+     * the variables)
+     */
+    UBool variablesAreOwned;
+
     /**
      * The character that represents variables[0].  Characters
      * variablesBase through variablesBase +
@@ -82,15 +98,27 @@ public:
 
 public:
 
+    /**
+     * Constructor
+     * @param status Output param set to success/failure code on exit.
+     */
     TransliterationRuleData(UErrorCode& status);
 
+    /**
+     * Copy Constructor
+     */
     TransliterationRuleData(const TransliterationRuleData&);
 
+    /**
+     * destructor
+     */
     ~TransliterationRuleData();
 
     /**
      * Given a stand-in character, return the UnicodeFunctor that it
      * represents, or NULL if it doesn't represent anything.
+     * @param standIn    the given stand-in character.
+     * @return           the UnicodeFunctor that 'standIn' represents
      */
     UnicodeFunctor* lookup(UChar32 standIn) const;
 
@@ -98,6 +126,8 @@ public:
      * Given a stand-in character, return the UnicodeMatcher that it
      * represents, or NULL if it doesn't represent anything or if it
      * represents something that is not a matcher.
+     * @param standIn    the given stand-in character.
+     * @return           return the UnicodeMatcher that 'standIn' represents
      */
     UnicodeMatcher* lookupMatcher(UChar32 standIn) const;
 
@@ -105,10 +135,18 @@ public:
      * Given a stand-in character, return the UnicodeReplacer that it
      * represents, or NULL if it doesn't represent anything or if it
      * represents something that is not a replacer.
+     * @param standIn    the given stand-in character.
+     * @return           return the UnicodeReplacer that 'standIn' represents
      */
     UnicodeReplacer* lookupReplacer(UChar32 standIn) const;
+
+
+private:
+    TransliterationRuleData &operator=(const TransliterationRuleData &other); // forbid copying of this class
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
