@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2006, International Business Machines Corporation and    *
+* Copyright (C) 1997-2007, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -24,6 +24,8 @@
 ********************************************************************************
 */
 
+#define ZID_KEY_MAX 128
+
 #include "unicode/utypes.h"
 
 #if !UCONFIG_NO_FORMATTING
@@ -81,6 +83,9 @@ static const UChar SUPPRESS_NEGATIVE_PREFIX[] = {0xAB00, 0};
  * These are the tags we expect to see in normal resource bundle files associated
  * with a locale.
  */
+static const char kSUPPLEMENTAL[]="supplementalData";
+static const char gZoneFormattingTag[]="zoneFormatting";
+static const char gAliases[]="aliases";
 static const char gDateTimePatternsTag[]="DateTimePatterns";
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(SimpleDateFormat)
@@ -230,6 +235,9 @@ SimpleDateFormat::SimpleDateFormat(const SimpleDateFormat& other)
 
 SimpleDateFormat& SimpleDateFormat::operator=(const SimpleDateFormat& other)
 {
+    if (this == &other) {
+        return *this;
+    }
     DateFormat::operator=(other);
 
     delete fSymbols;
@@ -537,6 +545,7 @@ SimpleDateFormat::fgPatternIndexToCalendarField[] =
     /*L*/   UCAL_MONTH,
     /*Q*/   UCAL_MONTH,
     /*q*/   UCAL_MONTH,
+    /*V*/   UCAL_ZONE_OFFSET,
 };
 
 // Map index into pattern character string to DateFormat field number
@@ -555,6 +564,7 @@ SimpleDateFormat::fgPatternIndexToDateFormatField[] = {
     /*L*/   UDAT_STANDALONE_MONTH_FIELD,
     /*Q*/   UDAT_QUARTER_FIELD,
     /*q*/   UDAT_STANDALONE_QUARTER_FIELD,
+    /*V*/   UDAT_TIMEZONE_SPECIAL_FIELD,
 };
 
 //----------------------------------------------------------------------
@@ -765,9 +775,11 @@ SimpleDateFormat::subFormat(UnicodeString &appendTo,
     // then the time zone shows up as "GMT+hh:mm" or "GMT-hh:mm" (where "hh:mm" is the
     // offset from GMT) regardless of how many z's were in the pattern symbol
     case UDAT_TIMEZONE_FIELD: 
+    case UDAT_TIMEZONE_SPECIAL_FIELD: 
     case UDAT_TIMEZONE_GENERIC_FIELD: {
         UnicodeString str;
         UnicodeString zid;
+        UnicodeString mzid;
         UnicodeString displayString;
         zid = fSymbols->getZoneID(cal.getTimeZone().getID(str), zid, status);
         if(U_FAILURE(status)){
@@ -777,25 +789,107 @@ SimpleDateFormat::subFormat(UnicodeString &appendTo,
             appendGMT(appendTo, cal, status);
         }
         else {
-
+            zoneIDCanonicalize(zid);
             if (patternCharIndex == UDAT_TIMEZONE_GENERIC_FIELD) {
                 if(count < 4){
-                    fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_GENERIC, displayString, status);
+                    if (cal.getTimeZone().useDaylightTime()) {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_GENERIC, displayString, status);
+                        if ( !fSymbols->isCommonlyUsed(zid)) {
+                            displayString.remove();
+                        }
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_GENERIC, cal, displayString, status);
+                            if ( !fSymbols->isCommonlyUsed(mzid)) {
+                                displayString.remove();
+                            }
+                        }
+                    }
+                    else {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, displayString, status);
+                        if ( !fSymbols->isCommonlyUsed(zid)) {
+                            displayString.remove();
+                        }
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, cal, displayString, status);
+                            if ( !fSymbols->isCommonlyUsed(mzid)) {
+                                displayString.remove();
+                            }
+                        }
+                    }
+                    if(displayString.length()==0) {
+                       fSymbols->getFallbackString(zid, displayString, status);
+                    }
                 }else{
-                    fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_LONG_GENERIC, displayString, status);
+                    if (cal.getTimeZone().useDaylightTime()) {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_LONG_GENERIC, displayString, status);
+                        if(displayString.length()==0) {
+                           mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_LONG_GENERIC, cal, displayString, status);
+                        }
+                    }
+                    else {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_LONG_STANDARD, displayString, status);
+                        if(displayString.length()==0) {
+                           mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_LONG_STANDARD, cal, displayString, status);
+                        }
+                    }
+
+                    if(displayString.length()==0) {
+                       fSymbols->getFallbackString(zid, displayString, status);
+                    }
+                }
+            }
+            else if (patternCharIndex == UDAT_TIMEZONE_SPECIAL_FIELD) {
+                if(count == 4){ // VVVV format - always get fallback string.
+                       fSymbols->getFallbackString(zid, displayString, status);
+                }
+                else if (count == 1){ // V format - ignore commonlyUsed
+                    if (cal.get(UCAL_DST_OFFSET, status) != 0) {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_DAYLIGHT, displayString, status);
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_DAYLIGHT, cal, displayString, status);
+                        }
+                    }
+                    else {
+                        fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, displayString, status);
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, cal, displayString, status);
+                        }
+                    }
                 }
             } else {
                 if (cal.get(UCAL_DST_OFFSET, status) != 0) {
                     if(count < 4){
                         fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_DAYLIGHT, displayString, status);
+                        if ( fSymbols->isCommonlyUsed(zid) == FALSE ) {
+                            displayString.remove();
+                        }
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_DAYLIGHT, cal, displayString, status);
+                            if ( fSymbols->isCommonlyUsed(mzid) == FALSE ) {
+                                displayString.remove();
+                            }
+                        }
                     }else{
                         fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_LONG_DAYLIGHT, displayString, status);
+                        if(displayString.length()==0)
+                           fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_LONG_DAYLIGHT, cal, displayString, status);
                     }
                 }else{
                     if(count < 4){
                         fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, displayString, status);
+                        if ( !fSymbols->isCommonlyUsed(zid)) {
+                            displayString.remove();
+                        }
+                        if(displayString.length()==0) {
+                            mzid = fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_SHORT_STANDARD, cal, displayString, status);
+                            if ( fSymbols->isCommonlyUsed(mzid) == FALSE ) {
+                                displayString.remove();
+                            }
+                        }
                     }else{
                         fSymbols->getZoneString(zid, DateFormatSymbols::TIMEZONE_LONG_STANDARD, displayString, status);
+                        if(displayString.length()==0)
+                           fSymbols->getMetazoneString(zid, DateFormatSymbols::TIMEZONE_LONG_STANDARD, cal, displayString, status);
                     }
                 }
             }
@@ -817,7 +911,7 @@ SimpleDateFormat::subFormat(UnicodeString &appendTo,
                 value = -value;
                 sign = 45/*'-'*/;
             }
-            value = (value / 3) * 5 + (value % 60); // minutes => KKmm
+            value = (value / 60) * 100 + (value % 60); // minutes => KKmm
             appendTo += sign;
             zeroPaddingNumber(appendTo, value, 4, 4);
         }
@@ -861,7 +955,86 @@ SimpleDateFormat::subFormat(UnicodeString &appendTo,
         pos.setEndIndex(appendTo.length());
     }
 }
+//----------------------------------------------------------------------
 
+void
+SimpleDateFormat::zoneIDCanonicalize(UnicodeString &zid) const
+{
+    UnicodeString colon = UNICODE_STRING_SIMPLE(":");
+    UnicodeString solidus = UNICODE_STRING_SIMPLE("/");
+    char zidkey[ZID_KEY_MAX];
+    int32_t len;
+
+    UErrorCode status = U_ZERO_ERROR;
+    UResourceBundle* supplementalDataBundle = ures_openDirect(NULL, kSUPPLEMENTAL, &status);
+    if (U_FAILURE(status)) {
+        return;
+    }
+       
+    UResourceBundle* zoneFormatting = ures_getByKey(supplementalDataBundle, gZoneFormattingTag, NULL, &status);
+
+    // First try to lookup the zone itself, since most of the time the canonical ID and the ID itself
+    // will be the same.  This should save us a significant amount of time.
+
+    len = zid.length();
+    len = (len >= (ZID_KEY_MAX-1) ? ZID_KEY_MAX-1 : len);
+    u_UCharsToChars(zid.getBuffer(), zidkey, len);
+    zidkey[len] = 0; // NULL terminate
+
+    // Replace / with : for zid
+    len = (int32_t)uprv_strlen(zidkey);
+    for (int i = 0; i < len; i++) {
+        if (zidkey[i] == '/') {
+            zidkey[i] = ':';
+        }
+    }
+
+    UResourceBundle* tryThisZone = ures_getByKey(zoneFormatting,zidkey,NULL,&status);
+    if (U_SUCCESS(status)) {
+         ures_close(tryThisZone);
+         ures_close(zoneFormatting);
+         ures_close(supplementalDataBundle);
+         return;
+    }
+    else {
+        status = U_ZERO_ERROR;
+    }
+
+    // Didn't find it, so go searching for an alias
+
+    while ( ures_hasNext(zoneFormatting)) {
+        UResourceBundle *currentZone = ures_getNextResource(zoneFormatting,NULL,&status);
+        if (U_FAILURE(status)) {
+             break;
+        }
+        
+        const char *currentZoneString= ures_getKey(currentZone);
+
+        UResourceBundle *zoneAliases = ures_getByKey(currentZone,gAliases,NULL, &status);
+        if (U_FAILURE(status)) {
+            status = U_ZERO_ERROR;
+            ures_close(currentZone);
+            continue;
+        }
+        while ( ures_hasNext(zoneAliases)) {
+            int32_t len;
+            const UChar* alias = ures_getNextString(zoneAliases,&len,NULL,&status);
+            if ( zid.compare(alias)==0 ) {
+                zid.setTo(UnicodeString(currentZoneString, -1, US_INV));
+                zid.findAndReplace(colon,solidus);
+                ures_close(zoneAliases);
+                ures_close(currentZone);
+                ures_close(zoneFormatting);
+                ures_close(supplementalDataBundle);
+                return;
+            }     
+        }   
+        ures_close(zoneAliases);
+        ures_close(currentZone);
+    }
+    ures_close(zoneFormatting);
+    ures_close(supplementalDataBundle);
+}
 //----------------------------------------------------------------------
 
 void
@@ -1048,7 +1221,7 @@ SimpleDateFormat::parse(const UnicodeString& text, Calendar& cal, ParsePosition&
                 // Advance over run in input text
                 int32_t s = pos;
                 while (pos<text.length() &&
-                       u_isUWhiteSpace(text.charAt(pos))) {
+                       ( u_isUWhiteSpace(text.charAt(pos)) || uprv_isRuleWhiteSpace(text.charAt(pos)))) {
                     ++pos;
                 }
 
@@ -1766,34 +1939,15 @@ SimpleDateFormat::subParseZoneString(const UnicodeString& text, int32_t start, C
   TimeZone *tz = NULL;
   UnicodeString id;  
   UnicodeString zid, value;
-  DateFormatSymbols::TimeZoneTranslationType type;
+  DateFormatSymbols::TimeZoneTranslationType type = DateFormatSymbols::TIMEZONE_COUNT;
   fSymbols->getZoneID(getTimeZone().getID(id), zid, status);
   if(zid.length() > 0){
-      fSymbols->getZoneType(zid, text, start, type, value, status);
-      tz = getTimeZone().clone();
-  }
-  
-  // optimize for default time zone, assume different from caller
-  if (tz == NULL) {
-      TimeZone* defaultZone = TimeZone::createDefault(); 
-      fSymbols->getZoneID(defaultZone->getID(id), zid, status);
-      if(zid.length() > 0){
-          fSymbols->getZoneType(zid, text, start, type, value, status);
-          tz = defaultZone;
-      }
-      if (tz == NULL) {
-          delete defaultZone;
-      }
-  }
-
-  // still no luck, check all time zone strings
-  if(tz == NULL){
       fSymbols->findZoneIDTypeValue(zid, text, start, type, value, status);
-      if(zid.length() > 0){
+      if(type != DateFormatSymbols::TIMEZONE_COUNT) {
           tz = TimeZone::createTimeZone(zid);
       }
   }
-
+  
   if(U_FAILURE(status)){
       return 0;
   }
