@@ -1,6 +1,6 @@
 /*  
 ********************************************************************************
-*   Copyright (C) 1997-2005, International Business Machines
+*   Copyright (C) 1997-2007, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ********************************************************************************
 *
@@ -22,6 +22,7 @@
 
 #if !UCONFIG_NO_FORMATTING
 
+#include "unicode/calendar.h"
 #include "unicode/uobject.h"
 #include "unicode/locid.h"
 #include "unicode/ures.h"
@@ -35,6 +36,7 @@ U_NAMESPACE_BEGIN
 
 /* forward declaration */
 class SimpleDateFormat;
+class Hashtable;
 
 /**
  * DateFormatSymbols is a public class for encapsulating localizable date-time
@@ -197,10 +199,18 @@ public:
      *
      * @param count    Filled in with length of the array.
      * @return         the era name strings.
-     * @draft ICU 3.4
+     * @stable ICU 3.4
      */
     const UnicodeString* getEraNames(int32_t& count) const;
 
+    /**
+     * Sets era name strings. For example: "Anno Domini" and "Before Christ".
+     * @param eraNames  Array of era name strings (DateFormatSymbols retains ownership.)
+     * @param count Filled in with length of the array.
+     * @stable ICU 3.6
+     */
+    void setEraNames(const UnicodeString* eraNames, int32_t count);
+
     /**
      * Gets month strings. For example: "January", "February", etc.
      * @param count Filled in with length of the array.
@@ -235,47 +245,48 @@ public:
      */
     void setShortMonths(const UnicodeString* shortMonths, int32_t count);
 
+    /**
+     * Selector for date formatting context
+     * @stable ICU 3.6
+     */
     enum DtContextType {
-        /**
-         * Selector for format date context
-         * @draft ICU 3.4
-         */
-         FORMAT = 0,
-       /**
-         * Selector for standalone date context
-         * @draft ICU 3.4
-         */
-         STANDALONE = 1
+         FORMAT,
+         STANDALONE,
+         DT_CONTEXT_COUNT
     };
 
+    /**
+     * Selector for date formatting width
+     * @stable ICU 3.6
+     */
     enum DtWidthType {
-        /**
-         * Selector for wide strings
-         * @draft ICU 3.4
-         */
-         WIDE = 4,
-       /**
-         * Selector for abbreviated strings
-         * @draft ICU 3.4
-         */
-         ABBREVIATED = 3,
-       /**
-         * Selector for narrow strings
-         * @draft ICU 3.4
-         */
-         NARROW = 5
+         ABBREVIATED,
+         WIDE,
+         NARROW,
+         DT_WIDTH_COUNT
     };
 
     /**
      * Gets month strings by width and context. For example: "January", "February", etc.
      * @param count Filled in with length of the array.
-     * @param context The day formatting context, either FORMAT or STANDALONE
+     * @param context The formatting context, either FORMAT or STANDALONE
      * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
      * @return the month strings. (DateFormatSymbols retains ownership.)
-     * @draft ICU 3.4
+     * @stable ICU 3.4
      */
     const UnicodeString* getMonths(int32_t& count, DtContextType context, DtWidthType width) const;
 
+    /**
+     * Sets month strings by width and context. For example: "January", "February", etc.
+     *
+     * @param months  The new month strings. (not adopted; caller retains ownership)
+     * @param count   Filled in with length of the array.
+     * @param context The formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
+     * @stable ICU 3.6
+     */
+    void setMonths(const UnicodeString* months, int32_t count, DtContextType context, DtWidthType width);
+
     /**
      * Gets weekday strings. For example: "Sunday", "Monday", etc.
      * @param count        Filled in with length of the array.
@@ -312,13 +323,46 @@ public:
     /**
      * Gets weekday strings by width and context. For example: "Sunday", "Monday", etc.
      * @param count   Filled in with length of the array.
-     * @param context The day formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
+     * @param context The formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW
      * @return the month strings. (DateFormatSymbols retains ownership.)
-     * @draft ICU 3.4
+     * @stable ICU 3.4
      */
     const UnicodeString* getWeekdays(int32_t& count, DtContextType context, DtWidthType width) const;
 
+    /**
+     * Sets weekday strings by width and context. For example: "Sunday", "Monday", etc.
+     * @param weekdays  The new weekday strings. (not adopted; caller retains ownership)
+     * @param count     Filled in with length of the array.
+     * @param context   The formatting context, either FORMAT or STANDALONE
+     * @param width     The width of returned strings, either WIDE, ABBREVIATED, or NARROW
+     * @stable ICU 3.6
+     */
+    void setWeekdays(const UnicodeString* weekdays, int32_t count, DtContextType context, DtWidthType width);
+
+    /**
+     * Gets quarter strings by width and context. For example: "1st Quarter", "2nd Quarter", etc.
+     * @param count Filled in with length of the array.
+     * @param context The formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE or ABBREVIATED. There
+     *                are no NARROW quarters.
+     * @return the quarter strings. (DateFormatSymbols retains ownership.)
+     * @stable ICU 3.6
+     */
+    const UnicodeString* getQuarters(int32_t& count, DtContextType context, DtWidthType width) const;
+
+    /**
+     * Sets quarter strings by width and context. For example: "1st Quarter", "2nd Quarter", etc.
+     *
+     * @param quarters  The new quarter strings. (not adopted; caller retains ownership)
+     * @param count   Filled in with length of the array.
+     * @param context The formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE or ABBREVIATED. There
+     *                are no NARROW quarters.
+     * @stable ICU 3.6
+     */
+    void setQuarters(const UnicodeString* quarters, int32_t count, DtContextType context, DtWidthType width);
+
     /**
      * Gets AM/PM strings. For example: "AM" and "PM".
      * @param count        Filled in with length of the array.
@@ -340,7 +384,7 @@ public:
      * @param rowCount      Output param to receive number of rows.
      * @param columnCount   Output param to receive number of columns.
      * @return              The timezone strings as a 2-d array. (DateFormatSymbols retains ownership.)
-     * @stable ICU 2.0
+     * @deprecated ICU 3.6
      */
     const UnicodeString** getZoneStrings(int32_t& rowCount, int32_t& columnCount) const;
 
@@ -362,6 +406,10 @@ public:
 
     /**
      * Gets localized date-time pattern characters. For example: 'u', 't', etc.
+     * <p>
+     * Note: ICU no longer provides localized date-time pattern characters for a locale
+     * starting ICU 3.8.  This method returns the non-localized date-time pattern
+     * characters unless user defined localized data is set by setLocalPatternChars.
      * @param result    Output param which will receive the localized date-time pattern characters.
      * @return          A reference to 'result'.
      * @stable ICU 2.0
@@ -379,7 +427,7 @@ public:
     /**
      * Returns the locale for this object. Two flavors are available:
      * valid and actual locale.
-     * @draft ICU 2.8 likely to change after ICU 3.0, based on feedback
+     * @stable ICU 2.8
      */
     Locale getLocale(ULocDataLocaleType type, UErrorCode& status) const;
 
@@ -397,6 +445,100 @@ public:
      */
     static UClassID U_EXPORT2 getStaticClassID();
 
+    /**
+     * The translation type of the translated zone strings
+     * @internal ICU 3.6
+     */
+    enum TimeZoneTranslationType {
+        TIMEZONE_SHORT_GENERIC,
+        TIMEZONE_SHORT_STANDARD,
+        TIMEZONE_SHORT_DAYLIGHT,
+        TIMEZONE_LONG_GENERIC,
+        TIMEZONE_LONG_STANDARD,
+        TIMEZONE_LONG_DAYLIGHT,
+        TIMEZONE_EXEMPLAR_CITY,
+        TIMEZONE_COUNT
+    };
+    
+    /**
+     * Creates an enumeration of time zone IDs. The object is owned by the caller and should delete it after use.
+     * The time zone IDs are just for programmatic lookup. NOT LOCALIZED!!!
+     * @param status   Input/output parameter, set to success or
+     *                 failure code upon return.
+     * @return A new StringEnumeration object
+     * @internal ICU 3.6
+     */
+    virtual StringEnumeration* createZoneStringIDs(UErrorCode &status);
+
+    /**
+     * Gets timezone string give the key and translation type
+     * @param ID       The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is  for programmatic lookup.
+     * @param type     The translation type requested
+     * @param result   Output parameter to recieve the translation string
+     * @param status   Input/output parameter, set to success or
+     *                 failure code upon return.
+     * @return the input UnicodeString parameter for chaining
+     * @internal ICU 3.8
+     */
+    UnicodeString& getZoneString(const UnicodeString &ID, const TimeZoneTranslationType type, UnicodeString &result, UErrorCode &status);
+
+    /**
+     * Gets metazone string given the key and translation type and calendar
+     * @param ID       The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is  for programmatic lookup.
+     * @param type     The translation type requested
+     * @param cal      The calendar
+     * @param result   Output parameter to recieve the translation string
+     * @param status   Input/output parameter, set to success or
+     *                 failure code upon return.
+     * @return the input UnicodeString parameter for chaining
+     * @internal ICU 3.8
+     */
+    UnicodeString getMetazoneString(const UnicodeString &ID, const TimeZoneTranslationType type, Calendar &cal, UnicodeString &result, UErrorCode &status);
+
+    /**
+     * Gets fallback string given the key
+     * @param ID       The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is  for programmatic lookup.
+     * @param result   Output parameter to recieve the translation string
+     * @param status   Input/output parameter, set to success or
+     *                 failure code upon return.
+     * @return the input UnicodeString parameter for chaining
+     * @internal ICU 3.8
+     */
+    UnicodeString& getFallbackString(const UnicodeString &ID, UnicodeString &result, UErrorCode &status);
+
+    /**
+     * Sets timezone string for the given the ID and translation type
+     * @param ID       The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is for programmatic lookup.
+     * @param type     The translation type to set the value for
+     * @param value    The string with which current translation needs to be replaced
+     * @param status   Input/output parameter, set to success or     
+     * @internal ICU 3.6
+     */
+
+    /**
+     * Determines if the Commonly Used flag is set for this zone 
+     * @param zid      The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is  for programmatic lookup.
+     * @return         A boolean value indicating if the zone is commonlyUsed or not.
+     * @internal ICU 3.8
+     */
+    UBool isCommonlyUsed(const UnicodeString &zid);
+    
+    /**
+     * Sets timezone string for the given the ID and translation type
+     * @param ID       The ID of zone strings,  e.g: "America/Los_Angeles".
+     *                 The time zone ID is for programmatic lookup.
+     * @param type     The translation type to set the value for
+     * @param value    The string with which current translation needs to be replaced
+     * @param status   Input/output parameter, set to success or     
+     * @internal ICU 3.6
+     */
+    void setZoneString(const UnicodeString &ID, const TimeZoneTranslationType type, const UnicodeString &value, UErrorCode &status);
+
 private:
 
     friend class SimpleDateFormat;
@@ -492,12 +634,40 @@ private:
     UnicodeString*  fAmPms;
     int32_t         fAmPmsCount;
 
+    /**
+     * Quarter strings. For example: "1st quarter", "2nd quarter", etc.
+     */
+    UnicodeString  *fQuarters;
+    int32_t         fQuartersCount;
+
+    /**
+     * Short quarters. For example: "Q1", "Q2", etc.
+     */
+    UnicodeString  *fShortQuarters;
+    int32_t         fShortQuartersCount;
+
+    /**
+     * Standalone quarter strings. For example: "1st quarter", "2nd quarter", etc.
+     */
+    UnicodeString  *fStandaloneQuarters;
+    int32_t         fStandaloneQuartersCount;
+
+    /**
+     * Standalone short quarter strings. For example: "Q1", "Q2", etc.
+     */
+    UnicodeString  *fStandaloneShortQuarters;
+    int32_t         fStandaloneShortQuartersCount;
+
     /**
      * The format data of all the timezones in this locale.
      */
     UnicodeString** fZoneStrings;
     int32_t         fZoneStringsRowCount;
     int32_t         fZoneStringsColCount;
+    StringEnumeration* fZoneIDEnumeration;
+    Hashtable*         fZoneStringsHash;
+    UResourceBundle* fResourceBundle;
+    const char*      fCountry;
 
     /**
      * Localized date-time pattern characters. For example: use 'u' as 'y'.
@@ -588,6 +758,63 @@ private:
      * Delete just the zone strings.
      */
     void disposeZoneStrings(void);
+
+    /**
+     * Initializes the zoneStrings hash and keys StringEnumeration after reading the zoneStrings resource
+     */
+    void initZoneStrings(UErrorCode &status);
+    /** 
+     * initialzes the zoneStrings has and keys enumeration after reading the strings[][]. Required for backwards
+     * compatibility of setZoneStrings method
+     */
+    void initZoneStrings(const UnicodeString** strings, int32_t rowCount, int32_t collumnCount, UErrorCode& status);
+    /**
+     * initialization of the fZoneStrings data member
+     */
+    void initZoneStringsArray(UErrorCode& status);
+    /**
+     * Creates a deep clone of the Hashtable
+     */
+    Hashtable* createZoneStringsHash(const Hashtable* otherHash);
+    
+    /**
+     * Fetches the key from the hashtable for a given ID.
+     * e.g: for a given ID such as PST returns "Americal/Los_Angeles"
+     * Used by SimpleDateFormat class.
+     * @param ID The id of the time zone for which the key needs to be fetched
+     * @param result Output parameter to recieve the key.
+     * @return the input UnicodeString object for chaining
+     */
+    UnicodeString& getZoneID(const UnicodeString& zid, UnicodeString& result, UErrorCode& status);
+    
+    /**
+     * Fetches the zone type and zone string from the hashtable for a given key.
+     * e.g: for key: "Americal/Los_Angeles", text: "2004/1/1 PT 1:00" and start:9
+     * returns TIMEZONE_SHORT_GENERIC and "PT". 
+     * Used by SimpleDateFormat class.
+     * @param ID the name of the timezone
+     * @param text the string containing the time zone translation
+     * @param start The position in string where time zone string starts
+     * @param type  output parameter to recieve the type of time zone string
+     * @param value output parameter to recieve the the acutal time zone string
+     */
+    void getZoneType(const UnicodeString& zid, const UnicodeString& text, int32_t start, TimeZoneTranslationType& type, UnicodeString& value, UErrorCode& status);
+    
+    /**
+     * Fetches the zone type and zone string from the hashtable by cycling through all elements in the hashtable.
+     * e.g: text: "2004/1/1 PT 1:00" and start:9
+     * returns "Americal/Los_Angeles", TIMEZONE_SHORT_GENERIC and "PT". Used by SimpleDateFormat class.
+     * Used by SimpleDateFormat class.
+     * @param ID output parameter to recieve the key name of the time zone
+     * @param text the string containing the time zone translation
+     * @param start The position in string where time zone string starts
+     * @param type  output parameter to recieve the type of time zone string
+     * @param value output parameter to recieve the the acutal time zone string
+     * @param status output parameter to recive the error information
+     */
+    void findZoneIDTypeValue(UnicodeString& zid, const UnicodeString& text, int32_t start, TimeZoneTranslationType& type, UnicodeString& value, UErrorCode& status);
+
+    UnicodeString resolveParsedMetazone(const UnicodeString& zid);
 };
 
 U_NAMESPACE_END
