@@ -649,6 +649,13 @@ private:
                                 Calendar& cal,
                                 UErrorCode& status) const; // in case of illegal argument
 
+    /**
+     * Used to resolve Time Zone aliases
+     *
+     * @param zid       Time Zone ID to Canonicalize ( resolve aliases )
+     */
+    void zoneIDCanonicalize( UnicodeString & ) const;
+
     /**
      * Used by subFormat() to format a numeric value.
      * Appends to toAppendTo a string representation of "value"
