@@ -1,6 +1,7 @@
-/*
-* Copyright (C) {1997-2001}, International Business Machines Corporation and others. All Rights Reserved.
-********************************************************************************
+/*************************************************************************
+* Copyright (c) 1997-2007, International Business Machines Corporation
+* and others. All Rights Reserved.
+**************************************************************************
 *
 * File TIMEZONE.H
 *
@@ -20,17 +21,31 @@
 *                           Hashtable replaced by new static data structures.
 *   12/14/99    aliu        Made GMT public.
 *   08/15/01    grhoten     Made GMT private and added the getGMT() function
-********************************************************************************
+**************************************************************************
 */
 
 #ifndef TIMEZONE_H
 #define TIMEZONE_H
 
+#include "unicode/utypes.h"
+
+/**
+ * \file 
+ * \brief C++ API: TimeZone object
+ */
+
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/uobject.h"
 #include "unicode/unistr.h"
+#include "unicode/ures.h"
 
 U_NAMESPACE_BEGIN
 
+class StringEnumeration;
+
 /**
+ *
  * <code>TimeZone</code> represents a time zone offset, and also figures out daylight
  * savings.
  *
@@ -45,11 +60,11 @@ U_NAMESPACE_BEGIN
  * with a time zone ID. For instance, the time zone ID for the Pacific
  * Standard Time zone is "PST". So, you can get a PST <code>TimeZone</code> object
  * with:
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  * TimeZone *tz = TimeZone::createTimeZone("PST");
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  * You can use <code>getAvailableIDs</code> method to iterate through
  * all the supported time zone IDs. You can then choose a
  * supported ID to get a <code>TimeZone</code>.
@@ -57,11 +72,11 @@ U_NAMESPACE_BEGIN
  * supported IDs, then you can create a custom time zone ID with
  * the following syntax:
  *
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  * GMT[+|-]hh[[:]mm]
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  *
  * For example, you might specify GMT+14:00 as a custom
  * time zone ID.  The <code>TimeZone</code> that is returned
@@ -70,14 +85,16 @@ U_NAMESPACE_BEGIN
  *
  * TimeZone is an abstract class representing a time zone.  A TimeZone is needed for
  * Calendar to produce local time for a particular time zone.  A TimeZone comprises
- * three basic pieces of information:<ul>
+ * three basic pieces of information:
+ * <ul>
  *    <li>A time zone offset; that, is the number of milliseconds to add or subtract
  *      from a time expressed in terms of GMT to convert it to the same time in that
- *      time zone (without taking daylight savings time into account).
+ *      time zone (without taking daylight savings time into account).</li>
  *    <li>Logic necessary to take daylight savings time into account if daylight savings
  *      time is observed in that time zone (e.g., the days and hours on which daylight
- *      savings time begins and ends).
- *    <li>An ID.  This is a text string that uniquely identifies the time zone.</ul>
+ *      savings time begins and ends).</li>
+ *    <li>An ID.  This is a text string that uniquely identifies the time zone.</li>
+ * </ul>
  *
  * (Only the ID is actually implemented in TimeZone; subclasses of TimeZone may handle
  * daylight savings time and GMT offset in different ways.  Currently we only have one
@@ -93,7 +110,7 @@ U_NAMESPACE_BEGIN
  * Anywhere the API requires an ID, you can use either form.
  * <P>
  * To create a new TimeZone, you call the factory function TimeZone::createTimeZone()
- * and pass it a time zone ID.  You can use the createAvailableIDs() function to
+ * and pass it a time zone ID.  You can use the createEnumeration() function to
  * obtain a list of all the time zone IDs recognized by createTimeZone().
  * <P>
  * You can also use TimeZone::createDefault() to create a TimeZone.  This function uses
@@ -102,28 +119,20 @@ U_NAMESPACE_BEGIN
  * your machine is set up correctly), TimeZone::createDefault() will return a TimeZone
  * for Japanese Standard Time ("Asia/Tokyo").
  */
-class U_I18N_API TimeZone {
+class U_I18N_API TimeZone : public UObject {
 public:
     /**
-     * @stable
+     * @stable ICU 2.0
      */
     virtual ~TimeZone();
 
-#ifdef ICU_TIMEZONE_USE_DEPRECATES
-    /**
-     * The GMT zone has a raw offset of zero and does not use daylight
-     * savings time.
-     * @deprecated This variable can be improperly intialized when used during
-     * the static initialization process.  Use getGMT() instead.
-     */
-    static const TimeZone* GMT;
-#endif
-
     /**
      * The GMT time zone has a raw offset of zero and does not use daylight
      * savings time. This is a commonly used time zone.
+     * @return the GMT time zone.
+     * @stable ICU 2.0
      */
-    static const TimeZone* getGMT(void);
+    static const TimeZone* U_EXPORT2 getGMT(void);
 
     /**
      * Creates a <code>TimeZone</code> for the given ID.
@@ -134,10 +143,51 @@ public:
      * cannot be understood.  Return result guaranteed to be non-null.  If you
      * require that the specific zone asked for be returned, check the ID of the
      * return result.
-     * @stable
+     * @stable ICU 2.0
      */
-    static TimeZone* createTimeZone(const UnicodeString& ID);
+    static TimeZone* U_EXPORT2 createTimeZone(const UnicodeString& ID);
 
+    /**
+     * Returns an enumeration over all recognized time zone IDs. (i.e.,
+     * all strings that createTimeZone() accepts)
+     *
+     * @return an enumeration object, owned by the caller.
+     * @stable ICU 2.4
+     */
+    static StringEnumeration* U_EXPORT2 createEnumeration();
+
+    /**
+     * Returns an enumeration over time zone IDs with a given raw
+     * offset from GMT.  There may be several times zones with the
+     * same GMT offset that differ in the way they handle daylight
+     * savings time.  For example, the state of Arizona doesn't
+     * observe daylight savings time.  If you ask for the time zone
+     * IDs corresponding to GMT-7:00, you'll get back an enumeration
+     * over two time zone IDs: "America/Denver," which corresponds to
+     * Mountain Standard Time in the winter and Mountain Daylight Time
+     * in the summer, and "America/Phoenix", which corresponds to
+     * Mountain Standard Time year-round, even in the summer.
+     *
+     * @param rawOffset an offset from GMT in milliseconds, ignoring
+     * the effect of daylight savings time, if any
+     * @return an enumeration object, owned by the caller
+     * @stable ICU 2.4
+     */
+    static StringEnumeration* U_EXPORT2 createEnumeration(int32_t rawOffset);
+
+    /**
+     * Returns an enumeration over time zone IDs associated with the
+     * given country.  Some zones are affiliated with no country
+     * (e.g., "UTC"); these may also be retrieved, as a group.
+     *
+     * @param country The ISO 3166 two-letter country code, or NULL to
+     * retrieve zones not affiliated with any country.
+     * @return an enumeration object, owned by the caller
+     * @stable ICU 2.4
+     */
+    static StringEnumeration* U_EXPORT2 createEnumeration(const char* country);
+
+#ifdef U_USE_TIMEZONE_OBSOLETE_2_8
     /**
      * Returns a list of time zone IDs, one for each time zone with a given GMT offset.
      * The return value is a list because there may be several times zones with the same
@@ -149,7 +199,9 @@ public:
      * Mountain Standard Time year-round, even in the summer.
      * <P>
      * The caller owns the list that is returned, but does not own the strings contained
-     * in that list.  Delete the array, but DON'T delete the elements in the array.
+     * in that list.  Delete the array with uprv_free(), but DON'T delete the elements in the array.
+     *
+     * <p>NOTE: uprv_free() is declared in the private header source/common/cmemory.h.
      *
      * @param rawOffset  An offset from GMT in milliseconds.
      * @param numIDs     Receives the number of items in the array that is returned.
@@ -157,7 +209,7 @@ public:
      *                   a time zone ID for a time zone with the given GMT offset.  If
      *                   there is no timezone that matches the GMT offset
      *                   specified, NULL is returned.
-     * @stable
+     * @obsolete ICU 2.8.  Use createEnumeration(int32_t) instead since this API will be removed in that release.
      */
     static const UnicodeString** createAvailableIDs(int32_t rawOffset, int32_t& numIDs);
 
@@ -167,9 +219,11 @@ public:
      * "UTC"); these may also be retrieved, as a group.
      *
      * <P>The caller owns the list that is returned, but does not own
-     * the strings contained in that list.  Delete the array, but
+     * the strings contained in that list.  Delete the array with uprv_free(), but
      * <b>DON'T</b> delete the elements in the array.
      *
+     * <p>NOTE: uprv_free() is declared in the private header source/common/cmemory.h.
+     *
      * @param country The ISO 3166 two-letter country code, or NULL to
      * retrieve zones not affiliated with any country.
      * @param numIDs Receives the number of items in the array that is
@@ -178,6 +232,7 @@ public:
      * UnicodeString is a time zone ID for a time zone with the given
      * country.  If there is no timezone that matches the country
      * specified, NULL is returned.
+     * @obsolete ICU 2.8.  Use createEnumeration(const char*) instead since this API will be removed in that release.
      */
     static const UnicodeString** createAvailableIDs(const char* country,
                                                           int32_t& numIDs);
@@ -185,15 +240,18 @@ public:
     /**
      * Returns a list of all time zone IDs supported by the TimeZone class (i.e., all
      * IDs that it's legal to pass to createTimeZone()).  The caller owns the list that
-     * is returned, but does not own the strings contained in that list.  Delete the array,
+     * is returned, but does not own the strings contained in that list.  Delete the array with uprv_free(),
      * but DON'T delete the elements in the array.
      *
+     * <p>NOTE: uprv_free() is declared in the private header source/common/cmemory.h.
+     *
      * @param numIDs  Receives the number of zone IDs returned.
      * @return        An array of UnicodeString pointers, where each is a time zone ID
      *                supported by the TimeZone class.
-     * @stable
+     * @obsolete ICU 2.8.  Use createEnumeration(void) instead since this API will be removed in that release.
      */
     static const UnicodeString** createAvailableIDs(int32_t& numIDs);
+#endif
 
     /**
      * Returns the number of IDs in the equivalency group that
@@ -207,9 +265,9 @@ public:
      * @return the number of zones in the equivalency group containing
      * 'id', or zero if 'id' is not a valid system ID
      * @see #getEquivalentID
-     * @draft ICU 1.7
+     * @stable ICU 2.0
      */
-    static int32_t countEquivalentIDs(const UnicodeString& id);
+    static int32_t U_EXPORT2 countEquivalentIDs(const UnicodeString& id);
 
     /**
      * Returns an ID in the equivalency group that
@@ -228,9 +286,9 @@ public:
      * containing 'id', or an empty string if 'id' is not a valid
      * system ID or 'index' is out of range
      * @see #countEquivalentIDs
-     * @draft ICU 1.7
+     * @stable ICU 2.0
      */
-    static const UnicodeString getEquivalentID(const UnicodeString& id,
+    static const UnicodeString U_EXPORT2 getEquivalentID(const UnicodeString& id,
                                                int32_t index);
 
     /**
@@ -243,20 +301,20 @@ public:
      *
      * @return   A default TimeZone. Clients are responsible for deleting the time zone
      *           object returned.
-     * @stable
+     * @stable ICU 2.0
      */
-    static TimeZone* createDefault(void);
+    static TimeZone* U_EXPORT2 createDefault(void);
 
     /**
-     * Sets the default time zone (i.e., what's returned by getDefault()) to be the
+     * Sets the default time zone (i.e., what's returned by createDefault()) to be the
      * specified time zone.  If NULL is specified for the time zone, the default time
      * zone is set to the default host time zone.  This call adopts the TimeZone object
      * passed in; the clent is no longer responsible for deleting it.
      *
      * @param zone  A pointer to the new TimeZone object to use as the default.
-     * @stable
+     * @stable ICU 2.0
      */
-    static void adoptDefault(TimeZone* zone);
+    static void U_EXPORT2 adoptDefault(TimeZone* zone);
 
     /**
      * Same as adoptDefault(), except that the TimeZone object passed in is NOT adopted;
@@ -265,7 +323,15 @@ public:
      * @param zone  The given timezone.
      * @system
      */
-    static void setDefault(const TimeZone& zone);
+    static void U_EXPORT2 setDefault(const TimeZone& zone);
+
+    /**
+     * Returns the timezone data version currently used by ICU.
+     * @param status Output param to filled in with a success or an error.
+     * @return the version string, such as "2007f"
+     * @draft ICU 3.8
+     */
+    static const char* U_EXPORT2 getTZDataVersion(UErrorCode& status);
 
     /**
      * Returns true if the two TimeZones are equal.  (The TimeZone version only compares
@@ -274,7 +340,7 @@ public:
      * @param that  The TimeZone object to be compared with.
      * @return      True if the given TimeZone is equal to this TimeZone; false
      *              otherwise.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const TimeZone& that) const;
 
@@ -285,7 +351,7 @@ public:
      * @param that  The TimeZone object to be compared with.
      * @return      True if the given TimeZone is not equal to this TimeZone; false
      *              otherwise.
-     * @stable
+     * @stable ICU 2.0
      */
     UBool operator!=(const TimeZone& that) const {return !operator==(that);}
 
@@ -297,28 +363,33 @@ public:
      * that is returned (in other words, what is the adjusted GMT offset in this time zone
      * at this particular date and time?).  For the time zones produced by createTimeZone(),
      * the reference data is specified according to the Gregorian calendar, and the date
-     * and time fields are in GMT, NOT local time.
+     * and time fields are local standard time.
+     *
+     * <p>Note: Don't call this method. Instead, call the getOffset(UDate...) overload,
+     * which returns both the raw and the DST offset for a given time. This method
+     * is retained only for backward compatibility.
      *
      * @param era        The reference date's era
      * @param year       The reference date's year
      * @param month      The reference date's month (0-based; 0 is January)
      * @param day        The reference date's day-in-month (1-based)
      * @param dayOfWeek  The reference date's day-of-week (1-based; 1 is Sunday)
-     * @param millis     The reference date's milliseconds in day, UTT (NOT local time).
+     * @param millis     The reference date's milliseconds in day, local standard time
+     * @param status     Output param to filled in with a success or an error.
      * @return           The offset in milliseconds to add to GMT to get local time.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                               uint8_t dayOfWeek, int32_t millis, UErrorCode& status) const = 0;
-    /**
-     * @deprecated Remove after 2000-dec-31. Use the other getOffset().
-     */
-    virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
-                              uint8_t dayOfWeek, int32_t millis) const = 0;
 
     /**
      * Gets the time zone offset, for current date, modified in case of
      * daylight savings. This is the offset to add *to* UTC to get local time.
+     *
+     * <p>Note: Don't call this method. Instead, call the getOffset(UDate...) overload,
+     * which returns both the raw and the DST offset for a given time. This method
+     * is retained only for backward compatibility.
+     *
      * @param era the era of the given date.
      * @param year the year in the given date.
      * @param month the month in the given date.
@@ -327,19 +398,46 @@ public:
      * @param dayOfWeek the day-of-week of the given date.
      * @param milliseconds the millis in day in <em>standard</em> local time.
      * @param monthLength the length of the given month in days.
+     * @param status     Output param to filled in with a success or an error.
      * @return the offset to add *to* GMT to get local time.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                            uint8_t dayOfWeek, int32_t milliseconds,
                            int32_t monthLength, UErrorCode& status) const = 0;
 
+    /**
+     * Returns the time zone raw and GMT offset for the given moment
+     * in time.  Upon return, local-millis = GMT-millis + rawOffset +
+     * dstOffset.  All computations are performed in the proleptic
+     * Gregorian calendar.  The default implementation in the TimeZone
+     * class delegates to the 8-argument getOffset().
+     *
+     * @param date moment in time for which to return offsets, in
+     * units of milliseconds from January 1, 1970 0:00 GMT, either GMT
+     * time or local wall time, depending on `local'.
+     * @param local if true, `date' is local wall time; otherwise it
+     * is in GMT time.
+     * @param rawOffset output parameter to receive the raw offset, that
+     * is, the offset not including DST adjustments
+     * @param dstOffset output parameter to receive the DST offset,
+     * that is, the offset to be added to `rawOffset' to obtain the
+     * total offset between local and GMT time. If DST is not in
+     * effect, this value is zero; otherwise it is a positive value,
+     * typically one hour.
+     * @param ec input-output error code
+     *
+     * @stable ICU 2.8
+     */
+    virtual void getOffset(UDate date, UBool local, int32_t& rawOffset,
+                           int32_t& dstOffset, UErrorCode& ec) const;
+
     /**
      * Sets the TimeZone's raw GMT offset (i.e., the number of milliseconds to add
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @param offsetMillis  The new raw GMT offset for this time zone.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void setRawOffset(int32_t offsetMillis) = 0;
 
@@ -348,7 +446,7 @@ public:
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @return   The TimeZone's raw GMT offset.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t getRawOffset(void) const = 0;
 
@@ -356,8 +454,8 @@ public:
      * Fills in "ID" with the TimeZone's ID.
      *
      * @param ID  Receives this TimeZone's ID.
-     * @return    "ID"
-     * @stable
+     * @return    A reference to 'ID'
+     * @stable ICU 2.0
      */
     UnicodeString& getID(UnicodeString& ID) const;
 
@@ -367,21 +465,29 @@ public:
      * blockquote><pre>
      * .     TimeZone* foo = TimeZone::createTimeZone("America/New_York");
      * .     foo.setID("America/Los_Angeles");
-     * </pre></blockquote>
+     * </pre>\htmlonly</blockquote>\endhtmlonly
      * the time zone's GMT offset and daylight-savings rules don't change to those for
      * Los Angeles.  They're still those for New York.  Only the ID has changed.)
      *
      * @param ID  The new timezone ID.
-     * @stable
+     * @stable ICU 2.0
      */
     void setID(const UnicodeString& ID);
 
     /**
      * Enum for use with getDisplayName
+     * @stable ICU 2.4
      */
-
     enum EDisplayType {
+        /**
+         * Selector for short display name
+         * @stable ICU 2.4
+         */
         SHORT = 1,
+        /**
+         * Selector for long display name
+         * @stable ICU 2.4
+         */
         LONG
     };
 
@@ -392,8 +498,9 @@ public:
      * If the display name is not available for the locale,
      * then this method returns a string in the format
      * <code>GMT[+-]hh:mm</code>.
-     * @return the human-readable name of this time zone in the default locale.
-     * @stable
+     * @param result the human-readable name of this time zone in the default locale.
+     * @return       A reference to 'result'.
+     * @stable ICU 2.0
      */
     UnicodeString& getDisplayName(UnicodeString& result) const;
 
@@ -405,9 +512,10 @@ public:
      * then this method returns a string in the format
      * <code>GMT[+-]hh:mm</code>.
      * @param locale the locale in which to supply the display name.
-     * @return the human-readable name of this time zone in the given locale
-     * or in the default locale if the given locale is not recognized.
-     * @stable
+     * @param result the human-readable name of this time zone in the given locale
+     *               or in the default locale if the given locale is not recognized.
+     * @return       A reference to 'result'.
+     * @stable ICU 2.0
      */
     UnicodeString& getDisplayName(const Locale& locale, UnicodeString& result) const;
 
@@ -419,8 +527,9 @@ public:
      * <code>GMT[+-]hh:mm</code>.
      * @param daylight if true, return the daylight savings name.
      * @param style either <code>LONG</code> or <code>SHORT</code>
-     * @return the human-readable name of this time zone in the default locale.
-     * @stable
+     * @param result the human-readable name of this time zone in the default locale.
+     * @return       A reference to 'result'.
+     * @stable ICU 2.0
      */
     UnicodeString& getDisplayName(UBool daylight, EDisplayType style, UnicodeString& result) const;
 
@@ -433,9 +542,10 @@ public:
      * @param daylight if true, return the daylight savings name.
      * @param style either <code>LONG</code> or <code>SHORT</code>
      * @param locale the locale in which to supply the display name.
-     * @return the human-readable name of this time zone in the given locale
-     * or in the default locale if the given locale is not recognized.
-     * @stable
+     * @param result the human-readable name of this time zone in the given locale
+     *               or in the default locale if the given locale is not recognized.
+     * @return       A refence to 'result'.
+     * @stable ICU 2.0
      */
     UnicodeString& getDisplayName(UBool daylight, EDisplayType style, const Locale& locale, UnicodeString& result) const;
 
@@ -443,17 +553,22 @@ public:
      * Queries if this time zone uses daylight savings time.
      * @return true if this time zone uses daylight savings time,
      * false, otherwise.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool useDaylightTime(void) const = 0;
 
     /**
      * Queries if the given date is in daylight savings time in
      * this time zone.
+     * This method is wasteful since it creates a new GregorianCalendar and
+     * deletes it each time it is called. This is a deprecated method
+     * and provided only for Java compatibility.
+     *
      * @param date the given UDate.
+     * @param status Output param filled in with success/error code.
      * @return true if the given date is in daylight savings time,
      * false, otherwise.
-     * @deprecated Remove after 2000-dec-31. Use useDaylightTime() instead.
+     * @deprecated ICU 2.4. Use Calendar::inDaylightTime() instead.
      */
     virtual UBool inDaylightTime(UDate date, UErrorCode& status) const = 0;
 
@@ -463,7 +578,7 @@ public:
      * @param other the <code>TimeZone</code> object to be compared with
      * @return true if the given zone is the same as this one,
      * with the possible exception of the ID
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool hasSameRules(const TimeZone& other) const;
 
@@ -472,69 +587,89 @@ public:
      * the TimeZone object cloned.
      *
      * @return   A new copy of this TimeZone object.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual TimeZone* clone(void) const = 0;
 
     /**
      * Return the class ID for this class.  This is useful only for
-     * comparing to a return value from getDynamicClassID().  For example:
-     * <pre>
-     * .   Base* polymorphic_pointer = createPolymorphicObject();
-     * .   if (polymorphic_pointer->getDynamicClassID() ==
-     * .       Derived::getStaticClassID()) ...
-     * </pre>
+     * comparing to a return value from getDynamicClassID().
      * @return The class ID for all objects of this class.
-     * @stable
+     * @stable ICU 2.0
      */
-    static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
+    static UClassID U_EXPORT2 getStaticClassID(void);
 
     /**
-     * Returns a unique class ID POLYMORPHICALLY. Pure virtual method. This method is to
+     * Returns a unique class ID POLYMORPHICALLY. This method is to
      * implement a simple version of RTTI, since not all C++ compilers support genuine
      * RTTI. Polymorphic operator==() and clone() methods call this method.
      * <P>
-     * Concrete subclasses of TimeZone must implement getDynamicClassID() and also a
-     * static method and data member:
-     * <pre>
-     * .     static UClassID getStaticClassID() { return (UClassID)&fgClassID; }
-     * .     static char fgClassID;
-     * </pre>
+     * Concrete subclasses of TimeZone must use the UOBJECT_DEFINE_RTTI_IMPLEMENTATION
+     *  macro from uobject.h in their implementation to provide correct RTTI information.
      * @return   The class ID for this object. All objects of a given class have the
      *           same class ID. Objects of other classes have different class IDs.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UClassID getDynamicClassID(void) const = 0;
+    
+    /**
+     * Returns the amount of time to be added to local standard time
+     * to get local wall clock time.
+     * <p>
+     * The default implementation always returns 3600000 milliseconds
+     * (i.e., one hour) if this time zone observes Daylight Saving
+     * Time. Otherwise, 0 (zero) is returned.
+     * <p>
+     * If an underlying TimeZone implementation subclass supports
+     * historical Daylight Saving Time changes, this method returns
+     * the known latest daylight saving value.
+     *
+     * @return the amount of saving time in milliseconds
+     * @stable ICU 3.6
+     */
+    virtual int32_t getDSTSavings() const;
 
 protected:
 
     /**
      * Default constructor.  ID is initialized to the empty string.
-     * @stable
+     * @stable ICU 2.0
      */
     TimeZone();
 
     /**
      * Construct a timezone with a given ID.
-     * @stable
+     * @param id a system time zone ID
+     * @stable ICU 2.0
      */
     TimeZone(const UnicodeString &id);
 
     /**
      * Copy constructor.
-     * @stable
+     * @param source the object to be copied.
+     * @stable ICU 2.0
      */
     TimeZone(const TimeZone& source);
 
     /**
      * Default assignment operator.
-     * @stable
+     * @param right the object to be copied.
+     * @stable ICU 2.0
      */
     TimeZone& operator=(const TimeZone& right);
 
-private:
-    static const char fgClassID;
+    /**
+     * Utility function. For internally loading rule data.
+     * @param top Top resource bundle for tz data
+     * @param ruleid ID of rule to load
+     * @param oldbundle Old bundle to reuse or NULL
+     * @param status Status parameter
+     * @return either a new bundle or *oldbundle
+     * @internal
+     */
+    static UResourceBundle* loadRule(const UResourceBundle* top, const UnicodeString& ruleid, UResourceBundle* oldbundle, UErrorCode&status);
 
+private:
     static TimeZone*        createCustomTimeZone(const UnicodeString&); // Creates a time zone based on the string.
 
     /**
@@ -545,6 +680,13 @@ private:
     static void             initDefault(void);
 
     // See source file for documentation
+    /**
+     * Lookup the given name in our system zone table.  If found,
+     * instantiate a new zone of that name and return it.  If not
+     * found, return 0.
+     * @param name tthe given name of a system time zone.
+     * @return the timezone indicated by the 'name'.
+     */
     static TimeZone*        createSystemTimeZone(const UnicodeString& name);
 
     UnicodeString           fID;    // this time zone's ID
@@ -569,5 +711,7 @@ TimeZone::setID(const UnicodeString& ID)
 }
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif //_TIMEZONE
 //eof
