@@ -1,16 +1,16 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2007, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
-/********************************************************************************
+/*******************************************************************************
 *
 * File CALLCOLL.C
 *
 * Modification History:
 *        Name                     Description            
 *     Madhu Katragadda              Ported for C API
-*********************************************************************************
+********************************************************************************
 */
 
 /*
@@ -42,6 +42,7 @@
 #include "unicode/ustring.h"
 #include "unicode/uclean.h"
 #include "unicode/putil.h"
+#include "unicode/uenum.h"
 
 #include "cintltst.h"
 #include "ccolltst.h"
@@ -84,6 +85,8 @@ static void TestJitterbug1098(void);
 
 static void TestFCDCrash(void);
 
+static void TestJ5298(void);
+
 const UCollationResult results[] = {
     UCOL_LESS,
     UCOL_LESS, /*UCOL_GREATER,*/
@@ -199,8 +202,8 @@ void addAllCollTest(TestNode** root)
     addTest(root, &TestJB1401, "tscoll/callcoll/TestJB1401");      
     addTest(root, &TestJitterbug1098, "tscoll/callcoll/TestJitterbug1098");     
     addTest(root, &TestFCDCrash, "tscoll/callcoll/TestFCDCrash");
-
-   }
+    addTest(root, &TestJ5298, "tscoll/callcoll/TestJ5298");
+}
 
 UBool hasCollationElements(const char *locName) {
 
@@ -540,7 +543,7 @@ backAndForth(UCollationElements *iter)
     free(orders);
 }
 
-void genericOrderingTestWithResult(UCollator *coll, const char *s[], uint32_t size, UCollationResult result) {
+void genericOrderingTestWithResult(UCollator *coll, const char * const s[], uint32_t size, UCollationResult result) {
   UChar t1[2048] = {0};
   UChar t2[2048] = {0};
   UCollationElements *iter;
@@ -571,11 +574,11 @@ void genericOrderingTestWithResult(UCollator *coll, const char *s[], uint32_t si
   ucol_closeElements(iter);
 }
 
-void genericOrderingTest(UCollator *coll, const char *s[], uint32_t size) {
+void genericOrderingTest(UCollator *coll, const char * const s[], uint32_t size) {
   genericOrderingTestWithResult(coll, s, size, UCOL_LESS);
 }
 
-void genericLocaleStarter(const char *locale, const char *s[], uint32_t size) {
+void genericLocaleStarter(const char *locale, const char * const s[], uint32_t size) {
   UErrorCode status = U_ZERO_ERROR;
   UCollator *coll = ucol_open(locale, &status);
 
@@ -592,7 +595,7 @@ void genericLocaleStarter(const char *locale, const char *s[], uint32_t size) {
   ucol_close(coll);
 }
 
-void genericLocaleStarterWithResult(const char *locale, const char *s[], uint32_t size, UCollationResult result) {
+void genericLocaleStarterWithResult(const char *locale, const char * const s[], uint32_t size, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   UCollator *coll = ucol_open(locale, &status);
 
@@ -610,7 +613,7 @@ void genericLocaleStarterWithResult(const char *locale, const char *s[], uint32_
 }
 
 /* currently not used with options */
-void genericRulesStarterWithOptionsAndResult(const char *rules, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+void genericRulesStarterWithOptionsAndResult(const char *rules, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   UChar rlz[RULE_BUFFER_LEN] = { 0 };
   uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
@@ -633,7 +636,7 @@ void genericRulesStarterWithOptionsAndResult(const char *rules, const char *s[],
   ucol_close(coll);
 }
 
-void genericLocaleStarterWithOptionsAndResult(const char *locale, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+void genericLocaleStarterWithOptionsAndResult(const char *locale, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   uint32_t i;
 
@@ -655,11 +658,11 @@ void genericLocaleStarterWithOptionsAndResult(const char *locale, const char *s[
   ucol_close(coll);
 }
 
-void genericLocaleStarterWithOptions(const char *locale, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
+void genericLocaleStarterWithOptions(const char *locale, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
   genericLocaleStarterWithOptionsAndResult(locale, s, size, attrs, values, attsize, UCOL_LESS);
 }
 
-void genericRulesStarterWithResult(const char *rules, const char *s[], uint32_t size, UCollationResult result) {
+void genericRulesStarterWithResult(const char *rules, const char * const s[], uint32_t size, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   UChar rlz[RULE_BUFFER_LEN] = { 0 };
   uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
@@ -678,7 +681,7 @@ void genericRulesStarterWithResult(const char *rules, const char *s[], uint32_t
   }
 }
 
-void genericRulesStarter(const char *rules, const char *s[], uint32_t size) {
+void genericRulesStarter(const char *rules, const char * const s[], uint32_t size) {
   genericRulesStarterWithResult(rules, s, size, UCOL_LESS);
 }
 
@@ -859,10 +862,8 @@ static void TestJB581(void)
     /* Now, do the same comparison with keys */
     sourceKeyOut = ucol_getSortKey(myCollator, source, -1, sourceKeyArray, 100);
     targetKeyOut = ucol_getSortKey(myCollator, target, -1, targetKeyArray, 100);
-    result = 0;
     bufferLen = ((targetKeyOut > 100) ? 100 : targetKeyOut);
-    result = memcmp(sourceKeyArray, targetKeyArray, bufferLen);
-    if (result != 0)
+    if (memcmp(sourceKeyArray, targetKeyArray, bufferLen) != 0)
     {
         log_err("Comparing two strings with sort keys in C failed.\n");
     }
@@ -1254,4 +1255,49 @@ TestFCDCrash(void) {
 
 }
 
+/*static UBool
+find(UEnumeration* list, const char* str, UErrorCode* status){
+    const char* value = NULL;
+    int32_t length=0;
+    if(U_FAILURE(*status)){
+        return FALSE;
+    }
+    uenum_reset(list, status);
+    while( (value= uenum_next(list, &length, status))!=NULL){
+        if(strcmp(value, str)==0){
+            return TRUE;
+        }
+    }
+    return FALSE;
+}*/
+
+static void TestJ5298(void)
+{
+    UErrorCode status = U_ZERO_ERROR;
+    char input[256], output[256];
+    UBool isAvailable;
+    int32_t i = 0;
+    UEnumeration* values = NULL;
+    const char *keywordValue = NULL;
+    log_verbose("Number of collator locales returned : %i \n", ucol_countAvailable());
+    values = ucol_getKeywordValues("collation", &status);
+    for (i = 0; i < ucol_countAvailable(); i++) {
+        uenum_reset(values, &status);
+        while ((keywordValue = uenum_next(values, NULL, &status)) != NULL) {
+            strcpy(input, ucol_getAvailable(i));
+            if (strcmp(keywordValue, "standard") != 0) {
+                strcat(input, "@collation=");
+                strcat(input, keywordValue);
+            }
+
+            ucol_getFunctionalEquivalent(output, 256, "collation", input, &isAvailable, &status);
+            if (strcmp(input, output) == 0) { /* Unique locale, print it out */
+                log_verbose("%s, \n", output);
+            }
+        }
+    }
+    uenum_close(values);
+    log_verbose("\n");
+}
 #endif /* #if !UCONFIG_NO_COLLATION */
+
