@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2007, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -15,8 +15,11 @@
 
 /* C API TEST FOR DATE FORMAT */
 
-#include "unicode/uloc.h"
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/uloc.h"
 #include "unicode/udat.h"
 #include "unicode/ucal.h"
 #include "unicode/unum.h"
@@ -26,27 +29,39 @@
 #include "cformtst.h"
 #include "cmemory.h"
 
+#include <math.h>
+
+static void TestExtremeDates(void);
+static void TestAllLocales(void);
+
+#define LEN(a) (sizeof(a)/sizeof(a[0]))
+
 void addDateForTest(TestNode** root);
 
+#define TESTCASE(x) addTest(root, &x, "tsformat/cdattst/" #x)
 
 void addDateForTest(TestNode** root)
 {
-    addTest(root, &TestDateFormat, "tsformat/cdattst/TestDateFormat");
-    addTest(root, &TestSymbols, "tsformat/cdattst/TestSymbols");
-    addTest(root, &TestDateFormatCalendar, "tsformat/cdattst/TestDateFormatCalendar");
+    TESTCASE(TestDateFormat);
+    TESTCASE(TestSymbols);
+    TESTCASE(TestDateFormatCalendar);
+    TESTCASE(TestExtremeDates);
+    TESTCASE(TestAllLocales);
 }
 /* Testing the DateFormat API */
 static void TestDateFormat()
 {
     UDateFormat *def, *fr, *it, *de, *def1, *fr_pat;
+    UDateFormat *any;
     UDateFormat *copy;
     UErrorCode status = U_ZERO_ERROR;
     UChar* result = NULL;
     const UCalendar *cal;
     const UNumberFormat *numformat1, *numformat2;
-    UChar temp[30];
+    UChar temp[50];
     int32_t numlocales;
     UDate d1;
+    int i;
     int32_t resultlength;
     int32_t resultlengthneeded;
     int32_t parsepos;
@@ -55,6 +70,9 @@ static void TestDateFormat()
     /*const char* str="yyyy.MM.dd G 'at' hh:mm:ss z";
     const char t[]="2/3/76 2:50 AM";*/
     /*Testing udat_open() to open a dateformat */
+
+    ctest_setTimeZone(NULL, &status);
+
     log_verbose("\nTesting udat_open() with various parameters\n");
     fr = udat_open(UDAT_FULL, UDAT_DEFAULT, "fr_FR", NULL,0, NULL, 0,&status);
     if(U_FAILURE(status))
@@ -102,11 +120,19 @@ static void TestDateFormat()
     numlocales=udat_countAvailable();
     /* use something sensible w/o hardcoding the count */
     if(numlocales < 0)
-        log_err("FAIL: error in countAvailable\n");
+        log_data_err("FAIL: error in countAvailable\n");
     log_verbose("The number of locales for which date/time formatting patterns are available is %d\n", numlocales);
-    /*for(i=0;i<numlocales;i++)
-        log_verbose("%s\n", uloc_getName(udat_getAvailable(i))); */
     
+    for(i=0;i<numlocales;i++) {
+      UErrorCode subStatus = U_ZERO_ERROR;
+      log_verbose("Testing open of %s\n", udat_getAvailable(i));
+      any = udat_open(UDAT_SHORT, UDAT_SHORT, udat_getAvailable(i), NULL ,0, NULL, 0, &subStatus);
+      if(U_FAILURE(subStatus)) {
+        log_data_err("FAIL: date format %s (getAvailable(%d)) is not instantiable: %s\n", udat_getAvailable(i), i, u_errorName(subStatus));
+      }
+      udat_close(any);
+    }
+
     /*Testing udat_clone()*/
     log_verbose("\nTesting the udat_clone() function of date format\n");
     copy=udat_clone(def, &status);
@@ -127,15 +153,16 @@ static void TestDateFormat()
         status=U_ZERO_ERROR;
         resultlength=resultlengthneeded+1;
         if(result != NULL) {
-            uprv_free(result);
+            free(result);
             result = NULL;
         }
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_format(def, d, result, resultlength, NULL, &status);
     }
-    if(U_FAILURE(status))
+    if(U_FAILURE(status) || !result)
     {
         log_err("FAIL: Error in formatting using udat_format(.....) %s\n", myErrorName(status) );
+        return;
     }
     else
         log_verbose("PASS: formatting successful\n");
@@ -145,29 +172,40 @@ static void TestDateFormat()
         log_err("FAIL: Date Format for US locale failed using udat_format()\n");
     /*format using fr */
     
-    u_uastrcpy(temp, "10 juil. 96 16 h 05 GMT-07:00");
+    u_unescape("10 juil. 1996 16:05:28 \\u00C9tats-Unis (Los Angeles)", temp, 50);
     if(result != NULL) {
-        uprv_free(result);
+        free(result);
         result = NULL;
     }
     result=myDateFormat(fr, d);
     if(u_strcmp(result, temp)==0)
-        log_verbose("PASS: Date Format for french locale successful uisng udat_format()\n");
+        log_verbose("PASS: Date Format for french locale successful using udat_format()\n");
     else
-        log_err("FAIL: Date Format for french locale failed using udat_format()\n");
-    /*foramt using it */
-    u_uastrcpy(temp, "10/lug/96 16:05:28");
-    
-    if(u_strcmp(myDateFormat(it,d), temp)==0)
-        log_verbose("PASS: Date Format for italian locale successful uisng udat_format()\n");
-    else
-        log_err("FAIL: Date Format for italian locale failed using udat_format()\n");
+        log_data_err("FAIL: Date Format for french locale failed using udat_format().\n" );
 
+    /*format using it */
+    u_uastrcpy(temp, "10/lug/1996 16:05:28");
+    
+    { 
+        UChar *fmtted;
+        char g[100];
+        char x[100];
+        
+        fmtted = myDateFormat(it,d);
+        u_austrcpy(g, fmtted);
+        u_austrcpy(x, temp);
+        if(u_strcmp(fmtted, temp)==0) {
+            log_verbose("PASS: Date Format for italian locale successful uisng udat_format() - wanted %s, got %s\n", x, g);
+        } else {
+            log_data_err("FAIL: Date Format for italian locale failed using udat_format() - wanted %s, got %s\n", x, g);
+        }
+    }
     
     /*Testing parsing using udat_parse()*/
     log_verbose("\nTesting parsing using udat_parse()\n");
     u_uastrcpy(temp,"2/3/76 2:50 AM");
     parsepos=0;
+    status=U_ZERO_ERROR;
     
     d1=udat_parse(def, temp, u_strlen(temp), &parsepos, &status);
     if(U_FAILURE(status))
@@ -182,6 +220,18 @@ static void TestDateFormat()
     if(u_strcmp(myDateFormat(def, d1),temp)!=0)
         log_err("FAIL: error in parsing\n");
 
+    /*Testing parsing using udat_parse()*/
+    log_verbose("\nTesting parsing using udat_parse()\n");
+    u_uastrcpy(temp,"2/Don't parse this part");
+    status=U_ZERO_ERROR;
+    
+    d1=udat_parse(def, temp, u_strlen(temp), NULL, &status);
+    if(status != U_PARSE_ERROR)
+    {
+        log_err("FAIL: udat_parse(\"bad string\") passed when it should have failed\n");
+    }
+    else
+        log_verbose("PASS: parsing succesful\n");
         
         
     
@@ -210,7 +260,7 @@ static void TestDateFormat()
     {
         status=U_ZERO_ERROR;
         resultlength=resultlengthneeded + 1;
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_toPattern(def1, FALSE, result, resultlength, &status);
     }
     if(U_FAILURE(status))
@@ -224,7 +274,7 @@ static void TestDateFormat()
         log_verbose("PASS: applyPattern and toPattern work fine\n");
     
     if(result != NULL) {
-        uprv_free(result);    
+        free(result);    
         result = NULL;
     }
     
@@ -299,7 +349,7 @@ static void TestDateFormat()
         log_verbose("PASS: getting and setting calendar successful\n");
         
     if(result!=NULL) {
-        uprv_free(result);
+        free(result);
     }
     
     /*Closing the UDateForamt */
@@ -311,6 +361,7 @@ static void TestDateFormat()
     udat_close(fr_pat);
     udat_close(copy);
     
+    ctest_resetTimeZone();
 }
 
 /*Testing udat_getSymbols() and udat_setSymbols() and udat_countSymbols()*/
@@ -354,6 +405,7 @@ static void TestSymbols()
     if(udat_countSymbols(def, UDAT_ERAS)!=2 || udat_countSymbols(def, UDAT_MONTHS)!=12 || 
         udat_countSymbols(def, UDAT_SHORT_MONTHS)!=12 || udat_countSymbols(def, UDAT_WEEKDAYS)!=8 ||
         udat_countSymbols(def, UDAT_SHORT_WEEKDAYS)!=8 || udat_countSymbols(def, UDAT_AM_PMS)!=2 ||
+        udat_countSymbols(def, UDAT_QUARTERS) != 4 || udat_countSymbols(def, UDAT_SHORT_QUARTERS) != 4 ||
         udat_countSymbols(def, UDAT_LOCALIZED_CHARS)!=1)
     {
         log_err("FAIL: error in udat_countSymbols\n");
@@ -363,7 +415,7 @@ static void TestSymbols()
 
     /*testing getSymbols*/
     log_verbose("\nTesting getSymbols\n");
-    pattern=(UChar*)uprv_malloc(sizeof(UChar) * 10);
+    pattern=(UChar*)malloc(sizeof(UChar) * 10);
     u_uastrcpy(pattern, "jeudi");
     resultlength=0;
     resultlengthout=udat_getSymbols(fr, UDAT_WEEKDAYS, 5 , NULL, resultlength, &status);
@@ -372,10 +424,10 @@ static void TestSymbols()
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
         if(result != NULL) {
-            uprv_free(result);
+            free(result);
             result = NULL;
         }
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(fr, UDAT_WEEKDAYS, 5, result, resultlength, &status);
         
     }
@@ -389,7 +441,7 @@ static void TestSymbols()
     if(u_strcmp(result, pattern)==0)
         log_verbose("PASS: getSymbols retrieved the right value\n");
     else
-        log_err("FAIL: getSymbols retrieved the wrong value\n");
+        log_data_err("FAIL: getSymbols retrieved the wrong value\n");
 
     /*run series of tests to test getsymbols regressively*/
     log_verbose("\nTesting getSymbols() regressively\n");
@@ -404,14 +456,18 @@ static void TestSymbols()
     VerifygetSymbols(def, UDAT_AM_PMS, 1, "PM");
     VerifygetSymbols(fr, UDAT_SHORT_MONTHS, 0, "janv.");
     VerifygetSymbols(def, UDAT_SHORT_MONTHS, 11, "Dec");
-    VerifygetSymbols(def,UDAT_LOCALIZED_CHARS, 0, "GyMdkHmsSEDFwWahKzYe");
+    VerifygetSymbols(fr, UDAT_QUARTERS, 0, "1er trimestre");
+    VerifygetSymbols(def, UDAT_QUARTERS, 3, "4th quarter");
+    VerifygetSymbols(fr, UDAT_SHORT_QUARTERS, 1, "T2");
+    VerifygetSymbols(def, UDAT_SHORT_QUARTERS, 2, "Q3");
+    VerifygetSymbols(def,UDAT_LOCALIZED_CHARS, 0, "GyMdkHmsSEDFwWahKzYeugAZvcLQqV");
 
 
     if(result != NULL) {
-        uprv_free(result);
+        free(result);
         result = NULL;
     }
-uprv_free(pattern);    
+free(pattern);    
     
     log_verbose("\nTesting setSymbols\n");
     /*applying the pattern so that setSymbolss works */
@@ -421,7 +477,7 @@ uprv_free(pattern);
     {
         status=U_ZERO_ERROR;
         resultlength=resultlengthout + 1;
-        pattern=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        pattern=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_toPattern(fr, FALSE, pattern, resultlength, &status);
     }
     if(U_FAILURE(status))
@@ -438,10 +494,10 @@ uprv_free(pattern);
         status=U_ZERO_ERROR;
         resultlength=resultlengthout + 1;
         if(result != NULL) {
-            uprv_free(result);
+            free(result);
             result = NULL;
         }
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_toPattern(fr, FALSE,result, resultlength, &status);
     }
     if(U_FAILURE(status))
@@ -454,7 +510,7 @@ uprv_free(pattern);
     else
         log_err("pattern could not be applied properly\n");
 
-uprv_free(pattern);
+free(pattern);
     /*testing set symbols */
     resultlength=0;
     resultlengthout=udat_getSymbols(fr, UDAT_MONTHS, 11 , NULL, resultlength, &status);
@@ -462,10 +518,10 @@ uprv_free(pattern);
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
         if(result != NULL) {
-            uprv_free(result);
+            free(result);
             result = NULL;
         }
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(fr, UDAT_MONTHS, 11, result, resultlength, &status);
         
     }
@@ -486,25 +542,38 @@ uprv_free(pattern);
     if(status==U_BUFFER_OVERFLOW_ERROR){
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
-        value=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        value=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(def, UDAT_MONTHS, 11, value, resultlength, &status);
     }
     if(U_FAILURE(status))
         log_err("FAIL: error in retrieving the value using getSymbols i.e roundtrip\n");
     
     if(u_strcmp(result, value)!=0)
-        log_err("FAIL: Error in settting and getting symbols\n");
+        log_data_err("FAIL: Error in settting and getting symbols\n");
     else
         log_verbose("PASS: setSymbols successful\n");
     
     
     /*run series of tests to test setSymbols regressively*/
     log_verbose("\nTesting setSymbols regressively\n");
-    VerifysetSymbols(def, UDAT_WEEKDAYS, 1, "Sundayweek");
     VerifysetSymbols(def, UDAT_ERAS, 0, "BeforeChrist");
+    VerifysetSymbols(def, UDAT_ERA_NAMES, 1, "AnnoDomini");
+    VerifysetSymbols(def, UDAT_WEEKDAYS, 1, "Sundayweek");
     VerifysetSymbols(def, UDAT_SHORT_WEEKDAYS, 7, "Satweek");
+    VerifysetSymbols(def, UDAT_NARROW_WEEKDAYS, 4, "M");
+    VerifysetSymbols(def, UDAT_STANDALONE_WEEKDAYS, 1, "Sonntagweek");
+    VerifysetSymbols(def, UDAT_STANDALONE_SHORT_WEEKDAYS, 7, "Sams");
+    VerifysetSymbols(def, UDAT_STANDALONE_NARROW_WEEKDAYS, 4, "V");
     VerifysetSymbols(fr, UDAT_MONTHS, 11, "december");
     VerifysetSymbols(fr, UDAT_SHORT_MONTHS, 0, "Jan");
+    VerifysetSymbols(fr, UDAT_NARROW_MONTHS, 1, "R");
+    VerifysetSymbols(fr, UDAT_STANDALONE_MONTHS, 11, "dezember");
+    VerifysetSymbols(fr, UDAT_STANDALONE_SHORT_MONTHS, 7, "Aug");
+    VerifysetSymbols(fr, UDAT_STANDALONE_NARROW_MONTHS, 2, "M");
+    VerifysetSymbols(fr, UDAT_QUARTERS, 0, "1. Quart");
+    VerifysetSymbols(fr, UDAT_SHORT_QUARTERS, 1, "QQ2");
+    VerifysetSymbols(fr, UDAT_STANDALONE_QUARTERS, 2, "3rd Quar.");
+    VerifysetSymbols(fr, UDAT_STANDALONE_SHORT_QUARTERS, 3, "4QQ");
 
     
     /*run series of tests to test get and setSymbols regressively*/
@@ -525,10 +594,10 @@ uprv_free(pattern);
     udat_close(fr);
     udat_close(def);
     if(result != NULL) {
-        uprv_free(result);
+        free(result);
         result = NULL;
     }
-    uprv_free(value);
+    free(value);
     
 }
 
@@ -544,6 +613,8 @@ static void TestDateFormatCalendar() {
     UDate when;
     UErrorCode ec = U_ZERO_ERROR;
 
+    ctest_setTimeZone(NULL, &ec);
+
     /* Create a formatter for date fields. */
     date = udat_open(UDAT_NONE, UDAT_SHORT, "en_US", NULL, 0, NULL, 0, &ec);
     if (U_FAILURE(ec)) {
@@ -613,7 +684,7 @@ static void TestDateFormatCalendar() {
     if (when == 986517900000.0) {
         log_verbose("Ok: Parsed result: %s\n", cbuf);
     } else {
-        log_err("FAIL: Parsed result: %s, exp 4/5/2001 5:45 PM", cbuf);
+        log_err("FAIL: Parsed result: %s, exp 4/5/2001 5:45 PM\n", cbuf);
     }
 
  FAIL:    
@@ -621,6 +692,8 @@ static void TestDateFormatCalendar() {
     udat_close(time);
     udat_close(full);
     ucal_close(cal);
+
+    ctest_resetTimeZone();
 }
 
 /*INTERNAL FUNCTIONS USED*/
@@ -632,7 +705,7 @@ static void VerifygetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     int32_t resultlength, resultlengthout;
 
     
-    pattern=(UChar*)uprv_malloc(sizeof(UChar) * (strlen(expected)+1));
+    pattern=(UChar*)malloc(sizeof(UChar) * (strlen(expected)+1));
     u_uastrcpy(pattern, expected);
     resultlength=0;
     resultlengthout=udat_getSymbols(datfor, type, index , NULL, resultlength, &status);
@@ -640,7 +713,7 @@ static void VerifygetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     {
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(datfor, type, index, result, resultlength, &status);
         
     }
@@ -652,11 +725,11 @@ static void VerifygetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     if(u_strcmp(result, pattern)==0)
         log_verbose("PASS: getSymbols retrieved the right value\n");
     else{
-        log_err("FAlL: getSymbols retrieved the wrong value\n Expected %s Got %s\n", austrdup(pattern), 
+        log_data_err("FAIL: getSymbols retrieved the wrong value\n Expected %s Got %s\n", austrdup(pattern), 
             austrdup(result) );
     }
-    uprv_free(result);
-    uprv_free(pattern);
+    free(result);
+    free(pattern);
 }
 
 static void VerifysetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, int32_t index, const char* expected)
@@ -666,7 +739,7 @@ static void VerifysetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     int32_t resultlength, resultlengthout;
     UErrorCode status = U_ZERO_ERROR;
 
-    value=(UChar*)uprv_malloc(sizeof(UChar) * (strlen(expected) + 1));
+    value=(UChar*)malloc(sizeof(UChar) * (strlen(expected) + 1));
     u_uastrcpy(value, expected);
     udat_setSymbols(datfor, type, index, value, u_strlen(value), &status);
     if(U_FAILURE(status))
@@ -680,7 +753,7 @@ static void VerifysetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     if(status==U_BUFFER_OVERFLOW_ERROR){
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(datfor, type, index, result, resultlength, &status);
     }
     if(U_FAILURE(status)){
@@ -696,8 +769,8 @@ static void VerifysetSymbols(UDateFormat* datfor, UDateFormatSymbolType type, in
     else
         log_verbose("PASS: setSymbols successful\n");
 
-    uprv_free(value);
-    uprv_free(result);
+    free(value);
+    free(result);
 }
 
 
@@ -713,7 +786,7 @@ static void VerifygetsetSymbols(UDateFormat* from, UDateFormat* to, UDateFormatS
     if(status==U_BUFFER_OVERFLOW_ERROR){
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
-        result=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(from, type, index, result, resultlength, &status);
     }
     if(U_FAILURE(status)){
@@ -734,7 +807,7 @@ static void VerifygetsetSymbols(UDateFormat* from, UDateFormat* to, UDateFormatS
     if(status==U_BUFFER_OVERFLOW_ERROR){
         status=U_ZERO_ERROR;
         resultlength=resultlengthout+1;
-        value=(UChar*)uprv_malloc(sizeof(UChar) * resultlength);
+        value=(UChar*)malloc(sizeof(UChar) * resultlength);
         udat_getSymbols(to, type, index, value, resultlength, &status);
     }
     if(U_FAILURE(status)){
@@ -744,14 +817,14 @@ static void VerifygetsetSymbols(UDateFormat* from, UDateFormat* to, UDateFormatS
     }
     
     if(u_strcmp(result, value)!=0){
-        log_err("FAIL:Error in setting and then getting symbols\n Expected %s Got %s\n", austrdup(result),
+        log_data_err("FAIL:Error in setting and then getting symbols\n Expected %s Got %s\n", austrdup(result),
             austrdup(value) );
     }
     else
         log_verbose("PASS: setSymbols successful\n");
 
-    uprv_free(value);
-    uprv_free(result);
+    free(value);
+    free(result);
 }
 
 
@@ -779,3 +852,107 @@ static UChar* myNumformat(const UNumberFormat* numfor, double d)
     
     return result2;
 }
+
+/**
+ * The search depth for TestExtremeDates.  The total number of
+ * dates that will be tested is (2^EXTREME_DATES_DEPTH) - 1.
+ */
+#define EXTREME_DATES_DEPTH 8
+
+/**
+ * Support for TestExtremeDates (below).
+ *
+ * Test a single date to see whether udat_format handles it properly.
+ */
+static UBool _aux1ExtremeDates(UDateFormat* fmt, UDate date,
+                               UChar* buf, int32_t buflen, char* cbuf,
+                               UErrorCode* ec) {
+    int32_t len = udat_format(fmt, date, buf, buflen, 0, ec);
+    if (!assertSuccess("udat_format", ec)) return FALSE;
+    u_austrncpy(cbuf, buf, buflen);
+    if (len < 4) {
+        log_err("FAIL: udat_format(%g) => \"%s\"\n", date, cbuf);
+    } else {
+        log_verbose("udat_format(%g) => \"%s\"\n", date, cbuf);
+    }
+    return TRUE;
+}
+
+/**
+ * Support for TestExtremeDates (below).
+ *
+ * Recursively test between 'small' and 'large', up to the depth
+ * limit specified by EXTREME_DATES_DEPTH.
+ */
+static UBool _aux2ExtremeDates(UDateFormat* fmt, UDate small, UDate large,
+                               UChar* buf, int32_t buflen, char* cbuf,
+                               int32_t count,
+                               UErrorCode* ec) {
+    /* Logarithmic midpoint; see below */
+    UDate mid = (UDate) exp((log(small) + log(large)) / 2);
+    if (count == EXTREME_DATES_DEPTH) {
+        return TRUE;
+    }
+    return
+        _aux1ExtremeDates(fmt, mid, buf, buflen, cbuf, ec) &&
+        _aux2ExtremeDates(fmt, small, mid, buf, buflen, cbuf, count+1, ec) &&
+        _aux2ExtremeDates(fmt, mid, large, buf, buflen, cbuf, count+1, ec);
+}
+
+/**
+ * http://www.jtcsv.com/cgibin/icu-bugs?findid=3659
+ *
+ * For certain large dates, udat_format crashes on MacOS.  This test
+ * attempts to reproduce this problem by doing a recursive logarithmic*
+ * binary search of a predefined interval (from 'small' to 'large').
+ *
+ * The limit of the search is given by EXTREME_DATES_DEPTH, above.
+ *
+ * *The search has to be logarithmic, not linear.  A linear search of the
+ *  range 0..10^30, for example, will find 0.5*10^30, then 0.25*10^30 and
+ *  0.75*10^30, etc.  A logarithmic search will find 10^15, then 10^7.5
+ *  and 10^22.5, etc.
+ */
+static void TestExtremeDates() {
+    UDateFormat *fmt;
+    UErrorCode ec;
+    UChar buf[256];
+    char cbuf[256];
+    const double small = 1000; /* 1 sec */
+    const double large = 1e+30; /* well beyond usable UDate range */
+
+    /* There is no need to test larger values from 1e+30 to 1e+300;
+       the failures occur around 1e+27, and never above 1e+30. */
+    
+    ec = U_ZERO_ERROR;
+    fmt = udat_open(UDAT_LONG, UDAT_LONG, "en_US",
+                    0, 0, 0, 0, &ec);
+    if (!assertSuccess("udat_open", &ec)) return;
+
+    _aux2ExtremeDates(fmt, small, large, buf, LEN(buf), cbuf, 0, &ec);
+
+    udat_close(fmt);
+}
+
+static void TestAllLocales(void) {
+    int32_t idx, dateIdx, timeIdx, localeCount;
+    static const UDateFormatStyle style[] = {
+        UDAT_FULL, UDAT_LONG, UDAT_MEDIUM, UDAT_SHORT
+    };
+    localeCount = uloc_countAvailable();
+    for (idx = 0; idx < localeCount; idx++) {
+        for (dateIdx = 0; dateIdx < (int32_t)(sizeof(style)/sizeof(style[0])); dateIdx++) {
+            for (timeIdx = 0; timeIdx < (int32_t)(sizeof(style)/sizeof(style[0])); timeIdx++) {
+                UErrorCode status = U_ZERO_ERROR;
+                udat_close(udat_open(style[dateIdx], style[timeIdx],
+                    uloc_getAvailable(idx), NULL, 0, NULL, 0, &status));
+                if (U_FAILURE(status)) {
+                    log_err("FAIL: udat_open(%s) failed with (%s) dateIdx=%d, timeIdx=%d\n",
+                        uloc_getAvailable(idx), u_errorName(status), dateIdx, timeIdx);
+                }
+            }
+        }
+    }
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
