@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2004, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -18,10 +18,15 @@
  */
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/uloc.h"
 #include "unicode/uchar.h"
 #include "unicode/ustring.h"
+#include "unicode/putil.h"
+#include "callcoll.h"
 #include "cmemory.h"
 #include "cintltst.h"
 #include "citertst.h"
@@ -370,18 +375,18 @@ static void TestNormalization()
     }
 
     srclen = u_unescape(testdata[0], source, 10);
-    iter = ucol_openElements(coll, source, srclen - 1, &status);
+    iter = ucol_openElements(coll, source, srclen, &status);
     backAndForth(iter);
     ucol_closeElements(iter);
 
     srclen = u_unescape(testdata[1], source, 10);
-    iter = ucol_openElements(coll, source, srclen - 1, &status);
+    iter = ucol_openElements(coll, source, srclen, &status);
     backAndForth(iter);
     ucol_closeElements(iter);
 
     while (count < 12) {
         srclen = u_unescape(testdata[count], source, 10);
-        iter = ucol_openElements(coll, source, srclen - 1, &status);
+        iter = ucol_openElements(coll, source, srclen, &status);
 
         if (U_FAILURE(status)){
             log_err("ERROR: in creation of collator element iterator\n %s\n",
@@ -567,7 +572,7 @@ static void TestOffset()
     u_uastrcpy(test1, "What subset of all possible test cases?");
     u_uastrcpy(test2, "has the highest probability of detecting");
     en_us = ucol_open("en_US", &status);
-    log_verbose("Testing getOffset and setOffset for CollationElements\n");
+    log_verbose("Testing getOffset and setOffset for collations\n");
     iter = ucol_openElements(en_us, test1, u_strlen(test1), &status);
     if(U_FAILURE(status)){
         log_err("ERROR: in creation of collation element iterator using ucol_openElements()\n %s\n",
@@ -575,6 +580,19 @@ static void TestOffset()
         ucol_close(en_us);
         return;
     }
+
+    /* testing boundaries */
+    ucol_setOffset(iter, 0, &status);
+    if (U_FAILURE(status) || ucol_previous(iter, &status) != UCOL_NULLORDER) {
+        log_err("Error: After setting offset to 0, we should be at the end "
+                "of the backwards iteration");
+    }
+    ucol_setOffset(iter, u_strlen(test1), &status);
+    if (U_FAILURE(status) || ucol_next(iter, &status) != UCOL_NULLORDER) {
+        log_err("Error: After setting offset to end of the string, we should "
+                "be at the end of the backwards iteration");
+    }
+
     /* Run all the way through the iterator, then get the offset */
 
     orders = getOrders(iter, &orderLength);
@@ -756,69 +774,6 @@ static void TestSetText()
     free(test2);
 }
 
-
-
-static void backAndForth(UCollationElements *iter)
-{
-    /* Run through the iterator forwards and stick it into an array */
-    int32_t index, o;
-    UErrorCode status = U_ZERO_ERROR;
-    int32_t orderLength = 0;
-    int32_t *orders;
-    orders= getOrders(iter, &orderLength);
-
-
-    /* Now go through it backwards and make sure we get the same values */
-    index = orderLength;
-    ucol_reset(iter);
-
-    /* synwee : changed */
-    while ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
-    {
-      if (o != orders[-- index])
-      {
-        if (o == 0)
-          index ++;
-        else
-        {
-          while (index > 0 && orders[-- index] == 0)
-          {
-          }
-          if (o != orders[index])
-          {
-            log_err("Mismatch at index : 0x%x\n", index);
-            return;
-          }
-
-        }
-      }
-    }
-
-    while (index != 0 && orders[index - 1] == 0) {
-      index --;
-    }
-
-    if (index != 0)
-    {
-        log_err("Didn't get back to beginning - index is %d\n", index);
-
-        ucol_reset(iter);
-        log_err("\nnext: ");
-        if ((o = ucol_next(iter, &status)) != UCOL_NULLORDER)
-        {
-            log_err("Error at %x\n", o);
-        }
-        log_err("\nprev: ");
-        if ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
-        {
-            log_err("Error at %x\n", o);
-        }
-        log_verbose("\n");
-    }
-
-    free(orders);
-}
-
 /** @bug 4108762
  * Test for getMaxExpansion()
  */
@@ -827,7 +782,10 @@ static void TestMaxExpansion()
     UErrorCode          status = U_ZERO_ERROR;
     UCollator          *coll   ;/*= ucol_open("en_US", &status);*/
     UChar               ch     = 0;
-    UChar               supplementary[2] = {0xD800, 0xDC00};
+    UChar32             unassigned = 0xEFFFD;
+    UChar               supplementary[2];
+    uint32_t            index = 0;
+    UBool               isError = FALSE;
     uint32_t            sorder = 0;
     UCollationElements *iter   ;/*= ucol_openElements(coll, &ch, 1, &status);*/
     uint32_t            temporder = 0;
@@ -836,164 +794,118 @@ static void TestMaxExpansion()
     u_uastrcpy(rule, "&a < ab < c/aba < d < z < ch");
     coll = ucol_openRules(rule, u_strlen(rule), UCOL_DEFAULT,
         UCOL_DEFAULT_STRENGTH,NULL, &status);
-    iter = ucol_openElements(coll, &ch, 1, &status);
+    if(U_SUCCESS(status) && coll) {
+      iter = ucol_openElements(coll, &ch, 1, &status);
 
-    while (ch < 0xFFFF && U_SUCCESS(status)) {
-        int      count = 1;
-        uint32_t order;
-        int32_t  size = 0;
+      while (ch < 0xFFFF && U_SUCCESS(status)) {
+          int      count = 1;
+          uint32_t order;
+          int32_t  size = 0;
 
-        ch ++;
+          ch ++;
 
-        ucol_setText(iter, &ch, 1, &status);
-        order = ucol_previous(iter, &status);
+          ucol_setText(iter, &ch, 1, &status);
+          order = ucol_previous(iter, &status);
 
-        /* thai management */
-        if (order == 0)
-            order = ucol_previous(iter, &status);
+          /* thai management */
+          if (order == 0)
+              order = ucol_previous(iter, &status);
 
-        while (U_SUCCESS(status) &&
-            ucol_previous(iter, &status) != UCOL_NULLORDER) {
-            count ++;
-        }
+          while (U_SUCCESS(status) &&
+              ucol_previous(iter, &status) != UCOL_NULLORDER) {
+              count ++;
+          }
 
-        size = ucol_getMaxExpansion(iter, order);
-        if (U_FAILURE(status) || size < count) {
-            log_err("Failure at codepoint %d, maximum expansion count < %d\n",
-                ch, count);
-        }
+          size = ucol_getMaxExpansion(iter, order);
+          if (U_FAILURE(status) || size < count) {
+              log_err("Failure at codepoint %d, maximum expansion count < %d\n",
+                  ch, count);
+          }
+      }
+
+      /* testing for exact max expansion */
+      ch = 0;
+      while (ch < 0x61) {
+          uint32_t order;
+          int32_t  size;
+          ucol_setText(iter, &ch, 1, &status);
+          order = ucol_previous(iter, &status);
+          size  = ucol_getMaxExpansion(iter, order);
+          if (U_FAILURE(status) || size != 1) {
+              log_err("Failure at codepoint %d, maximum expansion count < %d\n",
+                  ch, 1);
+          }
+          ch ++;
+      }
+
+      ch = 0x63;
+      ucol_setText(iter, &ch, 1, &status);
+      temporder = ucol_previous(iter, &status);
+
+      if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 3) {
+          log_err("Failure at codepoint %d, maximum expansion count != %d\n",
+                  ch, 3);
+      }
+
+      ch = 0x64;
+      ucol_setText(iter, &ch, 1, &status);
+      temporder = ucol_previous(iter, &status);
+
+      if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 1) {
+          log_err("Failure at codepoint %d, maximum expansion count != %d\n",
+                  ch, 3);
+      }
+
+      U16_APPEND(supplementary, index, 2, unassigned, isError);
+      ucol_setText(iter, supplementary, 2, &status);
+      sorder = ucol_previous(iter, &status);
+
+      if (U_FAILURE(status) || ucol_getMaxExpansion(iter, sorder) != 2) {
+          log_err("Failure at codepoint %d, maximum expansion count < %d\n",
+                  ch, 2);
+      }
+
+      /* testing jamo */
+      ch = 0x1165;
+
+      ucol_setText(iter, &ch, 1, &status);
+      temporder = ucol_previous(iter, &status);
+      if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) > 3) {
+          log_err("Failure at codepoint %d, maximum expansion count > %d\n",
+                  ch, 3);
+      }
+
+      ucol_closeElements(iter);
+      ucol_close(coll);
+
+      /* testing special jamo &a<\u1160 */
+      rule[0] = 0x26;
+      rule[1] = 0x71;
+      rule[2] = 0x3c;
+      rule[3] = 0x1165;
+      rule[4] = 0x2f;
+      rule[5] = 0x71;
+      rule[6] = 0x71;
+      rule[7] = 0x71;
+      rule[8] = 0x71;
+      rule[9] = 0;
+
+      coll = ucol_openRules(rule, u_strlen(rule), UCOL_DEFAULT,
+          UCOL_DEFAULT_STRENGTH,NULL, &status);
+      iter = ucol_openElements(coll, &ch, 1, &status);
+
+      temporder = ucol_previous(iter, &status);
+      if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 6) {
+          log_err("Failure at codepoint %d, maximum expansion count > %d\n",
+                  ch, 5);
+      }
+
+      ucol_closeElements(iter);
+      ucol_close(coll);
+    } else {
+      log_data_err("Couldn't open collator\n");
     }
 
-    /* testing for exact max expansion */
-    ch = 0;
-    while (ch < 0x61) {
-        uint32_t order;
-        int32_t  size;
-        ucol_setText(iter, &ch, 1, &status);
-        order = ucol_previous(iter, &status);
-        size  = ucol_getMaxExpansion(iter, order);
-        if (U_FAILURE(status) || size != 1) {
-            log_err("Failure at codepoint %d, maximum expansion count < %d\n",
-                ch, 1);
-        }
-        ch ++;
-    }
-
-    ch = 0x63;
-    ucol_setText(iter, &ch, 1, &status);
-    temporder = ucol_previous(iter, &status);
-
-    if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 3) {
-        log_err("Failure at codepoint %d, maximum expansion count != %d\n",
-                ch, 3);
-    }
-
-    ch = 0x64;
-    ucol_setText(iter, &ch, 1, &status);
-    temporder = ucol_previous(iter, &status);
-
-    if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 1) {
-        log_err("Failure at codepoint %d, maximum expansion count != %d\n",
-                ch, 3);
-    }
-
-    ucol_setText(iter, supplementary, 2, &status);
-    sorder = ucol_previous(iter, &status);
-
-    if (U_FAILURE(status) || ucol_getMaxExpansion(iter, sorder) != 2) {
-        log_err("Failure at codepoint %d, maximum expansion count < %d\n",
-                ch, 2);
-    }
-
-    /* testing jamo */
-    ch = 0x1165;
-
-    ucol_setText(iter, &ch, 1, &status);
-    temporder = ucol_previous(iter, &status);
-    if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) > 3) {
-        log_err("Failure at codepoint %d, maximum expansion count > %d\n",
-                ch, 3);
-    }
-
-    ucol_closeElements(iter);
-    ucol_close(coll);
-
-    /* testing special jamo &a<\u1160 */
-    rule[0] = 0x26;
-    rule[1] = 0x71;
-    rule[2] = 0x3c;
-    rule[3] = 0x1165;
-    rule[4] = 0x2f;
-    rule[5] = 0x71;
-    rule[6] = 0x71;
-    rule[7] = 0x71;
-    rule[8] = 0x71;
-    rule[9] = 0;
-
-    coll = ucol_openRules(rule, u_strlen(rule), UCOL_DEFAULT,
-        UCOL_DEFAULT_STRENGTH,NULL, &status);
-    iter = ucol_openElements(coll, &ch, 1, &status);
-
-    temporder = ucol_previous(iter, &status);
-    if (U_FAILURE(status) || ucol_getMaxExpansion(iter, temporder) != 6) {
-        log_err("Failure at codepoint %d, maximum expansion count > %d\n",
-                ch, 5);
-    }
-
-    ucol_closeElements(iter);
-    ucol_close(coll);
-
-}
-
-/**
- * Return an integer array containing all of the collation orders
- * returned by calls to next on the specified iterator
- */
-static int32_t* getOrders(UCollationElements *iter, int32_t *orderLength)
-{
-    UErrorCode status;
-    int32_t order;
-    int32_t maxSize = 100;
-    int32_t size = 0;
-    int32_t *temp;
-    int32_t *orders =(int32_t*)malloc(sizeof(int32_t) * maxSize);
-    status= U_ZERO_ERROR;
-
-
-    while ((order=ucol_next(iter, &status)) != UCOL_NULLORDER)
-    {
-        if (size == maxSize)
-        {
-            maxSize *= 2;
-            temp = (int32_t*)malloc(sizeof(int32_t) * maxSize);
-
-            memcpy(temp, orders, size * sizeof(int32_t));
-            free(orders);
-            orders = temp;
-
-        }
-
-        orders[size++] = order;
-    }
-
-    if (maxSize > size)
-    {
-        if (size == 0) {
-            size = 1;
-            temp = (int32_t*)malloc(sizeof(int32_t) * size);
-            temp[0] = 0;
-        }
-        else {
-            temp = (int32_t*)malloc(sizeof(int32_t) * size);
-            memcpy(temp, orders, size * sizeof(int32_t));
-        }
-
-        free(orders);
-        orders = temp;
-    }
-
-    *orderLength = size;
-    return orders;
 }
 
 
@@ -1048,49 +960,53 @@ static void TestSmallBuffer()
     }
 
     coll = ucol_open("th_TH", &status);
-    testiter = ucol_openElements(coll, teststr, 500, &status);
-    iter = ucol_openElements(coll, str, 2, &status);
+    if(U_SUCCESS(status) && coll) {
+      testiter = ucol_openElements(coll, teststr, 500, &status);
+      iter = ucol_openElements(coll, str, 2, &status);
 
-    orders     = getOrders(iter, &count);
-    if (count != 2) {
-        log_err("Error collation elements size is not 2 for \\u0300\\u031A\n");
+      orders     = getOrders(iter, &count);
+      if (count != 2) {
+          log_err("Error collation elements size is not 2 for \\u0300\\u031A\n");
+      }
+
+      /*
+      this will rearrange the string data to 250 characters of 0x300 first then
+      250 characters of 0x031A
+      */
+      testorders = getOrders(testiter, &count);
+
+      if (count != 500) {
+          log_err("Error decomposition does not give the right sized collation elements\n");
+      }
+
+      while (count != 0) {
+          /* UCA collation element for 0x0F76 */
+          if ((count > 250 && testorders[-- count] != orders[1]) ||
+              (count <= 250 && testorders[-- count] != orders[0])) {
+              log_err("Error decomposition does not give the right collation element at %d count\n", count);
+              break;
+          }
+      }
+
+      free(testorders);
+      free(orders);
+
+      ucol_reset(testiter);
+      /* ensures that the writable buffer was cleared */
+      if (testiter->iteratordata_.writableBuffer !=
+          testiter->iteratordata_.stackWritableBuffer) {
+          log_err("Error Writable buffer in collation element iterator not reset\n");
+      }
+
+      /* ensures closing of elements done properly to clear writable buffer */
+      ucol_next(testiter, &status);
+      ucol_next(testiter, &status);
+      ucol_closeElements(testiter);
+      ucol_closeElements(iter);
+      ucol_close(coll);
+    } else {
+      log_data_err("Couldn't open collator\n");
     }
-
-    /*
-    this will rearrange the string data to 250 characters of 0x300 first then
-    250 characters of 0x031A
-    */
-    testorders = getOrders(testiter, &count);
-
-    if (count != 500) {
-        log_err("Error decomposition does not give the right sized collation elements\n");
-    }
-
-    while (count != 0) {
-        /* UCA collation element for 0x0F76 */
-        if ((count > 250 && testorders[-- count] != orders[1]) ||
-            (count <= 250 && testorders[-- count] != orders[0])) {
-            log_err("Error decomposition does not give the right collation element at %d count\n", count);
-            break;
-        }
-    }
-
-    free(testorders);
-    free(orders);
-
-    ucol_reset(testiter);
-    /* ensures that the writable buffer was cleared */
-    if (testiter->iteratordata_.writableBuffer !=
-        testiter->iteratordata_.stackWritableBuffer) {
-        log_err("Error Writable buffer in collation element iterator not reset\n");
-    }
-
-    /* ensures closing of elements done properly to clear writable buffer */
-    ucol_next(testiter, &status);
-    ucol_next(testiter, &status);
-    ucol_closeElements(testiter);
-    ucol_closeElements(iter);
-    ucol_close(coll);
 }
 
 /**
@@ -1195,9 +1111,9 @@ getSingleCEValue(char *primary, char *secondary, char *tertiary,
             *terend = '\0';
         }
 
-        primvalue = (*primary!='\0')?strtoul(primary, &primend, 16):0;
-        secvalue  = (*secondary!='\0')?strtoul(secondary, &secend, 16):0;
-        tervalue  = (*tertiary!='\0')?strtoul(tertiary, &terend, 16):0;
+        primvalue = (*primary!='\0')?uprv_strtoul(primary, &primend, 16):0;
+        secvalue  = (*secondary!='\0')?uprv_strtoul(secondary, &secend, 16):0;
+        tervalue  = (*tertiary!='\0')?uprv_strtoul(tertiary, &terend, 16):0;
         if(primvalue <= 0xFF) {
           primvalue <<= 8;
         }
@@ -1300,19 +1216,22 @@ static FileStream * getFractionalUCA(void)
     FileStream *result = NULL;
 
     /* Look inside ICU_DATA first */
-    uprv_strcpy(newPath, u_getDataDirectory());
+    uprv_strcpy(newPath, ctest_dataSrcDir());
     uprv_strcat(newPath, "unidata" U_FILE_SEP_STRING );
     uprv_strcat(newPath, "FractionalUCA.txt");
 
     /* As a fallback, try to guess where the source data was located
      *   at the time ICU was built, and look there.
      */
-    #if defined (U_TOPSRCDIR)
-        strcpy(backupPath, U_TOPSRCDIR  U_FILE_SEP_STRING "data");
-    #else
-        strcpy(backupPath, u_getDataDirectory());
-        strcat(backupPath, ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING "data");
-    #endif
+#if defined (U_TOPSRCDIR)
+    strcpy(backupPath, U_TOPSRCDIR  U_FILE_SEP_STRING "data");
+#else
+    {
+        UErrorCode errorCode = U_ZERO_ERROR;
+        strcpy(backupPath, loadTestData(&errorCode));
+        strcat(backupPath, U_FILE_SEP_STRING ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING ".." U_FILE_SEP_STRING "data");
+    }
+#endif
     strcat(backupPath, U_FILE_SEP_STRING "unidata" U_FILE_SEP_STRING "FractionalUCA.txt");
 
     result = T_FileStream_open(newPath, "rb");
@@ -1331,12 +1250,13 @@ static FileStream * getFractionalUCA(void)
 */
 static void TestCEs() {
     FileStream *file = NULL;
-    char        line[300];
+    char        line[1024];
     char       *str;
     UChar       codepoints[5];
     uint32_t    ces[20];
     UErrorCode  status = U_ZERO_ERROR;
     UCollator          *coll = ucol_open("", &status);
+    uint32_t lineNo = 0;
 
     if (U_FAILURE(status)) {
         log_err("Error in opening root collator\n");
@@ -1350,9 +1270,11 @@ static void TestCEs() {
         return;
     }
 
+
     while (T_FileStream_readLine(file, line, sizeof(line)) != NULL) {
         int                 count = 0;
         UCollationElements *iter;
+        lineNo++;
         /* skip this line if it is empty or a comment or is a return value
         or start of some variable section */
         if(line[0] == 0 || line[0] == '#' || line[0] == '\n' ||
@@ -1361,6 +1283,15 @@ static void TestCEs() {
         }
 
         str = getCodePoints(line, codepoints);
+
+        /* these are 'fake' codepoints in the fractional UCA, and are used just 
+         * for positioning of indirect values. They should not go through this
+         * test.
+         */
+        if(*codepoints == 0xFDD0) {
+          continue;
+        }
+
         getCEs(str, ces, &status);
         if (U_FAILURE(status)) {
             log_err("Error in parsing collation elements in FractionalUCA.txt\n");
@@ -1376,6 +1307,12 @@ static void TestCEs() {
             if (ce == 0xFFFFFFFF) {
                 ce = 0;
             }
+            /* we now unconditionally reorder Thai/Lao prevowels, so this
+             * test would fail if we don't skip here.
+             */
+            if(UCOL_ISTHAIPREVOWEL(*codepoints) && ce == 0 && count == 0) {
+              continue;
+            }
             if (ce != ces[count] || U_FAILURE(status)) {
                 log_err("Collation elements in FractionalUCA.txt and iterators do not match!\n");
                 break;
@@ -1470,7 +1407,7 @@ static void TestDiscontiguos() {
             if (e == 0) {
                 e = u_strchr(s, 0);
             }
-            ucol_setText(resultiter, s, e - s, &status);
+            ucol_setText(resultiter, s, (int32_t)(e - s), &status);
             ce = ucol_next(resultiter, &status);
             if (U_FAILURE(status)) {
                 log_err("Error manipulating collation iterator\n");
@@ -1514,6 +1451,7 @@ static void TestCEBufferOverflow()
     coll = ucol_openRules(rule, u_strlen(rule), UCOL_OFF, UCOL_DEFAULT_STRENGTH, NULL,&status);
     if (U_FAILURE(status)) {
         log_err("Rule based collator not created for testing ce buffer overflow\n");
+        return;
     }
 
     /* 0xDCDC is a trail surrogate hence deemed unsafe by the heuristic
@@ -1649,8 +1587,13 @@ static UBool checkCEValidity(const UCollator *coll, const UChar *codepoints,
                    goto fail;
                }
            }
-           if ((primary != 0 && primary < primarymax) || (primary >= 0xFF00 && !isContinuation(ce))) {
-               log_err("UCA primary weight out of bounds\n");
+           if ((primary != 0 && primary < primarymax) 
+               || ((primary & 0xFF) == 0xFF) || (((primary>>8) & 0xFF) == 0xFF) 
+               || ((primary & 0xFF) && ((primary & 0xFF) <= 0x03)) 
+               || (((primary>>8) & 0xFF) && ((primary>>8) & 0xFF) <= 0x03)
+               || (primary >= 0xFE00 && !isContinuation(ce))) {
+               log_err("UCA primary weight out of bounds: %04X for string starting with %04X\n", 
+                   primary, codepoints[0]);
                goto fail;
            }
            /* case matching not done since data generated by ken */
@@ -1676,13 +1619,15 @@ static void TestCEValidity()
     /* testing UCA collation elements */
     UErrorCode  status      = U_ZERO_ERROR;
     /* en_US has no tailorings */
-    UCollator  *coll        = ucol_open("en_US", &status);
+    UCollator  *coll        = ucol_open("root", &status);
     /* tailored locales */
-    char        locale[][6] = {"fr_FR\0", "ko_KR\0", "sh_YU\0", "th_TH\0", "zh_CN\0"};
+    char        locale[][11] = {"fr_FR", "ko_KR", "sh_YU", "th_TH", "zh_CN", "zh__PINYIN"};
+    const char *loc;
     FileStream *file = getFractionalUCA();
-    char        line[300];
-    UChar       codepoints[5];
+    char        line[1024];
+    UChar       codepoints[10];
     int         count = 0;
+    int         maxCount = 0;
     UParseError parseError;
     if (U_FAILURE(status)) {
         log_err("en_US collator creation failed\n");
@@ -1717,7 +1662,12 @@ static void TestCEValidity()
 
     /* testing tailored collation elements */
     log_verbose("Testing tailored elements\n");
-    while (count < 5) {
+    if(QUICK) {
+        maxCount = sizeof(locale)/sizeof(locale[0]);
+    } else {
+        maxCount = uloc_countAvailable();
+    }
+    while (count < maxCount) {
         const UChar *rules = NULL,
                     *current = NULL;
         UChar *rulesCopy = NULL;
@@ -1735,10 +1685,21 @@ static void TestCEValidity()
         UColTokenParser src;
         uint32_t strength = 0;
         uint16_t specs = 0;
+        if(QUICK) {
+            loc = locale[count];
+        } else {
+            loc = uloc_getAvailable(count);
+            if(!hasCollationElements(loc)) {
+                count++;
+                continue;
+            }
+        }
 
-        coll      = ucol_open(locale[count], &status);
+        log_verbose("Testing CEs for %s\n", loc);
+
+        coll      = ucol_open(loc, &status);
         if (U_FAILURE(status)) {
-            log_err("%s collator creation failed\n", locale[count]);
+            log_err("%s collator creation failed\n", loc);
             return;
         }
 
@@ -1746,10 +1707,10 @@ static void TestCEValidity()
         rules = ucol_getRules(coll, &ruleLen);
 
         if (ruleLen > 0) {
-            rulesCopy = (UChar *)uprv_malloc((ruleLen +
+            rulesCopy = (UChar *)malloc((ruleLen +
                 UCOL_TOK_EXTRA_RULE_SPACE_SIZE) * sizeof(UChar));
             uprv_memcpy(rulesCopy, rules, ruleLen * sizeof(UChar));
-            src.source = src.current = rulesCopy;
+            src.current = src.source = rulesCopy;
             src.end = rulesCopy + ruleLen;
             src.extraCurrent = src.end;
             src.extraEnd = src.end + UCOL_TOK_EXTRA_RULE_SPACE_SIZE;
@@ -1765,12 +1726,12 @@ static void TestCEValidity()
               specs = src.parsedToken.flags;
 
                 startOfRules = FALSE;
-                uprv_memcpy(codepoints, rules + chOffset,
+                uprv_memcpy(codepoints, src.source + chOffset,
                                                        chLen * sizeof(UChar));
                 codepoints[chLen] = 0;
-                checkCEValidity(coll, codepoints, u_strlen(codepoints), 4, 85);
+                checkCEValidity(coll, codepoints, chLen, 4, 85);
             }
-            uprv_free(rulesCopy);
+            free(rulesCopy);
         }
 
         ucol_close(coll);
@@ -1857,10 +1818,10 @@ static void TestSortKeyValidity(void)
     /* en_US has no tailorings */
     UCollator  *coll        = ucol_open("en_US", &status);
     /* tailored locales */
-    char        locale[][6] = {"fr_FR\0", "ko_KR\0", "sh_YU\0", "th_TH\0", "zh_CN\0"};
+    char        locale[][6] = {"fr_FR", "ko_KR", "sh_YU", "th_TH", "zh_CN"};
     FileStream *file = getFractionalUCA();
-    char        line[300];
-    UChar       codepoints[5];
+    char        line[1024];
+    UChar       codepoints[10];
     int         count = 0;
     UParseError parseError;
     if (U_FAILURE(status)) {
@@ -1926,10 +1887,10 @@ static void TestSortKeyValidity(void)
         rules = ucol_getRules(coll, &ruleLen);
 
         if (ruleLen > 0) {
-            rulesCopy = (UChar *)uprv_malloc((ruleLen +
+            rulesCopy = (UChar *)malloc((ruleLen +
                 UCOL_TOK_EXTRA_RULE_SPACE_SIZE) * sizeof(UChar));
             uprv_memcpy(rulesCopy, rules, ruleLen * sizeof(UChar));
-            src.source = src.current = rulesCopy;
+            src.current = src.source = rulesCopy;
             src.end = rulesCopy + ruleLen;
             src.extraCurrent = src.end;
             src.extraEnd = src.end + UCOL_TOK_EXTRA_RULE_SPACE_SIZE;
@@ -1945,12 +1906,12 @@ static void TestSortKeyValidity(void)
                 specs = src.parsedToken.flags;
 
                 startOfRules = FALSE;
-                uprv_memcpy(codepoints, rules + chOffset,
+                uprv_memcpy(codepoints, src.source + chOffset,
                                                        chLen * sizeof(UChar));
                 codepoints[chLen] = 0;
-                checkSortKeyValidity(coll, codepoints, u_strlen(codepoints));
+                checkSortKeyValidity(coll, codepoints, chLen);
             }
-            uprv_free(rulesCopy);
+            free(rulesCopy);
         }
 
         ucol_close(coll);
@@ -1958,3 +1919,5 @@ static void TestSortKeyValidity(void)
     }
     T_FileStream_close(file);
 }
+
+#endif /* #if !UCONFIG_NO_COLLATION */
