@@ -1,7 +1,6 @@
 /********************************************************************
- * Copyright (c) 2001, 
- * International Business Machines Corporation and others. 
- * All Rights Reserved.
+ * Copyright (c) 2001-2006 International Business Machines 
+ * Corporation and others. All Rights Reserved.
  ********************************************************************
  * File USRCHDAT.H
  * Modification History:
@@ -9,20 +8,28 @@
  * synwee         July 31 2001    creation
  ********************************************************************/
 
+
+/*
+Note: This file is included by other C and C++ files. This file should not be directly compiled.
+*/
 #ifndef USRCHDAT_C
 #define USRCHDAT_C
 
 #include "unicode/ucol.h"
 
+#if !UCONFIG_NO_COLLATION
+
+U_CDECL_BEGIN
 struct SearchData {
     const char               *text;
     const char               *pattern;
     const char               *collator;
           UCollationStrength  strength;
     const char               *breaker;
-          int32_t         offset[32];
-          uint32_t            size[32];
+          int8_t              offset[32];
+          uint8_t             size[32];
 };
+U_CDECL_END
 
 typedef struct SearchData SearchData;
 
@@ -47,6 +54,7 @@ static const SearchData BASIC[] = {
     {"a\\u0300\\u0325", "\\u0300\\u0325", NULL, UCOL_TERTIARY, NULL, {1, -1}, 
     {2}},
     {"a\\u0300b", "\\u0300", NULL, UCOL_TERTIARY, NULL, {1, -1}, {1}},
+    {"\\u00c9", "e", NULL, UCOL_PRIMARY,  NULL, {0, -1}, {1}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -65,6 +73,7 @@ static const SearchData BREAKITERATOREXACT[] = {
      "characterbreaker", {1, 17, 30, -1}, {1, 1, 1}},
     {"testing that string ab\\u00e9cd does not match e", "e", NULL, 
      UCOL_TERTIARY, "characterbreaker", {1, 28, 41, -1}, {1, 1, 1}},
+    {"\\u00c9", "e", "fr", UCOL_PRIMARY,  "characterbreaker", {0, -1}, {1}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -81,6 +90,8 @@ static const SearchData STRENGTH[] = {
     {"A channel, another CHANNEL, more Channels, and one last channel...",
     "channel", "es", UCOL_PRIMARY, NULL, {2, 19, 33, 56, -1},
     {7, 7, 7, 7}},
+    {"\\u00c0 should match but not A", "A\\u0300", "en", UCOL_IDENTICAL, 
+    NULL, {0, -1}, {1, 0}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -199,6 +210,16 @@ static const SearchData SUPPLEMENTARY[] = {
     {"abc \\uD800\\uDC00 \\uD800\\uDC01 \\uD801\\uDC00 \\uD800\\uDC00abc abc\\uD800\\uDC00 \\uD800\\uD800\\uDC00 \\uD800\\uDC00\\uDC00", 
      "\\uD800\\uDC00", NULL, UCOL_TERTIARY, NULL, {4, 13, 22, 26, 29, -1}, 
     {2, 2, 2, 2, 2}},
+    {"and\\uD834\\uDDB9this sentence", "\\uD834\\uDDB9", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {2}},
+    {"and \\uD834\\uDDB9 this sentence", " \\uD834\\uDDB9 ", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and-\\uD834\\uDDB9-this sentence", "-\\uD834\\uDDB9-", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and,\\uD834\\uDDB9,this sentence", ",\\uD834\\uDDB9,", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and?\\uD834\\uDDB9?this sentence", "?\\uD834\\uDDB9?", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -264,6 +285,8 @@ static const SearchData BASICCANONICAL[] = {
     NULL, {0, -1}, {5}},
     {"a\\u0300\\u0325b\\u0300\\u0325c \\u0325b\\u0300 \\u0300b\\u0325", 
     "\\u0300b\\u0325", NULL, UCOL_TERTIARY, NULL, {1, 12, -1}, {5, 3}},
+    {"\\u00c4\\u0323", "A\\u0323\\u0308", NULL, UCOL_TERTIARY, NULL, {0, -1}, {2}},
+    {"\\u0308\\u0323", "\\u0323\\u0308", NULL, UCOL_TERTIARY, NULL, {0, -1}, {2}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -294,6 +317,7 @@ static const SearchData BREAKITERATORCANONICAL[] = {
      "characterbreaker", {1, 17, 30, -1}, {1, 1, 1}},
     {"testing that string ab\\u00e9cd does not match e", "e", NULL, 
      UCOL_TERTIARY, "characterbreaker", {1, 28, 41, -1}, {1, 1, 1}},
+    {"\\u00c9", "e", "fr", UCOL_PRIMARY,  "characterbreaker", {0, -1}, {1}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -420,6 +444,16 @@ static const SearchData SUPPLEMENTARYCANONICAL[] = {
     {"abc \\uD800\\uDC00 \\uD800\\uDC01 \\uD801\\uDC00 \\uD800\\uDC00abc abc\\uD800\\uDC00 \\uD800\\uD800\\uDC00 \\uD800\\uDC00\\uDC00", 
      "\\uD800\\uDC00", NULL, UCOL_TERTIARY, NULL, {4, 13, 22, 26, 29, -1}, 
     {2, 2, 2, 2, 2}},
+    {"and\\uD834\\uDDB9this sentence", "\\uD834\\uDDB9", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {2}},
+    {"and \\uD834\\uDDB9 this sentence", " \\uD834\\uDDB9 ", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and-\\uD834\\uDDB9-this sentence", "-\\uD834\\uDDB9-", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and,\\uD834\\uDDB9,this sentence", ",\\uD834\\uDDB9,", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
+    {"and?\\uD834\\uDDB9?this sentence", "?\\uD834\\uDDB9?", NULL, 
+     UCOL_TERTIARY, NULL, {3, -1}, {4}},
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
@@ -451,4 +485,6 @@ static const SearchData CONTRACTIONCANONICAL[] = {
     {NULL, NULL, NULL, UCOL_TERTIARY, NULL, {-1}, {0}}
 };
 
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif
