@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2005, International Business Machines Corporation and
+ * Copyright (c) 1997-2007, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -27,9 +27,14 @@ public:
     void TestBuddhist(void);
     void TestBuddhistFormat(void);
 
+    void TestTaiwan(void);
+
     void TestJapanese(void);
     void TestJapaneseFormat(void);
     void TestJapanese3860(void);
+    
+    void TestPersian(void);
+    void TestPersianFormat(void);
 
  protected:
     // Test a Gregorian-Like calendar
