@@ -1,7 +1,7 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation and
- * others. All Rights Reserved.
+ * Copyright (c) 1997-2007, International Business Machines
+ * Corporation and others. All Rights Reserved.
  ********************************************************************/
 
 /**
@@ -42,6 +42,11 @@
 #include "astrotst.h"       // AstroTest
 #include "incaltst.h"       // IntlCalendarTest
 #include "calcasts.h"       // CalendarCaseTest
+#include "tzrulets.h"       // TimeZoneRuleTest
+#include "dadrcal.h"        // DataDrivenCalendarTest
+#include "dadrfmt.h"        // DataDrivenFormatTest
+#include "dtptngts.h"       // IntlTestDateTimePatternGeneratorAPI
+
 
 #define TESTCLASS(id, TestClass)          \
     case id:                              \
@@ -101,6 +106,11 @@ void IntlTestFormat::runIndexedTest( int32_t index, UBool exec, const char* &nam
         TESTCLASS(26,IntlCalendarTest);
         TESTCLASS(27,AstroTest);
         TESTCLASS(28,CalendarCaseTest);
+        TESTCLASS(29,TimeZoneRuleTest);
+        TESTCLASS(30,DataDrivenCalendarTest);
+        TESTCLASS(31,DataDrivenFormatTest);
+        TESTCLASS(32,IntlTestDateTimePatternGeneratorAPI);
+
 
         default: name = ""; break; //needed to end loop
     }
