@@ -1,9 +1,14 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2000, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2007, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "itrbnf.h"
 
 #include "unicode/umachine.h"
@@ -12,6 +17,10 @@
 #include "unicode/coleitr.h"
 #include "unicode/ures.h"
 #include "unicode/ustring.h"
+#include "unicode/decimfmt.h"
+#include "unicode/udata.h"
+#include "testutil.h"
+
 //#include "llong.h"
 
 #include <string.h>
@@ -38,29 +47,89 @@ void IntlTestRBNF::runIndexedTest(int32_t index, UBool exec, const char* &name,
     if (exec) logln("TestSuite RuleBasedNumberFormat");
     switch (index) {
 #if U_HAVE_RBNF
-      TESTCASE(0, TestEnglishSpellout);
-      TESTCASE(1, TestOrdinalAbbreviations);
-      TESTCASE(2, TestDurations);
-      TESTCASE(3, TestSpanishSpellout);
-      TESTCASE(4, TestFrenchSpellout);
-      TESTCASE(5, TestSwissFrenchSpellout);
-      TESTCASE(6, TestItalianSpellout);
-      TESTCASE(7, TestGermanSpellout);
-      TESTCASE(8, TestThaiSpellout);
-      TESTCASE(9, TestAPI);
-      TESTCASE(10, TestFractionalRuleSet);
-      // TESTCASE(11, TestLLong);
+        TESTCASE(0, TestEnglishSpellout);
+        TESTCASE(1, TestOrdinalAbbreviations);
+        TESTCASE(2, TestDurations);
+        TESTCASE(3, TestSpanishSpellout);
+        TESTCASE(4, TestFrenchSpellout);
+        TESTCASE(5, TestSwissFrenchSpellout);
+        TESTCASE(6, TestItalianSpellout);
+        TESTCASE(7, TestGermanSpellout);
+        TESTCASE(8, TestThaiSpellout);
+        TESTCASE(9, TestAPI);
+        TESTCASE(10, TestFractionalRuleSet);
+        TESTCASE(11, TestSwedishSpellout);
+        TESTCASE(12, TestBelgianFrenchSpellout);
+        TESTCASE(13, TestSmallValues);
+        TESTCASE(14, TestLocalizations);
+        TESTCASE(15, TestAllLocales);
+        TESTCASE(16, TestHebrewFraction);
+        TESTCASE(17, TestPortugueseSpellout);
 #else
-      TESTCASE(0, TestRBNFDisabled);
+        TESTCASE(0, TestRBNFDisabled);
 #endif
     default:
-      name = "";
-      break;
+        name = "";
+        break;
     }
 }
 
 #if U_HAVE_RBNF
 
+void IntlTestRBNF::TestHebrewFraction() {
+    // this is the expected output for 123.45, with no '<' in it.
+    UChar text1[] = { 
+        0x05de, 0x05d0, 0x05d4, 0x0020, 
+        0x05e2, 0x05e9, 0x05e8, 0x05d9, 0x05dd, 0x0020,
+        0x05d5, 0x05e9, 0x05dc, 0x05d5, 0x05e9, 0x0020, 
+        0x05e0, 0x05e7, 0x05d5, 0x05d3, 0x05d4, 0x0020,
+        0x05d0, 0x05e8, 0x05d1, 0x05e2, 0x05d9, 0x05dd, 0x0020,
+        0x05d5, 0x05d7, 0x05de, 0x05e9, 0x0000,
+    };
+    UChar text2[] = { 
+        0x05DE, 0x05D0, 0x05D4, 0x0020, 
+        0x05E2, 0x05E9, 0x05E8, 0x05D9, 0x05DD, 0x0020, 
+        0x05D5, 0x05E9, 0x05DC, 0x05D5, 0x05E9, 0x0020, 
+        0x05E0, 0x05E7, 0x05D5, 0x05D3, 0x05D4, 0x0020, 
+        0x05D0, 0x05E4, 0x05E1, 0x0020, 
+        0x05D0, 0x05E4, 0x05E1, 0x0020, 
+        0x05D0, 0x05E8, 0x05D1, 0x05E2, 0x05D9, 0x05DD, 0x0020, 
+        0x05D5, 0x05D7, 0x05DE, 0x05E9, 0x0000,
+    };
+    UErrorCode status = U_ZERO_ERROR;
+    RuleBasedNumberFormat* formatter = new RuleBasedNumberFormat(URBNF_SPELLOUT, "he_IL", status);
+    UnicodeString result;
+    Formattable parseResult;
+    ParsePosition pp(0);
+    {
+        UnicodeString expected(text1);
+        formatter->format(123.45, result);
+        if (result != expected) {
+            errln((UnicodeString)"expected '" + TestUtility::hex(expected) + "'\nbut got: '" + TestUtility::hex(result) + "'");
+        } else {
+            formatter->parse(result, parseResult, pp);
+            if (parseResult.getDouble() != 123.45) {
+                errln("expected 123.45 but got: %g", parseResult.getDouble());
+            }
+        }
+    }
+    {
+        UnicodeString expected(text2);
+        result.remove();
+        formatter->format(123.0045, result);
+        if (result != expected) {
+            errln((UnicodeString)"expected '" + TestUtility::hex(expected) + "'\nbut got: '" + TestUtility::hex(result) + "'");
+        } else {
+            pp.setIndex(0);
+            formatter->parse(result, parseResult, pp);
+            if (parseResult.getDouble() != 123.0045) {
+                errln("expected 123.0045 but got: %g", parseResult.getDouble());
+            }
+        }
+    }
+    delete formatter;
+}
+
 void 
 IntlTestRBNF::TestAPI() {
   // This test goes through the APIs that were not tested before. 
@@ -98,7 +167,7 @@ IntlTestRBNF::TestAPI() {
   // test rule constructor
   {
     logln("Testing rule constructor");
-    UResourceBundle *en = ures_open(NULL, "en", &status);
+    UResourceBundle *en = ures_open(U_ICUDATA_NAME U_TREE_SEPARATOR_STRING "rbnf", "en", &status);
     if(U_FAILURE(status)) {
       errln("Unable to access resource bundle with data!");
     } else {
@@ -112,6 +181,12 @@ IntlTestRBNF::TestAPI() {
         if(!(ruleCtorResult == *formatter)) {
           errln("Formatter constructed from the original rules should be semantically equivalent to the original!");
         }
+        
+        // Jitterbug 4452, for coverage
+        RuleBasedNumberFormat nf(spelloutRules, (UnicodeString)"", Locale::getUS(), perror, status);
+        if(!(nf == *formatter)) {
+          errln("Formatter constructed from the original rules should be semantically equivalent to the original!");
+        }
       }
       ures_close(en);
     }
@@ -138,6 +213,7 @@ IntlTestRBNF::TestAPI() {
     }
   }
 
+#if !UCONFIG_NO_COLLATION
   // test ruleset names
   {
     logln("Testing getNumberOfRuleSetNames, getRuleSetName and format using rule set names");
@@ -198,6 +274,45 @@ IntlTestRBNF::TestAPI() {
     }   
     status = U_ZERO_ERROR;
   }
+#endif
+
+  // test API
+  UnicodeString expected("four point five","");
+  logln("Testing format(double)");
+  UnicodeString result;
+  formatter->format(4.5,result);
+  if(result != expected) {
+      errln("Formatted 4.5, expected " + expected + " got " + result);
+  } else {
+      logln("Formatted 4.5, expected " + expected + " got " + result);
+  }
+  result.remove();
+  expected = "four";
+  formatter->format((int32_t)4,result);
+  if(result != expected) {
+      errln("Formatted 4, expected " + expected + " got " + result);
+  } else {
+      logln("Formatted 4, expected " + expected + " got " + result);
+  }
+
+  result.remove();
+  FieldPosition pos;
+  formatter->format((int64_t)4, result, pos, status = U_ZERO_ERROR);
+  if(result != expected) {
+      errln("Formatted 4 int64_t, expected " + expected + " got " + result);
+  } else {
+      logln("Formatted 4 int64_t, expected " + expected + " got " + result);
+  }
+
+  //Jitterbug 4452, for coverage
+  result.remove();
+  FieldPosition pos2;
+  formatter->format((int64_t)4, formatter->getRuleSetName(0), result, pos2, status = U_ZERO_ERROR);
+  if(result != expected) {
+      errln("Formatted 4 int64_t, expected " + expected + " got " + result);
+  } else {
+      logln("Formatted 4 int64_t, expected " + expected + " got " + result);
+  }
 
   // clean up
   logln("Cleaning up");
@@ -237,13 +352,33 @@ void IntlTestRBNF::TestFractionalRuleSet()
         "    9: <0</9;\n"
         "   10: <0</10;\n");
 
+    // mondo hack
+    int len = fracRules.length();
+    int change = 2;
+    for (int i = 0; i < len; ++i) {
+        UChar ch = fracRules.charAt(i);
+        if (ch == '\n') {
+            change = 2; // change ok
+        } else if (ch == ':') {
+            change = 1; // change, but once we hit a non-space char, don't change
+        } else if (ch == ' ') {
+            if (change != 0) {
+                fracRules.setCharAt(i, (UChar)0x200e);
+            }
+        } else {
+            if (change == 1) {
+                change = 0;
+            }
+        }
+    }
+
     UErrorCode status = U_ZERO_ERROR;
     UParseError perror;
     RuleBasedNumberFormat formatter(fracRules, Locale::getEnglish(), perror, status);
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "0", "0" },
             { ".1", "1/10" },
             { ".11", "1/9" },
@@ -912,7 +1047,7 @@ IntlTestRBNF::TestEnglishSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "one" },
             { "2", "two" },
             { "15", "fifteen" },
@@ -938,6 +1073,7 @@ IntlTestRBNF::TestEnglishSpellout()
 
         doTest(formatter, testData, TRUE);
 
+#if !UCONFIG_NO_COLLATION
         formatter->setLenient(TRUE);
         static const char* lpTestData[][2] = {
             { "fifty-7", "57" },
@@ -949,6 +1085,7 @@ IntlTestRBNF::TestEnglishSpellout()
             { NULL, NULL}
         };
         doLenientParseTest(formatter, lpTestData);
+#endif
     }
     delete formatter;
 }
@@ -963,7 +1100,7 @@ IntlTestRBNF::TestOrdinalAbbreviations()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "1st" },
             { "2", "2nd" },
             { "3", "3rd" },
@@ -999,7 +1136,7 @@ IntlTestRBNF::TestDurations()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "3,600", "1:00:00" },     //move me and I fail
             { "0", "0 sec." },
             { "1", "1 sec." },
@@ -1016,12 +1153,14 @@ IntlTestRBNF::TestDurations()
         
         doTest(formatter, testData, TRUE);
         
+#if !UCONFIG_NO_COLLATION
         formatter->setLenient(TRUE);
         static const char* lpTestData[][2] = {
             { "2-51-33", "10,293" },
             { NULL, NULL}
         };
         doLenientParseTest(formatter, lpTestData);
+#endif
     }
     delete formatter;
 }
@@ -1036,7 +1175,7 @@ IntlTestRBNF::TestSpanishSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "uno" },
             { "6", "seis" },
             { "16", "diecis\\u00e9is" },
@@ -1076,7 +1215,7 @@ IntlTestRBNF::TestFrenchSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "un" },
             { "15", "quinze" },
             { "20", "vingt" },
@@ -1108,6 +1247,7 @@ IntlTestRBNF::TestFrenchSpellout()
         
         doTest(formatter, testData, TRUE);
         
+#if !UCONFIG_NO_COLLATION
         formatter->setLenient(TRUE);
         static const char* lpTestData[][2] = {
             { "trente-un", "31" },
@@ -1115,10 +1255,41 @@ IntlTestRBNF::TestFrenchSpellout()
             { NULL, NULL}
         };
         doLenientParseTest(formatter, lpTestData);
+#endif
     }
     delete formatter;
 }
 
+static const char* const swissFrenchTestData[][2] = {
+    { "1", "un" },
+    { "15", "quinze" },
+    { "20", "vingt" },
+    { "21", "vingt-et-un" },
+    { "23", "vingt-trois" },
+    { "62", "soixante-deux" },
+    { "70", "septante" },
+    { "71", "septante-et-un" },
+    { "73", "septante-trois" },
+    { "80", "huitante" },
+    { "88", "huitante-huit" },
+    { "100", "cent" },
+    { "106", "cent six" },
+    { "127", "cent vingt-sept" },
+    { "200", "deux cents" },
+    { "579", "cinq cents septante-neuf" },
+    { "1,000", "mille" },
+    { "1,123", "onze cents vingt-trois" },
+    { "1,594", "mille cinq cents nonante-quatre" },
+    { "2,000", "deux mille" },
+    { "3,004", "trois mille quatre" },
+    { "4,567", "quatre mille cinq cents soixante-sept" },
+    { "15,943", "quinze mille neuf cents quarante-trois" },
+    { "2,345,678", "deux million trois cents quarante-cinq mille six cents septante-huit" },
+    { "-36", "moins trente-six" },
+    { "234.567", "deux cents trente-quatre virgule cinq six sept" },
+    { NULL, NULL}
+};
+
 void 
 IntlTestRBNF::TestSwissFrenchSpellout() 
 {
@@ -1129,37 +1300,24 @@ IntlTestRBNF::TestSwissFrenchSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
-            { "1", "un" },
-            { "15", "quinze" },
-            { "20", "vingt" },
-            { "21", "vingt-et-un" },
-            { "23", "vingt-trois" },
-            { "62", "soixante-deux" },
-            { "70", "septante" },
-            { "71", "septante-et-un" },
-            { "73", "septante-trois" },
-            { "80", "octante" },
-            { "88", "octante-huit" },
-            { "100", "cent" },
-            { "106", "cent six" },
-            { "127", "cent vingt-sept" },
-            { "200", "deux cents" },
-            { "579", "cinq cents septante-neuf" },
-            { "1,000", "mille" },
-            { "1,123", "onze cents vingt-trois" },
-            { "1,594", "mille cinq cents nonante-quatre" },
-            { "2,000", "deux mille" },
-            { "3,004", "trois mille quatre" },
-            { "4,567", "quatre mille cinq cents soixante-sept" },
-            { "15,943", "quinze mille neuf cents quarante-trois" },
-            { "2,345,678", "deux million trois cents quarante-cinq mille six cents septante-huit" },
-            { "-36", "moins trente-six" },
-            { "234.567", "deux cents trente-quatre virgule cinq six sept" },
-            { NULL, NULL}
-        };
-        
-        doTest(formatter, testData, TRUE);
+        doTest(formatter, swissFrenchTestData, TRUE);
+    }
+    delete formatter;
+}
+
+void 
+IntlTestRBNF::TestBelgianFrenchSpellout() 
+{
+    UErrorCode status = U_ZERO_ERROR;
+    RuleBasedNumberFormat* formatter
+        = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("fr", "BE", ""), status);
+    
+    if (U_FAILURE(status)) {
+        errln("rbnf status: 0x%x (%s)\n", status, u_errorName(status));
+        errln("FAIL: could not construct formatter");
+    } else {
+        // Belgian french should match Swiss french.
+        doTest(formatter, swissFrenchTestData, TRUE);
     }
     delete formatter;
 }
@@ -1174,12 +1332,12 @@ IntlTestRBNF::TestItalianSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "uno" },
             { "15", "quindici" },
             { "20", "venti" },
-            { "23", "ventitre" },
-            { "73", "settantatre" },
+            { "23", "ventitr\\u00E9" },
+            { "73", "settantatr\\u00E9" },
             { "88", "ottantotto" },
             { "100", "cento" },
             { "106", "centosei" },
@@ -1192,9 +1350,9 @@ IntlTestRBNF::TestItalianSpellout()
             { "2,000", "duemila" },
             { "3,004", "tremilaquattro" },
             { "4,567", "quattromilacinquecentosessantasette" },
-            { "15,943", "quindicimilanovecentoquarantatre" },
-            { "-36", "meno trentisei" },
-            { "234.567", "duecentotrentiquattro virgola cinque sei sette" },
+            { "15,943", "quindicimilanovecentoquarantatr\\u00E9" },
+            { "-36", "meno trentasei" },
+            { "234.567", "duecentotrentaquattro virgola cinque sei sette" },
             { NULL, NULL}
         };
         
@@ -1203,6 +1361,44 @@ IntlTestRBNF::TestItalianSpellout()
     delete formatter;
 }
 
+void 
+IntlTestRBNF::TestPortugueseSpellout() 
+{
+    UErrorCode status = U_ZERO_ERROR;
+    RuleBasedNumberFormat* formatter
+        = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("pt","BR",""), status);
+
+    if (U_FAILURE(status)) {
+        errln("FAIL: could not construct formatter");
+    } else {
+        static const char* const testData[][2] = {
+            { "1", "um" },
+            { "15", "quinze" },
+            { "20", "vinte" },
+            { "23", "vinte e tr\\u00EAs" },
+            { "73", "setenta e tr\\u00EAs" },
+            { "88", "oitenta e oito" },
+            { "100", "cem" },
+            { "106", "cento e seis" },
+            { "108", "cento e oito" },
+            { "127", "cento e vinte e sete" },
+            { "181", "cento e oitenta e um" },
+            { "200", "duzcentos" },
+            { "579", "quinhentos e setenta e nove" },
+            { "1,000", "mil" },
+            { "2,000", "dois mil" },
+            { "3,004", "tr\\u00EAs mil e quatro" },
+            { "4,567", "quatro mil quinhentos e sessenta e sete" },
+            { "15,943", "quinze mil novecentos e quarenta e tr\\u00EAs" },
+            { "-36", "menos trinta e seis" },
+            { "234.567", "duzcentos e trinta e quatro ponto cinco seis sete" },
+            { NULL, NULL}
+        };
+        
+        doTest(formatter, testData, TRUE);
+    }
+    delete formatter;
+}
 void 
 IntlTestRBNF::TestGermanSpellout() 
 {
@@ -1213,7 +1409,7 @@ IntlTestRBNF::TestGermanSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "1", "eins" },
             { "15", "f\\u00fcnfzehn" },
             { "20", "zwanzig" },
@@ -1236,12 +1432,14 @@ IntlTestRBNF::TestGermanSpellout()
         
         doTest(formatter, testData, TRUE);
         
+#if !UCONFIG_NO_COLLATION
         formatter->setLenient(TRUE);
         static const char* lpTestData[][2] = {
             { "ein Tausend sechs Hundert fuenfunddreissig", "1,635" },
             { NULL, NULL}
         };
         doLenientParseTest(formatter, lpTestData);
+#endif
     }
     delete formatter;
 }
@@ -1256,7 +1454,7 @@ IntlTestRBNF::TestThaiSpellout()
     if (U_FAILURE(status)) {
         errln("FAIL: could not construct formatter");
     } else {
-        static const char* testData[][2] = {
+        static const char* const testData[][2] = {
             { "0", "\\u0e28\\u0e39\\u0e19\\u0e22\\u0e4c" },
             { "1", "\\u0e2b\\u0e19\\u0e36\\u0e48\\u0e07" },
             { "10", "\\u0e2a\\u0e34\\u0e1a" },
@@ -1272,13 +1470,311 @@ IntlTestRBNF::TestThaiSpellout()
     delete formatter;
 }
 
+void
+IntlTestRBNF::TestSwedishSpellout()
+{
+    UErrorCode status = U_ZERO_ERROR;
+    RuleBasedNumberFormat* formatter
+        = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("sv"), status);
+
+    if (U_FAILURE(status)) {
+        errln("FAIL: could not construct formatter");
+    } else {
+        static const char* testDataDefault[][2] = {
+            { "101", "etthundra\\u00aden" },
+            { "123", "etthundra\\u00adtjugotre" },
+            { "1,001", "ettusen en" },
+            { "1,100", "ettusen etthundra" },
+            { "1,101", "ettusen etthundra\\u00aden" },
+            { "1,234", "ettusen tv\\u00e5hundra\\u00adtrettiofyra" },
+            { "10,001", "tio\\u00adtusen en" },
+            { "11,000", "elva\\u00adtusen" },
+            { "12,000", "tolv\\u00adtusen" },
+            { "20,000", "tjugo\\u00adtusen" },
+            { "21,000", "tjugoen\\u00adtusen" },
+            { "21,001", "tjugoen\\u00adtusen en" },
+            { "200,000", "tv\\u00e5hundra\\u00adtusen" },
+            { "201,000", "tv\\u00e5hundra\\u00aden\\u00adtusen" },
+            { "200,200", "tv\\u00e5hundra\\u00adtusen tv\\u00e5hundra" },
+            { "2,002,000", "tv\\u00e5 miljoner tv\\u00e5\\u00adtusen" },
+            { "12,345,678", "tolv miljoner trehundra\\u00adfyrtiofem\\u00adtusen sexhundra\\u00adsjuttio\\u00e5tta" },
+            { "123,456.789", "etthundra\\u00adtjugotre\\u00adtusen fyrahundra\\u00adfemtiosex komma sju \\u00e5tta nio" },
+            { "-12,345.678", "minus tolv\\u00adtusen trehundra\\u00adfyrtiofem komma sex sju \\u00e5tta" },
+            { NULL, NULL }
+        };
+        doTest(formatter, testDataDefault, TRUE);
+
+        static const char* testDataNeutrum[][2] = {
+            { "101", "etthundra\\u00adett" },
+            { "1,001", "ettusen ett" },
+            { "1,101", "ettusen etthundra\\u00adett" },
+            { "10,001", "tio\\u00adtusen ett" },
+            { "21,001", "tjugoen\\u00adtusen ett" },
+            { NULL, NULL }
+        };
+
+        formatter->setDefaultRuleSet("%neutrum", status);
+        if (U_SUCCESS(status)) {
+            logln("testing neutrum rules");
+            doTest(formatter, testDataNeutrum, TRUE);
+        }
+        else {
+            errln("Can't test neutrum rules");
+        }
+
+        static const char* testDataYear[][2] = {
+            { "101", "etthundra\\u00adett" },
+            { "900", "niohundra" },
+            { "1,001", "tiohundra\\u00adett" },
+            { "1,100", "elvahundra" },
+            { "1,101", "elvahundra\\u00adett" },
+            { "1,234", "tolvhundra\\u00adtrettiofyra" },
+            { "2,001", "tjugohundra\\u00adett" },
+            { "10,001", "tio\\u00adtusen ett" },
+            { NULL, NULL }
+        };
+
+        formatter->setDefaultRuleSet("%year", status);
+        if (U_SUCCESS(status)) {
+            logln("testing year rules");
+            doTest(formatter, testDataYear, TRUE);
+        }
+        else {
+            errln("Can't test year rules");
+        }
+
+    }
+    delete formatter;
+}
+
+void
+IntlTestRBNF::TestSmallValues()
+{
+    UErrorCode status = U_ZERO_ERROR;
+    RuleBasedNumberFormat* formatter
+        = new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale("en_US"), status);
+
+    if (U_FAILURE(status)) {
+        errln("FAIL: could not construct formatter");
+    } else {
+        static const char* const testDataDefault[][2] = {
+        { "0.001", "zero point zero zero one" },
+        { "0.0001", "zero point zero zero zero one" },
+        { "0.00001", "zero point zero zero zero zero one" },
+        { "0.000001", "zero point zero zero zero zero zero one" },
+        { "0.0000001", "zero point zero zero zero zero zero zero one" },
+        { "0.00000001", "zero point zero zero zero zero zero zero zero one" },
+        { "0.000000001", "zero point zero zero zero zero zero zero zero zero one" },
+        { "0.0000000001", "zero point zero zero zero zero zero zero zero zero zero one" },
+        { "0.00000000001", "zero point zero zero zero zero zero zero zero zero zero zero one" },
+        { "0.000000000001", "zero point zero zero zero zero zero zero zero zero zero zero zero one" },
+        { "0.0000000000001", "zero point zero zero zero zero zero zero zero zero zero zero zero zero one" },
+        { "0.00000000000001", "zero point zero zero zero zero zero zero zero zero zero zero zero zero zero one" },
+        { "0.000000000000001", "zero point zero zero zero zero zero zero zero zero zero zero zero zero zero zero one" },
+        { "10,000,000.001", "ten million point zero zero one" },
+        { "10,000,000.0001", "ten million point zero zero zero one" },
+        { "10,000,000.00001", "ten million point zero zero zero zero one" },
+        { "10,000,000.000001", "ten million point zero zero zero zero zero one" },
+        { "10,000,000.0000001", "ten million point zero zero zero zero zero zero one" },
+//        { "10,000,000.00000001", "ten million point zero zero zero zero zero zero zero one" },
+//        { "10,000,000.000000002", "ten million point zero zero zero zero zero zero zero zero two" },
+        { "10,000,000", "ten million" },
+//        { "1,234,567,890.0987654", "one billion, two hundred and thirty-four million, five hundred and sixty-seven thousand, eight hundred and ninety point zero nine eight seven six five four" },
+//        { "123,456,789.9876543", "one hundred and twenty-three million, four hundred and fifty-six thousand, seven hundred and eighty-nine point nine eight seven six five four three" },
+//        { "12,345,678.87654321", "twelve million, three hundred and forty-five thousand, six hundred and seventy-eight point eight seven six five four three two one" },
+        { "1,234,567.7654321", "one million, two hundred and thirty-four thousand, five hundred and sixty-seven point seven six five four three two one" },
+        { "123,456.654321", "one hundred and twenty-three thousand, four hundred and fifty-six point six five four three two one" },
+        { "12,345.54321", "twelve thousand three hundred and forty-five point five four three two one" },
+        { "1,234.4321", "one thousand two hundred and thirty-four point four three two one" },
+        { "123.321", "one hundred and twenty-three point three two one" },
+        { "0.0000000011754944", "zero point zero zero zero zero zero zero zero zero one one seven five four nine four four" },
+        { "0.000001175494351", "zero point zero zero zero zero zero one one seven five four nine four three five one" },
+        { NULL, NULL }
+        };
+
+        doTest(formatter, testDataDefault, TRUE);
+
+        delete formatter;
+    }
+}
+
 void 
-IntlTestRBNF::doTest(RuleBasedNumberFormat* formatter, const char* testData[][2], UBool testParsing) 
+IntlTestRBNF::TestLocalizations(void)
+{
+    int i;
+    UnicodeString rules("%main:0:no;1:some;100:a lot;1000:tons;\n"
+        "%other:0:nada;1:yah, some;100:plenty;1000:more'n you'll ever need");
+
+    UErrorCode status = U_ZERO_ERROR;
+    UParseError perror;
+    RuleBasedNumberFormat formatter(rules, perror, status);
+    if (U_FAILURE(status)) {
+        errln("FAIL: could not construct formatter");           
+    } else {
+        {
+            static const char* const testData[][2] = {
+                { "0", "nada" },
+                { "5", "yah, some" },
+                { "423", "plenty" },
+                { "12345", "more'n you'll ever need" },
+                { NULL, NULL }
+            };
+            doTest(&formatter, testData, FALSE);
+        }
+
+        {
+            UnicodeString loc("<<%main, %other>,<en, Main, Other>,<fr, leMain, leOther>,<de, 'das Main', 'etwas anderes'>>");
+            static const char* const testData[][2] = {
+                { "0", "no" },
+                { "5", "some" },
+                { "423", "a lot" },
+                { "12345", "tons" },
+                { NULL, NULL }
+            };
+            RuleBasedNumberFormat formatter0(rules, loc, perror, status);
+            if (U_FAILURE(status)) {
+                errln("failed to build second formatter");
+            } else {
+                doTest(&formatter0, testData, FALSE);
+
+                {
+                // exercise localization info
+                    Locale locale0("en__VALLEY@turkey=gobblegobble");
+                    Locale locale1("de_DE_FOO");
+                    Locale locale2("ja_JP");
+                    UnicodeString name = formatter0.getRuleSetName(0);
+                    if ( formatter0.getRuleSetDisplayName(0, locale0) == "Main"
+                      && formatter0.getRuleSetDisplayName(0, locale1) == "das Main"
+                      && formatter0.getRuleSetDisplayName(0, locale2) == "%main"
+                      && formatter0.getRuleSetDisplayName(name, locale0) == "Main"
+                      && formatter0.getRuleSetDisplayName(name, locale1) == "das Main"
+                      && formatter0.getRuleSetDisplayName(name, locale2) == "%main"){
+                          logln("getRuleSetDisplayName tested");
+                    }else {
+                        errln("failed to getRuleSetDisplayName");
+                    }
+                }
+
+                for (i = 0; i < formatter0.getNumberOfRuleSetDisplayNameLocales(); ++i) {
+                    Locale locale = formatter0.getRuleSetDisplayNameLocale(i, status);
+                    if (U_SUCCESS(status)) {
+                        for (int j = 0; j < formatter0.getNumberOfRuleSetNames(); ++j) {
+                            UnicodeString name = formatter0.getRuleSetName(j);
+                            UnicodeString lname = formatter0.getRuleSetDisplayName(j, locale);
+                            UnicodeString msg = locale.getName();
+                            msg.append(": ");
+                            msg.append(name);
+                            msg.append(" = ");
+                            msg.append(lname);
+                            logln(msg);
+                        }
+                    }
+                }
+            }
+        }
+
+        {
+            static const char* goodLocs[] = {
+                "", // zero-length ok, same as providing no localization data
+                "<<>>", // no public rule sets ok
+                "<<%main>>", // no localizations ok
+                "<<%main,>,<en, Main,>>", // comma before close angle ok
+                "<<%main>,<en, ',<>\" '>>", // quotes everything until next quote
+                "<<%main>,<'en', \"it's ok\">>", // double quotes work too
+                "  \n <\n  <\n  %main\n  >\n  , \t <\t   en\t  ,  \tfoo \t\t > \n\n >  \n ", // rule whitespace ok
+           }; 
+            int32_t goodLocsLen = sizeof(goodLocs)/sizeof(goodLocs[0]);
+
+            static const char* badLocs[] = {
+                " ", // non-zero length
+                "<>", // empty array
+                "<", // unclosed outer array
+                "<<", // unclosed inner array
+                "<<,>>", // unexpected comma
+                "<<''>>", // empty string
+                "  x<<%main>>", // first non space char not open angle bracket
+                "<%main>", // missing inner array
+                "<<%main %other>>", // elements missing separating commma (spaces must be quoted)
+                "<<%main><en, Main>>", // arrays missing separating comma
+                "<<%main>,<en, main, foo>>", // too many elements in locale data
+                "<<%main>,<en>>", // too few elements in locale data
+                "<<<%main>>>", // unexpected open angle
+                "<<%main<>>>", // unexpected open angle
+                "<<%main, %other>,<en,,>>", // implicit empty strings
+                "<<%main>,<en,''>>", // empty string
+                "<<%main>, < en, '>>", // unterminated quote
+                "<<%main>, < en, \"<>>", // unterminated quote
+                "<<%main\">>", // quote in string
+                "<<%main'>>", // quote in string
+                "<<%main<>>", // open angle in string
+                "<<%main>> x", // extra non-space text at end
+
+            };
+            int32_t badLocsLen = sizeof(badLocs)/sizeof(badLocs[0]);
+
+            for (i = 0; i < goodLocsLen; ++i) {
+                logln("[%d] '%s'", i, goodLocs[i]);
+                UErrorCode status = U_ZERO_ERROR;
+                UnicodeString loc(goodLocs[i]);
+                RuleBasedNumberFormat fmt(rules, loc, perror, status);
+                if (U_FAILURE(status)) {
+                    errln("Failed parse of good localization string: '%s'", goodLocs[i]);
+                }
+            }
+
+            for (i = 0; i < badLocsLen; ++i) {
+                logln("[%d] '%s'", i, badLocs[i]);
+                UErrorCode status = U_ZERO_ERROR;
+                UnicodeString loc(badLocs[i]);
+                RuleBasedNumberFormat fmt(rules, loc, perror, status);
+                if (U_SUCCESS(status)) {
+                    errln("Successful parse of bad localization string: '%s'", badLocs[i]);
+                }
+            }
+        }
+    }
+}
+
+void
+IntlTestRBNF::TestAllLocales()
+{
+  const char* names[] = {
+    " (spellout) ",
+    " (ordinal)  ",
+    " (duration) "
+  };
+  int32_t count = 0;
+  const Locale* locales = Locale::getAvailableLocales(count);
+  for (int i = 0; i < count; ++i) {
+    const Locale* loc = &locales[i];
+    for (int j = 0; j < 3; ++j) {
+      UErrorCode status = U_ZERO_ERROR;
+      RuleBasedNumberFormat* f = new RuleBasedNumberFormat((URBNFRuleSetTag)j, *loc, status);
+      if (U_SUCCESS(status)) {
+        double n = 45.678;
+        UnicodeString str;
+        f->format(n, str);
+        delete f;
+
+        logln(UnicodeString(loc->getName()) + UnicodeString(names[j])
+            + UnicodeString("success: 45.678 -> ") + str);
+      } else {
+        errln(UnicodeString(loc->getName()) + UnicodeString(names[j])
+            + UnicodeString("ERROR could not instantiate -> ") + UnicodeString(u_errorName(status)));
+      }
+    }
+  }
+}
+
+void 
+IntlTestRBNF::doTest(RuleBasedNumberFormat* formatter, const char* const testData[][2], UBool testParsing) 
 {
   // man, error reporting would be easier with printf-style syntax for unicode string and formattable
 
     UErrorCode status = U_ZERO_ERROR;
-    NumberFormat* decFmt = NumberFormat::createInstance(Locale::getUS(), status);
+    DecimalFormatSymbols dfs("en", status);
+    // NumberFormat* decFmt = NumberFormat::createInstance(Locale::getUS(), status);
+    DecimalFormat decFmt("#,###.################", dfs, status);
     if (U_FAILURE(status)) {
         errln("FAIL: could not create NumberFormat");
     } else {
@@ -1286,8 +1782,9 @@ IntlTestRBNF::doTest(RuleBasedNumberFormat* formatter, const char* testData[][2]
             const char* numString = testData[i][0];
             const char* expectedWords = testData[i][1];
 
+            log("[%i] %s = ", i, numString);
             Formattable expectedNumber;
-            decFmt->parse(numString, expectedNumber, status);
+            decFmt.parse(numString, expectedNumber, status);
             if (U_FAILURE(status)) {
                 errln("FAIL: decFmt could not parse %s", numString);
                 break;
@@ -1297,49 +1794,49 @@ IntlTestRBNF::doTest(RuleBasedNumberFormat* formatter, const char* testData[][2]
                 formatter->format(expectedNumber, actualString/* , pos*/, status);
                 if (U_FAILURE(status)) {
                     UnicodeString msg = "Fail: formatter could not format ";
-                    decFmt->format(expectedNumber, msg, status);
+                    decFmt.format(expectedNumber, msg, status);
                     errln(msg);
                     break;
                 } else {
                     UnicodeString expectedString = UnicodeString(expectedWords).unescape();
                     if (actualString != expectedString) {
                         UnicodeString msg = "FAIL: check failed for ";
-                        decFmt->format(expectedNumber, msg, status);
+                        decFmt.format(expectedNumber, msg, status);
                         msg.append(", expected ");
                         msg.append(expectedString);
                         msg.append(" but got ");
                         msg.append(actualString);
                         errln(msg);
                         break;
-                    } else if (testParsing) {
-                        Formattable parsedNumber;
-                        formatter->parse(actualString, parsedNumber, status);
-                        if (U_FAILURE(status)) {
-                            UnicodeString msg = "FAIL: formatter could not parse ";
-                            msg.append(actualString);
-                            msg.append(" status code: " );
-                            char buffer[32];
-                            sprintf(buffer, "0x%x", status);
-                            msg.append(buffer);
-                            errln(msg);
-                            break;
-                        } else {
-                            if (parsedNumber != expectedNumber) {
-                                UnicodeString msg = "FAIL: parse failed for ";
+                    } else {
+                        logln(actualString);
+                        if (testParsing) {
+                            Formattable parsedNumber;
+                            formatter->parse(actualString, parsedNumber, status);
+                            if (U_FAILURE(status)) {
+                                UnicodeString msg = "FAIL: formatter could not parse ";
                                 msg.append(actualString);
-                                msg.append(", expected ");
-                                decFmt->format(expectedNumber, msg, status);
-                                msg.append(", but got ");
-                                decFmt->format(parsedNumber, msg, status);
+                                msg.append(" status code: " );
+                                msg.append(u_errorName(status));
                                 errln(msg);
                                 break;
+                            } else {
+                                if (parsedNumber != expectedNumber) {
+                                    UnicodeString msg = "FAIL: parse failed for ";
+                                    msg.append(actualString);
+                                    msg.append(", expected ");
+                                    decFmt.format(expectedNumber, msg, status);
+                                    msg.append(", but got ");
+                                    decFmt.format(parsedNumber, msg, status);
+                                    errln(msg);
+                                    break;
+                                }
                             }
                         }
                     }
                 }
             }
         }
-        delete decFmt;
     }
 }
 
@@ -1411,3 +1908,5 @@ IntlTestRBNF::TestRBNFDisabled() {
 
 /* U_HAVE_RBNF */
 #endif
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
