@@ -1,6 +1,6 @@
 /***********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation
+ * Copyright (c) 1997-2007, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
 
@@ -17,6 +17,9 @@
 #include "tztest.h"
 #include "cmemory.h"
 #include "putilimp.h"
+#include "cstring.h"
+
+#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
 
 #define CASE(id,test) case id:                               \
                           name = #test;                      \
@@ -47,6 +50,9 @@ void TimeZoneTest::runIndexedTest( int32_t index, UBool exec, const char* &name,
         CASE(10,TestCountries); 
         CASE(11,TestHistorical);
         CASE(12,TestEquivalentIDs);
+        CASE(13, TestAliasedNames);
+        CASE(14, TestFractionalDST);
+        CASE(15, TestFebruary);
        default: name = ""; break;
     }
 }
@@ -85,7 +91,7 @@ TimeZoneTest::TestGenericAPI()
 
     TimeZone* saveDefault = TimeZone::createDefault();
     logln((UnicodeString)"TimeZone::createDefault() => " + saveDefault->getID(id));
-    //TimeZone* pstZone = TimeZone::createTimeZone("PST");
+    TimeZone* pstZone = TimeZone::createTimeZone("America/Los_Angeles");
 
     logln("call uprv_timezone() which uses the host");
     logln("to get the difference in seconds between coordinated universal");
@@ -96,17 +102,12 @@ TimeZoneTest::TestGenericAPI()
     // Invert sign because UNIX semantics are backwards
     if (tzoffset < 0)
         tzoffset = -tzoffset;
-    // --- The following test would fail outside PST now that
-    // --- PST is generally set to be default timezone in format tests
-    //if ((*saveDefault == *pstZone) && (tzoffset != 28800)) {
-    //  errln("FAIL: t_timezone may be incorrect.  It is not 28800");
-    //}
-
-    if (tzoffset != 28800) {
-        logln("***** WARNING: If testing in the PST timezone, uprv_timezone should return 28800! *****");
+    if ((*saveDefault == *pstZone) && (tzoffset != 28800)) {
+        errln("FAIL: t_timezone may be incorrect.  It is not 28800");
     }
-    if ((tzoffset % 1800 != 0)) {
-      errln("FAIL: t_timezone may be incorrect. It is not a multiple of 30min. It is %d", tzoffset);
+
+    if ((tzoffset % 900) != 0) {
+        errln("FAIL: t_timezone may be incorrect. It is not a multiple of 15min. It is %d", tzoffset);
     }
 
     TimeZone::adoptDefault(zone);
@@ -117,7 +118,17 @@ TimeZoneTest::TestGenericAPI()
     TimeZone::adoptDefault(saveDefault);
     delete defaultzone;
     delete zoneclone;
-    //delete pstZone;
+    delete pstZone;
+
+    UErrorCode status = U_ZERO_ERROR;
+    const char* tzver = TimeZone::getTZDataVersion(status);
+    if (U_FAILURE(status)) {
+        errln("FAIL: getTZDataVersion failed");
+    } else if (uprv_strlen(tzver) != 5 /* 4 digits + 1 letter */) {
+        errln((UnicodeString)"FAIL: getTZDataVersion returned " + tzver);
+    } else {
+        logln((UnicodeString)"tzdata version: " + tzver);
+    }
 }
 
 // ---------------------------------------------------------------------------------
@@ -505,6 +516,9 @@ TimeZoneTest::TestGetAvailableIDs913()
  * definition. This test has been updated to reflect this.
  * 12/3/99 aliu
  *
+ * Added tests for additional zones and aliases from the icuzones file.
+ * Markus Scherer 2006-nov-06
+ *
  * [srl - from java - 7/5/1998]
  * @bug 4130885
  * Certain short zone IDs, used since 1.1.x, are incorrect.
@@ -608,6 +622,24 @@ void TimeZoneTest::TestShortZoneIDs()
         // "NST", 720, FALSE,
         // As of bug 4130885, fix NST (New Zealand)
         {"NST", 720, TRUE}, // Pacific/Auckland
+
+        // From icuzones:
+        {"Etc/Unknown", 0, FALSE},
+
+        {"SystemV/AST4ADT", -240, TRUE},
+        {"SystemV/EST5EDT", -300, TRUE},
+        {"SystemV/CST6CDT", -360, TRUE},
+        {"SystemV/MST7MDT", -420, TRUE},
+        {"SystemV/PST8PDT", -480, TRUE},
+        {"SystemV/YST9YDT", -540, TRUE},
+        {"SystemV/AST4", -240, FALSE},
+        {"SystemV/EST5", -300, FALSE},
+        {"SystemV/CST6", -360, FALSE},
+        {"SystemV/MST7", -420, FALSE},
+        {"SystemV/PST8", -480, FALSE},
+        {"SystemV/YST9", -540, FALSE},
+        {"SystemV/HST10", -600, FALSE},
+
         {"",0,FALSE}
     };
 
@@ -617,7 +649,7 @@ void TimeZoneTest::TestShortZoneIDs()
         UBool ok = TRUE;
         // Check existence.
         TimeZone *tz = TimeZone::createTimeZone(itsID);
-        if (!tz) {
+        if (!tz || (kReferenceList[i].offset != 0 && *tz == *TimeZone::getGMT())) {
             errln("FAIL: Time Zone " + itsID + " does not exist!");
             continue;
         }
@@ -654,14 +686,14 @@ void TimeZoneTest::TestShortZoneIDs()
 
     const char* compatibilityMap[] = {
         // This list is copied from tz.alias.  If tz.alias
-        // changes, this list must be updated.  Current as of Aug 2003
+        // changes, this list must be updated.  Current as of Mar 2007
         "ACT", "Australia/Darwin",
         "AET", "Australia/Sydney",
         "AGT", "America/Buenos_Aires",
         "ART", "Africa/Cairo",
         "AST", "America/Anchorage",
         "BET", "America/Sao_Paulo",
-        "BST", "Asia/Dhaka", // Spelling changed in 2000h
+        "BST", "Asia/Dhaka", // # spelling changed in 2000h; was Asia/Dacca
         "CAT", "Africa/Harare",
         "CNT", "America/St_Johns",
         "CST", "America/Chicago",
@@ -669,14 +701,14 @@ void TimeZoneTest::TestShortZoneIDs()
         "EAT", "Africa/Addis_Ababa",
         "ECT", "Europe/Paris",
         // EET Europe/Istanbul # EET is a standard UNIX zone
-        // "EST", "America/New_York", # EST is an Olson alias now (2003)
-        "HST", "Pacific/Honolulu",
+        // "EST", "America/New_York", # Defined as -05:00
+        // "HST", "Pacific/Honolulu", # Defined as -10:00
         "IET", "America/Indianapolis",
         "IST", "Asia/Calcutta",
         "JST", "Asia/Tokyo",
         // MET Asia/Tehran # MET is a standard UNIX zone
         "MIT", "Pacific/Apia",
-        // "MST", "America/Denver", # MST is an Olson alias now (2003)
+        // "MST", "America/Denver", # Defined as -07:00
         "NET", "Asia/Yerevan",
         "NST", "Pacific/Auckland",
         "PLT", "Asia/Karachi",
@@ -726,32 +758,52 @@ void TimeZoneTest::TestShortZoneIDs()
 /**
  * Utility function for TestCustomParse
  */
-UnicodeString& TimeZoneTest::formatMinutes(int32_t min, UnicodeString& rv)
+UnicodeString& TimeZoneTest::formatMinutes(int32_t min, UnicodeString& rv, UBool insertSep/*=TRUE*/)
 {
         rv.remove();
 
-        char sign = '+';
-        if (min < 0) { sign = '-'; min = -min; }
+        UChar sign = 0x002B;
+        if (min < 0) {
+            sign = 0x002D;
+            min = -min;
+        }
         int h = min/60;
         min = min%60;
 
-        rv += UChar(sign);
-        if(h > 10)
-            rv += UChar(0x0030 + (h/10));
-        rv += UChar(0x0030 + (h%10));
-
-        rv += ":";
-
-        if(min > 10)
-            rv += UChar(0x0030 + (min/10));
+        rv += (UChar)(sign);
+        if(h >= 10)
+            rv += (UChar)(0x0030 + (h/10));
         else
             rv += "0";
 
-        rv += UChar(0x0030 + (min%10));
+        rv += (UChar)(0x0030 + (h%10));
+
+        if (insertSep)
+            rv += ":";
+
+        if(min >= 10)
+            rv += (UChar)(0x0030 + (min/10));
+        else
+            rv += "0";
+
+        rv += (UChar)(0x0030 + (min%10));
 
         return rv;
 }
 
+/**
+ * Utility function for TestCustomParse, generating RFC822 style
+ * time zone string for the give offset in minutes
+ */
+UnicodeString& TimeZoneTest::formatRFC822TZ(int32_t min, UnicodeString& rv)
+{
+    UnicodeString offsetStr;
+    formatMinutes(min, offsetStr, FALSE);
+    rv.remove();
+    rv += "GMT";
+    rv += offsetStr;
+    return rv;
+}
 
 /**
  * As part of the VM fix (see CCC approved RFE 4028006, bug
@@ -765,7 +817,6 @@ void TimeZoneTest::TestCustomParse()
 {
     int32_t i;
     const int32_t kUnparseable = 604800; // the number of seconds in a week. More than any offset should be.
-    const UnicodeString kExpectedCustomID = "Custom";
 
     struct
     {
@@ -775,13 +826,13 @@ void TimeZoneTest::TestCustomParse()
     kData[] =
     {
         // ID        Expected offset in minutes
-        //"GMT",       kUnparseable,   Isn't custom. Can't test it here. [returns normal GMT]
+        //{"GMT",       kUnparseable},   //Isn't custom. Can't test it here. [returns normal GMT]
         {"GMT-YOUR.AD.HERE", kUnparseable},
         // {"GMT0",      kUnparseable}, // ICU 2.8: An Olson zone ID
         // {"GMT+0",     (0)}, // ICU 2.8: An Olson zone ID
         {"GMT+1",     (60)},
         {"GMT-0030",  (-30)},
-        {"GMT+15:99", (15*60+99)},
+        {"GMT+15:99", kUnparseable},
         {"GMT+",      kUnparseable},
         {"GMT-",      kUnparseable},
         {"GMT+0:",    kUnparseable},
@@ -842,8 +893,9 @@ void TimeZoneTest::TestCustomParse()
         {
             zone->getID(itsID);
             int32_t ioffset = zone->getRawOffset()/60000;
-            UnicodeString offset;
+            UnicodeString offset, expectedID;
             formatMinutes(ioffset, offset);
+            formatRFC822TZ(ioffset, expectedID);
             logln(id + " -> " + itsID + " GMT" + offset);
             if (exp == kUnparseable)
             {
@@ -852,10 +904,11 @@ void TimeZoneTest::TestCustomParse()
                                     ", id " + itsID);
             }
             else if (ioffset != exp ||
-                     (itsID.compare(kExpectedCustomID) != 0))
+                     (itsID.compare(expectedID) != 0))
             {
                 errln("Expected offset of " + formatMinutes(exp,temp) +
-                                    ", id Custom, for " + id +
+                                    ", id " + expectedID +
+                                    ", for " + id +
                                     ", got offset of " + offset +
                                     ", id " + itsID);
             }
@@ -864,6 +917,178 @@ void TimeZoneTest::TestCustomParse()
     }
 }
 
+void
+TimeZoneTest::TestAliasedNames()
+{
+    struct {
+        const char *from;
+        const char *to;
+    } kData[] = {
+        /* Generated by org.unicode.cldr.tool.CountItems */
+
+        /* zoneID, canonical zoneID */
+        {"Africa/Timbuktu", "Africa/Bamako"},
+        {"America/Argentina/Buenos_Aires", "America/Buenos_Aires"},
+        {"America/Argentina/Catamarca", "America/Catamarca"},
+        {"America/Argentina/ComodRivadavia", "America/Catamarca"},
+        {"America/Argentina/Cordoba", "America/Cordoba"},
+        {"America/Argentina/Jujuy", "America/Jujuy"},
+        {"America/Argentina/Mendoza", "America/Mendoza"},
+        {"America/Atka", "America/Adak"},
+        {"America/Ensenada", "America/Tijuana"},
+        {"America/Fort_Wayne", "America/Indiana/Indianapolis"},
+        {"America/Indianapolis", "America/Indiana/Indianapolis"},
+        {"America/Knox_IN", "America/Indiana/Knox"},
+        {"America/Louisville", "America/Kentucky/Louisville"},
+        {"America/Porto_Acre", "America/Rio_Branco"},
+        {"America/Rosario", "America/Cordoba"},
+        {"America/Virgin", "America/St_Thomas"},
+        {"Asia/Ashkhabad", "Asia/Ashgabat"},
+        {"Asia/Chungking", "Asia/Chongqing"},
+        {"Asia/Dacca", "Asia/Dhaka"},
+        {"Asia/Istanbul", "Europe/Istanbul"},
+        {"Asia/Macao", "Asia/Macau"},
+        {"Asia/Tel_Aviv", "Asia/Jerusalem"},
+        {"Asia/Thimbu", "Asia/Thimphu"},
+        {"Asia/Ujung_Pandang", "Asia/Makassar"},
+        {"Asia/Ulan_Bator", "Asia/Ulaanbaatar"},
+        {"Australia/ACT", "Australia/Sydney"},
+        {"Australia/Canberra", "Australia/Sydney"},
+        {"Australia/LHI", "Australia/Lord_Howe"},
+        {"Australia/NSW", "Australia/Sydney"},
+        {"Australia/North", "Australia/Darwin"},
+        {"Australia/Queensland", "Australia/Brisbane"},
+        {"Australia/South", "Australia/Adelaide"},
+        {"Australia/Tasmania", "Australia/Hobart"},
+        {"Australia/Victoria", "Australia/Melbourne"},
+        {"Australia/West", "Australia/Perth"},
+        {"Australia/Yancowinna", "Australia/Broken_Hill"},
+        {"Brazil/Acre", "America/Rio_Branco"},
+        {"Brazil/DeNoronha", "America/Noronha"},
+        {"Brazil/East", "America/Sao_Paulo"},
+        {"Brazil/West", "America/Manaus"},
+        {"Canada/Atlantic", "America/Halifax"},
+        {"Canada/Central", "America/Winnipeg"},
+        {"Canada/East-Saskatchewan", "America/Regina"},
+        {"Canada/Eastern", "America/Toronto"},
+        {"Canada/Mountain", "America/Edmonton"},
+        {"Canada/Newfoundland", "America/St_Johns"},
+        {"Canada/Pacific", "America/Vancouver"},
+        {"Canada/Saskatchewan", "America/Regina"},
+        {"Canada/Yukon", "America/Whitehorse"},
+        {"Chile/Continental", "America/Santiago"},
+        {"Chile/EasterIsland", "Pacific/Easter"},
+        {"Cuba", "America/Havana"},
+        {"Egypt", "Africa/Cairo"},
+        {"Eire", "Europe/Dublin"},
+        {"Etc/GMT+0", "Etc/GMT"},
+        {"Etc/GMT-0", "Etc/GMT"},
+        {"Etc/GMT0", "Etc/GMT"},
+        {"Etc/Greenwich", "Etc/GMT"},
+        {"Etc/UCT", "Etc/GMT"},
+        {"Etc/UTC", "Etc/GMT"},
+        {"Etc/Universal", "Etc/GMT"},
+        {"Etc/Zulu", "Etc/GMT"},
+        {"Europe/Belfast", "Europe/London"},
+        {"Europe/Nicosia", "Asia/Nicosia"},
+        {"Europe/Tiraspol", "Europe/Chisinau"},
+        {"GB", "Europe/London"},
+        {"GB-Eire", "Europe/London"},
+        {"GMT", "Etc/GMT"},
+        {"GMT+0", "Etc/GMT"},
+        {"GMT-0", "Etc/GMT"},
+        {"GMT0", "Etc/GMT"},
+        {"Greenwich", "Etc/GMT"},
+        {"Hongkong", "Asia/Hong_Kong"},
+        {"Iceland", "Atlantic/Reykjavik"},
+        {"Iran", "Asia/Tehran"},
+        {"Israel", "Asia/Jerusalem"},
+        {"Jamaica", "America/Jamaica"},
+        {"Japan", "Asia/Tokyo"},
+        {"Kwajalein", "Pacific/Kwajalein"},
+        {"Libya", "Africa/Tripoli"},
+        {"Mexico/BajaNorte", "America/Tijuana"},
+        {"Mexico/BajaSur", "America/Mazatlan"},
+        {"Mexico/General", "America/Mexico_City"},
+        {"NZ", "Pacific/Auckland"},
+        {"NZ-CHAT", "Pacific/Chatham"},
+        {"Navajo", "America/Shiprock"},
+        {"PRC", "Asia/Shanghai"},
+        {"Pacific/Samoa", "Pacific/Pago_Pago"},
+        {"Pacific/Yap", "Pacific/Truk"},
+        {"Poland", "Europe/Warsaw"},
+        {"Portugal", "Europe/Lisbon"},
+        {"ROC", "Asia/Taipei"},
+        {"ROK", "Asia/Seoul"},
+        {"Singapore", "Asia/Singapore"},
+        {"Turkey", "Europe/Istanbul"},
+        {"UCT", "Etc/GMT"},
+        {"US/Alaska", "America/Anchorage"},
+        {"US/Aleutian", "America/Adak"},
+        {"US/Arizona", "America/Phoenix"},
+        {"US/Central", "America/Chicago"},
+        {"US/East-Indiana", "America/Indiana/Indianapolis"},
+        {"US/Eastern", "America/New_York"},
+        {"US/Hawaii", "Pacific/Honolulu"},
+        {"US/Indiana-Starke", "America/Indiana/Knox"},
+        {"US/Michigan", "America/Detroit"},
+        {"US/Mountain", "America/Denver"},
+        {"US/Pacific", "America/Los_Angeles"},
+        {"US/Pacific-New", "America/Los_Angeles"},
+        {"US/Samoa", "Pacific/Pago_Pago"},
+        {"UTC", "Etc/GMT"},
+        {"Universal", "Etc/GMT"},
+        {"W-SU", "Europe/Moscow"},
+        {"Zulu", "Etc/GMT"},
+        /* Total: 113 */
+
+    };
+
+    TimeZone::EDisplayType styles[] = { TimeZone::SHORT, TimeZone::LONG };
+    UBool useDst[] = { FALSE, TRUE };
+    int32_t noLoc = uloc_countAvailable();
+
+    int32_t i, j, k, loc;
+    UnicodeString fromName, toName;
+    TimeZone *from = NULL, *to = NULL;
+    for(i = 0; i < (int32_t)(sizeof(kData)/sizeof(kData[0])); i++) {
+        from = TimeZone::createTimeZone(kData[i].from);
+        to = TimeZone::createTimeZone(kData[i].to);
+        if(!from->hasSameRules(*to)) {
+            errln("different at %i\n", i);
+        }
+        if(!quick) {
+            for(loc = 0; loc < noLoc; loc++) {
+                const char* locale = uloc_getAvailable(loc); 
+                for(j = 0; j < (int32_t)(sizeof(styles)/sizeof(styles[0])); j++) {
+                    for(k = 0; k < (int32_t)(sizeof(useDst)/sizeof(useDst[0])); k++) {
+                        fromName.remove();
+                        toName.remove();
+                        from->getDisplayName(useDst[k], styles[j],locale, fromName);
+                        to->getDisplayName(useDst[k], styles[j], locale, toName);
+                        if(fromName.compare(toName) != 0) {
+                            errln("Fail: Expected "+toName+" but got " + prettify(fromName) 
+                                + " for locale: " + locale + " index: "+ loc 
+                                + " to id "+ kData[i].to
+                                + " from id " + kData[i].from);
+                        }
+                    }
+                }
+            }
+        } else {
+            fromName.remove();
+            toName.remove();
+            from->getDisplayName(fromName);
+            to->getDisplayName(toName);
+            if(fromName.compare(toName) != 0) {
+                errln("Fail: Expected "+toName+" but got " + fromName);
+            }
+        }
+        delete from;
+        delete to;
+    }
+}
+
 /**
  * Test the basic functionality of the getDisplayName() API.
  *
@@ -1149,6 +1374,22 @@ TimeZoneTest::TestAlternateRules()
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 }
 
+void TimeZoneTest::TestFractionalDST() {
+    const char* tzName = "Australia/Lord_Howe"; // 30 min offset
+    TimeZone* tz_icu = TimeZone::createTimeZone(tzName);
+	int dst_icu = tz_icu->getDSTSavings();
+    UnicodeString id;
+    int32_t expected = 1800000;
+	if (expected != dst_icu) {
+	    errln(UnicodeString("java reports dst savings of ") + expected +
+	        " but icu reports " + dst_icu + 
+	        " for tz " + tz_icu->getID(id));
+	} else {
+	    logln(UnicodeString("both java and icu report dst savings of ") + expected + " for tz " + tz_icu->getID(id));
+	}
+    delete tz_icu;
+}
+
 /**
  * Test country code support.  Jitterbug 776.
  */
@@ -1300,4 +1541,119 @@ void TimeZoneTest::TestEquivalentIDs() {
     }
 }
 
+// Test that a transition at the end of February is handled correctly.
+void TimeZoneTest::TestFebruary() {
+    UErrorCode status = U_ZERO_ERROR;
+
+    // Time zone with daylight savings time from the first Sunday in November
+    // to the last Sunday in February.
+    // Similar to the new rule for Brazil (Sao Paulo) in tzdata2006n.
+    SimpleTimeZone tz1(-3 * U_MILLIS_PER_HOUR,          // raw offset: 3h before (west of) GMT
+                       UNICODE_STRING("nov-feb", 7),
+                       UCAL_NOVEMBER, 1, UCAL_SUNDAY,   // start: November, first, Sunday
+                       0,                               //        midnight wall time
+                       UCAL_FEBRUARY, -1, UCAL_SUNDAY,  // end:   February, last, Sunday
+                       0,                               //        midnight wall time
+                       status);
+    if (U_FAILURE(status)) {
+        errln("Unable to create the SimpleTimeZone(nov-feb): %s", u_errorName(status));
+        return;
+    }
+
+    // Time zone for Brazil, with effectively the same rules as above,
+    // but expressed with DOW_GE_DOM_MODE and DOW_LE_DOM_MODE rules.
+    TimeZone *tz2 = TimeZone::createTimeZone(UNICODE_STRING_SIMPLE("America/Sao_Paulo"));
+    if (U_FAILURE(status) || (*tz2 == *TimeZone::getGMT())) {
+        delete tz2;
+        errln("Unable to create the America/Sao_Paulo TimeZone: %s", u_errorName(status));
+        return;
+    }
+
+    // Now hardcode the same rules as for Brazil, so that we cover the intended code
+    // even when in the future zoneinfo hardcodes these transition dates.
+    SimpleTimeZone tz3(-3 * U_MILLIS_PER_HOUR,          // raw offset: 3h before (west of) GMT
+                       UNICODE_STRING("nov-feb2", 8),
+                       UCAL_NOVEMBER, 1, -UCAL_SUNDAY,  // start: November, 1 or after, Sunday
+                       0,                               //        midnight wall time
+                       UCAL_FEBRUARY, -29, -UCAL_SUNDAY,// end:   February, 29 or before, Sunday
+                       0,                               //        midnight wall time
+                       status);
+    if (U_FAILURE(status)) {
+        errln("Unable to create the SimpleTimeZone(nov-feb2): %s", u_errorName(status));
+        return;
+    }
+
+    // Gregorian calendar with the UTC time zone for getting sample test date/times.
+    GregorianCalendar gc(*TimeZone::getGMT(), status);
+    if (U_FAILURE(status)) {
+        errln("Unable to create the UTC calendar: %s", u_errorName(status));
+        return;
+    }
+
+    struct {
+        // UTC time.
+        int32_t year, month, day, hour, minute, second;
+        // Expected time zone offset in hours after GMT (negative=before GMT).
+        int32_t offsetHours;
+    } data[] = {
+        { 2006, UCAL_NOVEMBER,  5, 02, 59, 59, -3 },
+        { 2006, UCAL_NOVEMBER,  5, 03, 00, 00, -2 },
+        { 2007, UCAL_FEBRUARY, 25, 01, 59, 59, -2 },
+        { 2007, UCAL_FEBRUARY, 25, 02, 00, 00, -3 },
+
+        { 2007, UCAL_NOVEMBER,  4, 02, 59, 59, -3 },
+        { 2007, UCAL_NOVEMBER,  4, 03, 00, 00, -2 },
+        { 2008, UCAL_FEBRUARY, 24, 01, 59, 59, -2 },
+        { 2008, UCAL_FEBRUARY, 24, 02, 00, 00, -3 },
+
+        { 2008, UCAL_NOVEMBER,  2, 02, 59, 59, -3 },
+        { 2008, UCAL_NOVEMBER,  2, 03, 00, 00, -2 },
+        { 2009, UCAL_FEBRUARY, 22, 01, 59, 59, -2 },
+        { 2009, UCAL_FEBRUARY, 22, 02, 00, 00, -3 },
+
+        { 2009, UCAL_NOVEMBER,  1, 02, 59, 59, -3 },
+        { 2009, UCAL_NOVEMBER,  1, 03, 00, 00, -2 },
+        { 2010, UCAL_FEBRUARY, 28, 01, 59, 59, -2 },
+        { 2010, UCAL_FEBRUARY, 28, 02, 00, 00, -3 }
+    };
+
+    TimeZone *timezones[] = { &tz1, tz2, &tz3 };
+
+    TimeZone *tz;
+    UDate dt;
+    int32_t t, i, raw, dst;
+    for (t = 0; t < LENGTHOF(timezones); ++t) {
+        tz = timezones[t];
+        for (i = 0; i < LENGTHOF(data); ++i) {
+            gc.set(data[i].year, data[i].month, data[i].day,
+                   data[i].hour, data[i].minute, data[i].second);
+            dt = gc.getTime(status);
+            if (U_FAILURE(status)) {
+                errln("test case %d.%d: bad date/time %04d-%02d-%02d %02d:%02d:%02d",
+                      t, i,
+                      data[i].year, data[i].month + 1, data[i].day,
+                      data[i].hour, data[i].minute, data[i].second);
+                status = U_ZERO_ERROR;
+                continue;
+            }
+            tz->getOffset(dt, FALSE, raw, dst, status);
+            if (U_FAILURE(status)) {
+                errln("test case %d.%d: tz.getOffset(%04d-%02d-%02d %02d:%02d:%02d) fails: %s",
+                      t, i,
+                      data[i].year, data[i].month + 1, data[i].day,
+                      data[i].hour, data[i].minute, data[i].second,
+                      u_errorName(status));
+                status = U_ZERO_ERROR;
+            } else if ((raw + dst) != data[i].offsetHours * U_MILLIS_PER_HOUR) {
+                errln("test case %d.%d: tz.getOffset(%04d-%02d-%02d %02d:%02d:%02d) returns %d+%d != %d",
+                      t, i,
+                      data[i].year, data[i].month + 1, data[i].day,
+                      data[i].hour, data[i].minute, data[i].second,
+                      raw, dst, data[i].offsetHours * U_MILLIS_PER_HOUR);
+            }
+        }
+    }
+    delete tz2;
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
