@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2000-2001, International Business Machines
+*   Copyright (C) 2000-2007, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -23,20 +23,84 @@
 *   05/09/2001  weiv                    Case bits are now in the CEs, not in front
 */
 
+#include "unicode/utypes.h"
+#include "unicode/putil.h"
+#include "unicode/udata.h"
+#include "unicode/uclean.h"
+#include "ucol_imp.h"
 #include "genuca.h"
 #include "uoptions.h"
 #include "toolutil.h"
+#include "unewdata.h"
 #include "cstring.h"
-
 #include "cmemory.h"
 
-UCAElements le;
+#include <stdio.h>
 
 /*
  * Global - verbosity
  */
 UBool VERBOSE = FALSE;
 
+static UVersionInfo UCAVersion;
+
+#if UCONFIG_NO_COLLATION
+
+/* dummy UDataInfo cf. udata.h */
+static UDataInfo dummyDataInfo = {
+    sizeof(UDataInfo),
+    0,
+
+    U_IS_BIG_ENDIAN,
+    U_CHARSET_FAMILY,
+    U_SIZEOF_UCHAR,
+    0,
+
+    { 0, 0, 0, 0 },                 /* dummy dataFormat */
+    { 0, 0, 0, 0 },                 /* dummy formatVersion */
+    { 0, 0, 0, 0 }                  /* dummy dataVersion */
+};
+
+#else
+
+static const UDataInfo ucaDataInfo={
+    sizeof(UDataInfo),
+    0,
+
+    U_IS_BIG_ENDIAN,
+    U_CHARSET_FAMILY,
+    sizeof(UChar),
+    0,
+
+    {UCA_DATA_FORMAT_0, UCA_DATA_FORMAT_1, UCA_DATA_FORMAT_2, UCA_DATA_FORMAT_3},     /* dataFormat="UCol"            */
+    /* 03/26/2002 bumped up version since format has changed */
+    /* 09/16/2002 bumped up version since we went from UColAttributeValue */
+    /*            to int32_t in UColOptionSet */
+    /* 05/13/2003 This one also updated since we added UCA and UCD versions */
+    /*            to header */
+    /* 09/11/2003 Adding information required by data swapper */
+    {UCA_FORMAT_VERSION_0, UCA_FORMAT_VERSION_1, UCA_FORMAT_VERSION_2, UCA_FORMAT_VERSION_3},                 /* formatVersion                */
+    {0, 0, 0, 0}                  /* dataVersion = Unicode Version*/
+};
+
+static const UDataInfo invUcaDataInfo={
+    sizeof(UDataInfo),
+    0,
+
+    U_IS_BIG_ENDIAN,
+    U_CHARSET_FAMILY,
+    sizeof(UChar),
+    0,
+
+    {INVUCA_DATA_FORMAT_0, INVUCA_DATA_FORMAT_1, INVUCA_DATA_FORMAT_2, INVUCA_DATA_FORMAT_3},     /* dataFormat="InvC"            */
+    /* 03/26/2002 bumped up version since format has changed */
+    /* 04/29/2003 2.1 format - we have added UCA version to header */
+    {INVUCA_FORMAT_VERSION_0, INVUCA_FORMAT_VERSION_1, INVUCA_FORMAT_VERSION_2, INVUCA_FORMAT_VERSION_3},                 /* formatVersion                */
+    {0, 0, 0, 0}                  /* dataVersion = Unicode Version*/
+};
+
+UCAElements le;
+
 int32_t readElement(char **from, char *to, char separator, UErrorCode *status) {
     if(U_FAILURE(*status)) {
         return 0;
@@ -132,8 +196,6 @@ static void addNewInverse(UCAElements *element, UErrorCode *status) {
 }
 
 static void insertInverse(UCAElements *element, uint32_t position, UErrorCode *status) {
-  uint8_t space[4096];
-
   if(U_FAILURE(*status)) {
     return;
   }
@@ -143,8 +205,8 @@ static void insertInverse(UCAElements *element, uint32_t position, UErrorCode *s
   }
   if(position <= inversePos) {
     /*move stuff around */
-    uprv_memcpy(space, inverseTable[position], (inversePos - position+1)*sizeof(inverseTable[0]));
-    uprv_memcpy(inverseTable[position+1], space, (inversePos - position+1)*sizeof(inverseTable[0]));
+    uint32_t amountToMove = (inversePos - position+1)*sizeof(inverseTable[0]);
+    uprv_memmove(inverseTable[position+1], inverseTable[position], amountToMove);
   }
   inverseTable[position][0] = element->CEs[0];
   if(element->noOfCEs > 1 && isContinuation(element->CEs[1])) {
@@ -194,10 +256,58 @@ static void addToExistingInverse(UCAElements *element, uint32_t position, UError
       }
 }
 
+/* 
+ * Takes two CEs (lead and continuation) and 
+ * compares them as CEs should be compared:
+ * primary vs. primary, secondary vs. secondary
+ * tertiary vs. tertiary
+ */
+static int32_t compareCEs(uint32_t *source, uint32_t *target) {
+  uint32_t s1 = source[0], s2, t1 = target[0], t2;
+  if(isContinuation(source[1])) {
+    s2 = source[1];
+  } else {
+    s2 = 0;
+  }
+  if(isContinuation(target[1])) {
+    t2 = target[1];
+  } else {
+    t2 = 0;
+  }
+  
+  uint32_t s = 0, t = 0;
+  if(s1 == t1 && s2 == t2) {
+    return 0;
+  }
+  s = (s1 & 0xFFFF0000)|((s2 & 0xFFFF0000)>>16); 
+  t = (t1 & 0xFFFF0000)|((t2 & 0xFFFF0000)>>16); 
+  if(s < t) {
+    return -1;
+  } else if(s > t) {
+    return 1;
+  } else {
+    s = (s1 & 0x0000FF00) | (s2 & 0x0000FF00)>>8;
+    t = (t1 & 0x0000FF00) | (t2 & 0x0000FF00)>>8;
+    if(s < t) {
+      return -1;
+    } else if(s > t) {
+      return 1;
+    } else {
+      s = (s1 & 0x000000FF)<<8 | (s2 & 0x000000FF);
+      t = (t1 & 0x000000FF)<<8 | (t2 & 0x000000FF);
+      if(s < t) {
+        return -1;
+      } else {
+        return 1;
+      }
+    }
+  }
+}
+
 static uint32_t addToInverse(UCAElements *element, UErrorCode *status) {
-  uint32_t comp = 0;
   uint32_t position = inversePos;
   uint32_t saveElement = element->CEs[0];
+  int32_t compResult = 0;
   element->CEs[0] &= 0xFFFFFF3F;
   if(element->noOfCEs == 1) {
     element->CEs[1] = 0;
@@ -205,55 +315,34 @@ static uint32_t addToInverse(UCAElements *element, UErrorCode *status) {
   if(inversePos == 0) {
     inverseTable[0][0] = inverseTable[0][1] = inverseTable[0][2] = 0;
     addNewInverse(element, status);
-  } else if(inverseTable[inversePos][0] > element->CEs[0]) {
-    while(inverseTable[--position][0] > element->CEs[0]) {}
-    if(inverseTable[position][0] == element->CEs[0]) {
-      if(isContinuation(element->CEs[1])) {
-        comp = element->CEs[1];
-      } else {
-        comp = 0;
-      }
-      if(inverseTable[position][1] > comp) {
-        while(inverseTable[--position][1] > comp) {}
-      }
-      if(inverseTable[position][1] == comp) {
-        addToExistingInverse(element, position, status);
-      } else {
-        insertInverse(element, position+1, status);
-      }
+  } else if(compareCEs(inverseTable[inversePos], element->CEs) > 0) {
+    while((compResult = compareCEs(inverseTable[--position], element->CEs)) > 0);
+    if(VERBOSE) { fprintf(stdout, "p:%u ", (int)position); }
+    if(compResult == 0) {
+      addToExistingInverse(element, position, status);
     } else {
       insertInverse(element, position+1, status);
     }
-  } else if(inverseTable[inversePos][0] == element->CEs[0]) {
-    if(element->noOfCEs > 1 && isContinuation(element->CEs[1])) {
-      comp = element->CEs[1];
-      if(inverseTable[position][1] > comp) {
-        while(inverseTable[--position][1] > comp) {}
-      }
-      if(inverseTable[position][1] == comp) {
-        addToExistingInverse(element, position, status);
-      } else {
-        insertInverse(element, position+1, status);
-      }
-    } else {
-      addToExistingInverse(element, inversePos, status);
-    } 
+  } else if(compareCEs(inverseTable[inversePos], element->CEs) == 0) {
+    addToExistingInverse(element, inversePos, status);
   } else {
     addNewInverse(element, status);
   }
   element->CEs[0] = saveElement;
+  if(VERBOSE) { fprintf(stdout, "+"); }
   return inversePos;
 }
 
-static InverseTableHeader *assembleInverseTable(UErrorCode *status)
+static InverseUCATableHeader *assembleInverseTable(UErrorCode *status)
 {
-  InverseTableHeader *result = NULL;
-  uint32_t headerByteSize = paddedsize(sizeof(InverseTableHeader));
+  InverseUCATableHeader *result = NULL;
+  uint32_t headerByteSize = paddedsize(sizeof(InverseUCATableHeader));
   uint32_t inverseTableByteSize = (inversePos+2)*sizeof(uint32_t)*3;
   uint32_t contsByteSize = sContPos * sizeof(UChar);
   uint32_t i = 0;
 
-  result = (InverseTableHeader *)uprv_malloc(headerByteSize + inverseTableByteSize + contsByteSize);
+  result = (InverseUCATableHeader *)uprv_malloc(headerByteSize + inverseTableByteSize + contsByteSize);
+  uprv_memset(result, 0, headerByteSize + inverseTableByteSize + contsByteSize);
   if(result != NULL) {
     result->byteSize = headerByteSize + inverseTableByteSize + contsByteSize;
 
@@ -264,10 +353,10 @@ static InverseTableHeader *assembleInverseTable(UErrorCode *status)
     inversePos++;
 
     for(i = 2; i<inversePos; i++) {
-      if(inverseTable[i-1][0] > inverseTable[i][0]) {
-        fprintf(stderr, "Error at %i: %08X & %08X\n", i, inverseTable[i-1][0], inverseTable[i][0]);
+      if(compareCEs(inverseTable[i-1], inverseTable[i]) > 0) { 
+        fprintf(stderr, "Error at %i: %08X & %08X\n", (int)i, (int)inverseTable[i-1][0], (int)inverseTable[i][0]);
       } else if(inverseTable[i-1][0] == inverseTable[i][0] && !(inverseTable[i-1][1] < inverseTable[i][1])) {
-        fprintf(stderr, "Continuation error at %i: %08X %08X & %08X %08X\n", i, inverseTable[i-1][0], inverseTable[i-1][1], inverseTable[i][0], inverseTable[i][1]);
+        fprintf(stderr, "Continuation error at %i: %08X %08X & %08X %08X\n", (int)i, (int)inverseTable[i-1][0], (int)inverseTable[i-1][1], (int)inverseTable[i][0], (int)inverseTable[i][1]);
       }
     }
 
@@ -289,8 +378,8 @@ static InverseTableHeader *assembleInverseTable(UErrorCode *status)
 }
 
 
-static void writeOutInverseData(InverseTableHeader *data,
-                  const char *outputDir,
+static void writeOutInverseData(InverseUCATableHeader *data, 
+                  const char *outputDir, 
                   const char *copyright,
                   UErrorCode *status)
 {
@@ -298,11 +387,15 @@ static void writeOutInverseData(InverseTableHeader *data,
     
     long dataLength;
 
-    pData=udata_create(outputDir, INVC_DATA_TYPE, INVC_DATA_NAME, &invUcaDataInfo,
+    UDataInfo invUcaInfo;
+    uprv_memcpy(&invUcaInfo, &invUcaDataInfo, sizeof(UDataInfo));
+    u_getUnicodeVersion(invUcaInfo.dataVersion);
+
+    pData=udata_create(outputDir, INVC_DATA_TYPE, INVC_DATA_NAME, &invUcaInfo,
                        copyright, status);
 
     if(U_FAILURE(*status)) {
-        fprintf(stderr, "Error: unable to create data memory, error %d\n", *status);
+        fprintf(stderr, "Error: unable to create %s"INVC_DATA_NAME", error %s\n", outputDir, u_errorName(*status));
         return;
     }
 
@@ -336,7 +429,7 @@ static int32_t hex2num(char hex) {
     }
 }
 
-UCAElements *readAnElement(FILE *data, UErrorCode *status) {
+UCAElements *readAnElement(FILE *data, tempUCATable *t, UCAConstants *consts, UErrorCode *status) {
     char buffer[2048], primary[100], secondary[100], tertiary[100];
     UBool detectedContraction;
     int32_t i = 0;
@@ -347,6 +440,7 @@ UCAElements *readAnElement(FILE *data, UErrorCode *status) {
     char *endCodePoint = NULL;
     char *spacePointer = NULL;
     char *result = fgets(buffer, 2048, data);
+    int32_t buflen = (int32_t)uprv_strlen(buffer);
     if(U_FAILURE(*status)) {
         return 0;
     }
@@ -360,27 +454,125 @@ UCAElements *readAnElement(FILE *data, UErrorCode *status) {
             return NULL;
         }
     }
-    if(buffer[0] == '#' || buffer[0] == '\n') {
+    while(buflen>0 && (buffer[buflen-1] == '\r' || buffer[buflen-1] == '\n')) {
+      buffer[--buflen] = 0;
+    }
+
+    if(buffer[0] == 0 || buffer[0] == '#') {
         return NULL; // just a comment, skip whole line
     }
 
     UCAElements *element = &le; //(UCAElements *)malloc(sizeof(UCAElements));
 
+    enum ActionType {
+      READCE,
+      READHEX,
+      READUCAVERSION
+    };
+
+    // Directives.
     if(buffer[0] == '[') {
-      const char *vt = "[variable top = ";
-      uint32_t vtLen = uprv_strlen(vt);
-      if(uprv_strncmp(buffer, vt, vtLen) == 0) {
-        element->variableTop = TRUE;
-        if(sscanf(buffer+vtLen, "%4x", &theValue) != 1) /* read first code point */
-        {
-          fprintf(stderr, " scanf(hex) failed!\n ");
+      uint32_t cnt = 0;
+      static const struct {
+        char name[128];
+        uint32_t *what;
+        ActionType what_to_do;
+      } vt[]  = { {"[first tertiary ignorable",  consts->UCA_FIRST_TERTIARY_IGNORABLE,  READCE},
+                  {"[last tertiary ignorable",   consts->UCA_LAST_TERTIARY_IGNORABLE,   READCE},
+                  {"[first secondary ignorable", consts->UCA_FIRST_SECONDARY_IGNORABLE, READCE},
+                  {"[last secondary ignorable",  consts->UCA_LAST_SECONDARY_IGNORABLE,  READCE},
+                  {"[first primary ignorable",   consts->UCA_FIRST_PRIMARY_IGNORABLE,   READCE},
+                  {"[last primary ignorable",    consts->UCA_LAST_PRIMARY_IGNORABLE,    READCE},
+                  {"[first variable",            consts->UCA_FIRST_VARIABLE,            READCE},
+                  {"[last variable",             consts->UCA_LAST_VARIABLE,             READCE},
+                  {"[first regular",             consts->UCA_FIRST_NON_VARIABLE,        READCE},
+                  {"[last regular",              consts->UCA_LAST_NON_VARIABLE,         READCE},
+                  {"[first implicit",            consts->UCA_FIRST_IMPLICIT,            READCE},
+                  {"[last implicit",             consts->UCA_LAST_IMPLICIT,             READCE},
+                  {"[first trailing",            consts->UCA_FIRST_TRAILING,            READCE},
+                  {"[last trailing",             consts->UCA_LAST_TRAILING,             READCE},
+
+                  {"[fixed top",                       &consts->UCA_PRIMARY_TOP_MIN,           READHEX},
+                  {"[fixed first implicit byte",       &consts->UCA_PRIMARY_IMPLICIT_MIN,      READHEX},
+                  {"[fixed last implicit byte",        &consts->UCA_PRIMARY_IMPLICIT_MAX,      READHEX},
+                  {"[fixed first trail byte",          &consts->UCA_PRIMARY_TRAILING_MIN,      READHEX},
+                  {"[fixed last trail byte",           &consts->UCA_PRIMARY_TRAILING_MAX,      READHEX},
+                  {"[fixed first special byte",        &consts->UCA_PRIMARY_SPECIAL_MIN,       READHEX},
+                  {"[fixed last special byte",         &consts->UCA_PRIMARY_SPECIAL_MAX,       READHEX},
+                  {"[variable top = ",                &t->options->variableTopValue,          READHEX},
+                  {"[UCA version = ",                 NULL,                          READUCAVERSION}
+      };
+      for (cnt = 0; cnt<sizeof(vt)/sizeof(vt[0]); cnt++) {
+        uint32_t vtLen = (uint32_t)uprv_strlen(vt[cnt].name);
+        if(uprv_strncmp(buffer, vt[cnt].name, vtLen) == 0) {
+            element->variableTop = TRUE;
+            if(vt[cnt].what_to_do == READHEX) {
+              if(sscanf(buffer+vtLen, "%4x", &theValue) != 1) /* read first code point */
+              {
+                  fprintf(stderr, " scanf(hex) failed on !\n ");
+              }
+              *(vt[cnt].what) = (UChar)theValue;
+              //if(cnt == 1) { // first implicit
+                // we need to set the value for top next
+                //uint32_t nextTop = ucol_prv_calculateImplicitPrimary(0x4E00); // CJK base
+                //consts->UCA_NEXT_TOP_VALUE = theValue<<24 | 0x030303;
+              //}
+            } else if (vt[cnt].what_to_do == READCE) { /* vt[cnt].what_to_do == READCE */
+              pointer = strchr(buffer+vtLen, '[');
+              if(pointer) {
+                pointer++;
+                element->sizePrim[0]=readElement(&pointer, primary, ',', status);
+                element->sizeSec[0]=readElement(&pointer, secondary, ',', status);
+                element->sizeTer[0]=readElement(&pointer, tertiary, ']', status);
+
+                vt[cnt].what[0] = getSingleCEValue(primary, secondary, tertiary, status);
+                if(element->sizePrim[0] > 2 || element->sizeSec[0] > 1 || element->sizeTer[0] > 1) {
+                  uint32_t CEi = 1;
+                  uint32_t value = UCOL_CONTINUATION_MARKER; /* Continuation marker */
+                    if(2*CEi<element->sizePrim[i]) {
+                        value |= ((hex2num(*(primary+4*CEi))&0xF)<<28);
+                        value |= ((hex2num(*(primary+4*CEi+1))&0xF)<<24);
+                    }
+
+                    if(2*CEi+1<element->sizePrim[i]) {
+                        value |= ((hex2num(*(primary+4*CEi+2))&0xF)<<20);
+                        value |= ((hex2num(*(primary+4*CEi+3))&0xF)<<16);
+                    }
+
+                    if(CEi<element->sizeSec[i]) {
+                        value |= ((hex2num(*(secondary+2*CEi))&0xF)<<12);
+                        value |= ((hex2num(*(secondary+2*CEi+1))&0xF)<<8);
+                    }
+
+                    if(CEi<element->sizeTer[i]) {
+                        value |= ((hex2num(*(tertiary+2*CEi))&0x3)<<4);
+                        value |= (hex2num(*(tertiary+2*CEi+1))&0xF);
+                    }
+
+                    CEi++;
+
+                    vt[cnt].what[1] = value;
+                    //element->CEs[CEindex++] = value;
+                } else {
+                  vt[cnt].what[1] = 0;
+                }
+              } else {
+                fprintf(stderr, "Failed to read a CE from line %s\n", buffer);
+              }
+            } else { //vt[cnt].what_to_do == READUCAVERSION
+              u_versionFromString(UCAVersion, buffer+vtLen);
+              if(VERBOSE) {
+                fprintf(stdout, "UCA version [%hu.%hu.%hu.%hu]\n", UCAVersion[0], UCAVersion[1], UCAVersion[2], UCAVersion[3]);
+              }
+            }
+            //element->cPoints[0] = (UChar)theValue;
+            //return element; 
+            return NULL;
         }
-        element->cPoints[0] = (UChar)theValue;
-        return element; // just a comment, skip whole line
-      } else {
-        *status = U_INVALID_FORMAT_ERROR;
-        return NULL;
       }
+      fprintf(stderr, "Warning: unrecognized option: %s\n", buffer);
+      //*status = U_INVALID_FORMAT_ERROR;
+      return NULL;
     }
     element->variableTop = FALSE;
 
@@ -432,7 +624,7 @@ UCAElements *readAnElement(FILE *data, UErrorCode *status) {
 
     commentStart = strchr(startCodePoint, '#');
     if(commentStart == NULL) {
-        commentStart = strlen(startCodePoint) + startCodePoint - 1;
+        commentStart = strlen(startCodePoint) + startCodePoint;
     }
 
     i = 0;
@@ -486,20 +678,27 @@ UCAElements *readAnElement(FILE *data, UErrorCode *status) {
       i++;
     }
     element->noOfCEs = CEindex;
-
+#if 0
     element->isThai = UCOL_ISTHAIPREVOWEL(element->cPoints[0]);
-
+#endif
     // we don't want any strange stuff after useful data!
-    while(pointer < commentStart)  {
-        if(*pointer != ' ') {
-            *status=U_INVALID_FORMAT_ERROR;
-            break;
+    if (pointer == NULL) {
+        /* huh? Did we get ']' without the '['? Pair your brackets! */
+        *status=U_INVALID_FORMAT_ERROR;
+    }
+    else {
+        while(pointer < commentStart)  {
+            if(*pointer != ' ' && *pointer != '\t')
+            {
+                *status=U_INVALID_FORMAT_ERROR;
+                break;
+            }
+            pointer++;
         }
-        pointer++;
     }
 
     if(U_FAILURE(*status)) {
-        fprintf(stderr, "problem putting stuff in hash table\n");
+        fprintf(stderr, "problem putting stuff in hash table %s\n", u_errorName(*status));
         *status = U_INTERNAL_PROGRAM_ERROR;
         return NULL;
     }
@@ -509,6 +708,7 @@ UCAElements *readAnElement(FILE *data, UErrorCode *status) {
 
 
 void writeOutData(UCATableHeader *data,
+                  UCAConstants *consts,
                   UChar contractions[][3],
                   uint32_t noOfcontractions,
                   const char *outputDir,
@@ -521,6 +721,9 @@ void writeOutData(UCATableHeader *data,
 
     uint32_t size = data->size;
 
+    data->UCAConsts = data->size;
+    data->size += paddedsize(sizeof(UCAConstants));
+
     if(noOfcontractions != 0) {
       contractions[noOfcontractions][0] = 0;
       contractions[noOfcontractions][1] = 0;
@@ -528,19 +731,24 @@ void writeOutData(UCATableHeader *data,
       noOfcontractions++;
 
 
-      data->contractionUCACombos = size;
+      data->contractionUCACombos = data->size;
+      data->contractionUCACombosWidth = 3;
+      data->contractionUCACombosSize = noOfcontractions;
       data->size += paddedsize((noOfcontractions*3*sizeof(UChar)));
     }
 
     UNewDataMemory *pData;
     
     long dataLength;
+    UDataInfo ucaInfo;
+    uprv_memcpy(&ucaInfo, &ucaDataInfo, sizeof(UDataInfo));
+    u_getUnicodeVersion(ucaInfo.dataVersion);
 
-    pData=udata_create(outputDir, UCA_DATA_TYPE, UCA_DATA_NAME, &ucaDataInfo,
+    pData=udata_create(outputDir, UCA_DATA_TYPE, UCA_DATA_NAME, &ucaInfo,
                        copyright, status);
 
     if(U_FAILURE(*status)) {
-        fprintf(stderr, "Error: unable to create data memory, error %d\n", *status);
+        fprintf(stderr, "Error: unable to create %s"UCA_DATA_NAME", error %s\n", outputDir, u_errorName(*status));
         return;
     }
 
@@ -548,11 +756,14 @@ void writeOutData(UCATableHeader *data,
     if (VERBOSE) {
         fprintf(stdout, "Writing out UCA table: %s%c%s.%s\n", outputDir,
                                                         U_FILE_SEP_CHAR,
-                                                        UCA_DATA_NAME,
+                                                        U_ICUDATA_NAME "_" UCA_DATA_NAME,
                                                         UCA_DATA_TYPE);
     }
     udata_writeBlock(pData, data, size);
 
+    // output the constants here
+    udata_writeBlock(pData, consts, sizeof(UCAConstants));
+
     if(noOfcontractions != 0) {
       udata_writeBlock(pData, contractions, noOfcontractions*3*sizeof(UChar));
       udata_writePadding(pData, paddedsize((noOfcontractions*3*sizeof(UChar))) - noOfcontractions*3*sizeof(uint16_t));
@@ -573,21 +784,69 @@ write_uca_table(const char *filename,
                 UErrorCode *status)
 {
     FILE *data = fopen(filename, "r");
-    uint32_t line = 0;
-    UCAElements *element = NULL;
-    UChar variableTopValue = 0;
-    UCATableHeader *myD = (UCATableHeader *)uprv_malloc(sizeof(UCATableHeader));
-    UColOptionSet *opts = (UColOptionSet *)uprv_malloc(sizeof(UColOptionSet));
-    UChar contractionCEs[256][3];
-    uint32_t noOfContractions = 0;
-
-
     if(data == NULL) {
         fprintf(stderr, "Couldn't open file: %s\n", filename);
         return -1;
     }
+    uint32_t line = 0;
+    UCAElements *element = NULL;
+    UChar variableTopValue = 0;
+    UCATableHeader *myD = (UCATableHeader *)uprv_malloc(sizeof(UCATableHeader));
+    /* test for NULL */
+    if(myD == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        fclose(data);
+        return 0;
+    }
+    uprv_memset(myD, 0, sizeof(UCATableHeader));
+    UColOptionSet *opts = (UColOptionSet *)uprv_malloc(sizeof(UColOptionSet));
+    /* test for NULL */
+    if(opts == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        uprv_free(myD);
+        fclose(data);
+        return 0;
+    }
+    uprv_memset(opts, 0, sizeof(UColOptionSet));
+    UChar contractionCEs[512][3];
+    uprv_memset(contractionCEs, 0, 512*3*sizeof(UChar));
+    uint32_t noOfContractions = 0;
+    UCAConstants consts;
+    uprv_memset(&consts, 0, sizeof(consts));
+#if 0
+    UCAConstants consts = {
+      UCOL_RESET_TOP_VALUE,
+      UCOL_FIRST_PRIMARY_IGNORABLE,
+      UCOL_LAST_PRIMARY_IGNORABLE,
+      UCOL_LAST_PRIMARY_IGNORABLE_CONT,
+      UCOL_FIRST_SECONDARY_IGNORABLE,
+      UCOL_LAST_SECONDARY_IGNORABLE,
+      UCOL_FIRST_TERTIARY_IGNORABLE,
+      UCOL_LAST_TERTIARY_IGNORABLE,
+      UCOL_FIRST_VARIABLE,
+      UCOL_LAST_VARIABLE,
+      UCOL_FIRST_NON_VARIABLE,
+      UCOL_LAST_NON_VARIABLE,
 
-    memset(inverseTable, 0xDA, sizeof(int32_t)*3*0xFFFF);
+      UCOL_NEXT_TOP_VALUE,
+/*
+      UCOL_NEXT_FIRST_PRIMARY_IGNORABLE,
+      UCOL_NEXT_LAST_PRIMARY_IGNORABLE,
+      UCOL_NEXT_FIRST_SECONDARY_IGNORABLE,
+      UCOL_NEXT_LAST_SECONDARY_IGNORABLE,
+      UCOL_NEXT_FIRST_TERTIARY_IGNORABLE,
+      UCOL_NEXT_LAST_TERTIARY_IGNORABLE,
+      UCOL_NEXT_FIRST_VARIABLE,
+      UCOL_NEXT_LAST_VARIABLE,
+*/
+
+      PRIMARY_IMPLICIT_MIN,
+      PRIMARY_IMPLICIT_MAX
+    };
+#endif
+
+
+    uprv_memset(inverseTable, 0xDA, sizeof(int32_t)*3*0xFFFF);
 
     opts->variableTopValue = variableTopValue;
     opts->strength = UCOL_TERTIARY;
@@ -597,13 +856,18 @@ write_uca_table(const char *filename,
     opts->caseLevel = UCOL_OFF;         /* do we have an extra case level */
     opts->normalizationMode = UCOL_OFF; /* attribute for normalization */
     opts->hiraganaQ = UCOL_OFF; /* attribute for JIS X 4061, used only in Japanese */
-    /* populate the version info struct with version info*/
-    myD->version[0] = UCOL_BUILDER_VERSION;
-    /*TODO:The fractional rules version should be taken from FractionalUCA.txt*/
-    myD->version[1] = UCOL_FRACTIONAL_UCA_VERSION;
+    opts->numericCollation = UCOL_OFF;
     myD->jamoSpecial = FALSE;
 
-    tempUCATable *t = uprv_uca_initTempTable(myD, opts, NULL, IMPLICIT_TAG, status);
+    tempUCATable *t = uprv_uca_initTempTable(myD, opts, NULL, IMPLICIT_TAG, LEAD_SURROGATE_TAG, status);
+    if(U_FAILURE(*status))
+    {
+        fprintf(stderr, "Failed to init UCA temp table: %s\n", u_errorName(*status));
+        uprv_free(opts);
+        uprv_free(myD);
+        fclose(data);
+        return -1;
+    }
 
 #if 0
     IMPLICIT_TAG = 9,
@@ -632,11 +896,12 @@ struct {
       {0x2F800, 0x2FA1D, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)  },  //7 CJK_IMPLICIT_TAG,   /* 0x2F800-0x2FA1D*/
 #endif
       {0xAC00, 0xD7B0, UCOL_SPECIAL_FLAG | (HANGUL_SYLLABLE_TAG << 24) },  //0 HANGUL_SYLLABLE_TAG,/* AC00-D7AF*/
-      {0xD800, 0xDC00, UCOL_SPECIAL_FLAG | (LEAD_SURROGATE_TAG << 24)  },  //1 LEAD_SURROGATE_TAG,  /* D800-DBFF*/
+      //{0xD800, 0xDC00, UCOL_SPECIAL_FLAG | (LEAD_SURROGATE_TAG << 24)  },  //1 LEAD_SURROGATE_TAG,  /* D800-DBFF*/
       {0xDC00, 0xE000, UCOL_SPECIAL_FLAG | (TRAIL_SURROGATE_TAG << 24) },  //2 TRAIL_SURROGATE DC00-DFFF
-      {0x3400, 0x4DB6, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //3 CJK_IMPLICIT_TAG,   /* 0x3400-0x4DB5*/
-      {0x4E00, 0x9FA6, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //4 CJK_IMPLICIT_TAG,   /* 0x4E00-0x9FA5*/
-      {0xF900, 0xFA2E, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //5 CJK_IMPLICIT_TAG,   /* 0xF900-0xFA2D*/
+      // Now directly handled in the collation code by the swapCJK function. 
+      //{0x3400, 0x4DB6, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //3 CJK_IMPLICIT_TAG,   /* 0x3400-0x4DB5*/
+      //{0x4E00, 0x9FA6, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //4 CJK_IMPLICIT_TAG,   /* 0x4E00-0x9FA5*/
+      //{0xF900, 0xFA2E, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)    },  //5 CJK_IMPLICIT_TAG,   /* 0xF900-0xFA2D*/
       //{0x20000, 0x2A6D7, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)  },  //6 CJK_IMPLICIT_TAG,   /* 0x20000-0x2A6D6*/
       //{0x2F800, 0x2FA1E, UCOL_SPECIAL_FLAG | (CJK_IMPLICIT_TAG << 24)  },  //7 CJK_IMPLICIT_TAG,   /* 0x2F800-0x2FA1D*/
     };
@@ -651,21 +916,26 @@ struct {
     int32_t surrogateCount = 0;
     while(!feof(data)) {
         if(U_FAILURE(*status)) {
-            fprintf(stderr, "Something returned an error %i (%s) while processing line %i of %s. Exiting...\n",
-                *status, u_errorName(*status), line, filename);
+            fprintf(stderr, "Something returned an error %i (%s) while processing line %u of %s. Exiting...\n",
+                *status, u_errorName(*status), (int)line, filename);
             exit(*status);
         }
 
-        element = readAnElement(data, status);
+        element = readAnElement(data, t, &consts, status);
         line++;
+        if(VERBOSE) {
+          fprintf(stdout, "%u ", (int)line);
+        }
         if(element != NULL) {
             // we have read the line, now do something sensible with the read data!
-            if(element->variableTop == TRUE && variableTopValue == 0) {
-                t->options->variableTopValue = element->cPoints[0];
-            }
+
+            // Below stuff was taken care of in readAnElement
+            //if(element->variableTop == TRUE && variableTopValue == 0) {
+            //    t->options->variableTopValue = element->cPoints[0];
+            //}
 
             // if element is a contraction, we want to add it to contractions
-            if(element->cSize > 1) { // this is a contraction
+            if(element->cSize > 1 && element->cPoints[0] != 0xFDD0) { // this is a contraction
               if(UTF_IS_LEAD(element->cPoints[0]) && UTF_IS_TRAIL(element->cPoints[1]) && element->cSize == 2) {
                 surrogateCount++;
               } else {
@@ -683,19 +953,42 @@ struct {
             /* we're first adding to inverse, because addAnElement will reverse the order */
             /* of code points and stuff... we don't want that to happen */
             addToInverse(element, status);
-            uprv_uca_addAnElement(t, element, status);
+            if(!(element->cSize > 1 && element->cPoints[0] == 0xFDD0)) {
+              uprv_uca_addAnElement(t, element, status);
+            }
         }
     }
 
+    if(UCAVersion[0] == 0 && UCAVersion[1] == 0 && UCAVersion[2] == 0 && UCAVersion[3] == 0) {
+        fprintf(stderr, "UCA version not specified. Cannot create data file!\n");
+        uprv_uca_closeTempTable(t);
+        uprv_free(opts);
+        uprv_free(myD);
+        fclose(data);
+        return -1;
+    }
+/*    {
+        uint32_t trieWord = utrie_get32(t->mapping, 0xDC01, NULL);
+    }*/
 
     if (VERBOSE) {
-        fprintf(stdout, "\nLines read: %i\n", line);
-        fprintf(stdout, "Surrogate count: %i\n", surrogateCount);
+        fprintf(stdout, "\nLines read: %u\n", (int)line);
+        fprintf(stdout, "Surrogate count: %i\n", (int)surrogateCount);
         fprintf(stdout, "Raw data breakdown:\n");
         /*fprintf(stdout, "Compact array stage1 top: %i, stage2 top: %i\n", t->mapping->stage1Top, t->mapping->stage2Top);*/
-        fprintf(stdout, "Number of contractions: %i\n", noOfContractions);
-        fprintf(stdout, "Contraction image size: %i\n", t->image->contractionSize);
-        fprintf(stdout, "Expansions size: %i\n", t->expansions->position);
+        fprintf(stdout, "Number of contractions: %u\n", (int)noOfContractions);
+        fprintf(stdout, "Contraction image size: %u\n", (int)t->image->contractionSize);
+        fprintf(stdout, "Expansions size: %i\n", (int)t->expansions->position);
+    }
+
+
+    /* produce canonical closure for table */
+    /* first set up constants for implicit calculation */
+    uprv_uca_initImplicitConstants(consts.UCA_PRIMARY_IMPLICIT_MIN, consts.UCA_PRIMARY_IMPLICIT_MAX, status);
+    /* do the closure */
+    int32_t noOfClosures = uprv_uca_canonicalClosure(t, status);
+    if(noOfClosures != 0) {
+      fprintf(stderr, "Warning: %i canonical closures occured!\n", (int)noOfClosures);
     }
 
     /* test */
@@ -704,17 +997,39 @@ struct {
     if (VERBOSE) {
         fprintf(stdout, "Compacted data breakdown:\n");
         /*fprintf(stdout, "Compact array stage1 top: %i, stage2 top: %i\n", t->mapping->stage1Top, t->mapping->stage2Top);*/
-        fprintf(stdout, "Number of contractions: %i\n", noOfContractions);
-        fprintf(stdout, "Contraction image size: %i\n", t->image->contractionSize);
-        fprintf(stdout, "Expansions size: %i\n", t->expansions->position);
+        fprintf(stdout, "Number of contractions: %u\n", (int)noOfContractions);
+        fprintf(stdout, "Contraction image size: %u\n", (int)t->image->contractionSize);
+        fprintf(stdout, "Expansions size: %i\n", (int)t->expansions->position);
     }
 
-    writeOutData(myData, contractionCEs, noOfContractions, outputDir, copyright, status);
+    if(U_FAILURE(*status)) {
+        fprintf(stderr, "Error creating table: %s\n", u_errorName(*status));
+        uprv_uca_closeTempTable(t);
+        uprv_free(opts);
+        uprv_free(myD);
+        fclose(data);
+        return -1;
+    }
 
-    InverseTableHeader *inverse = assembleInverseTable(status);
+    /* populate the version info struct with version info*/
+    myData->version[0] = UCOL_BUILDER_VERSION;
+    myData->version[1] = UCAVersion[0];
+    myData->version[2] = UCAVersion[1];
+    myData->version[3] = UCAVersion[2];
+    /*TODO:The fractional rules version should be taken from FractionalUCA.txt*/
+    // Removed this macro. Instead, we use the fields below
+    //myD->version[1] = UCOL_FRACTIONAL_UCA_VERSION;
+    //myD->UCAVersion = UCAVersion; // out of FractionalUCA.txt
+    uprv_memcpy(myData->UCAVersion, UCAVersion, sizeof(UVersionInfo));
+    u_getUnicodeVersion(myData->UCDVersion);
+
+    writeOutData(myData, &consts, contractionCEs, noOfContractions, outputDir, copyright, status);
+
+    InverseUCATableHeader *inverse = assembleInverseTable(status);
+    uprv_memcpy(inverse->UCAVersion, UCAVersion, sizeof(UVersionInfo));
     writeOutInverseData(inverse, outputDir, copyright, status);
 
-    uprv_uca_closeTempTable(t);    
+    uprv_uca_closeTempTable(t);
     uprv_free(myD);
     uprv_free(opts);
 
@@ -726,6 +1041,8 @@ struct {
     return 0;
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 static UOption options[]={
     UOPTION_HELP_H,              /* 0  Numbers for those who*/ 
     UOPTION_HELP_QUESTION_MARK,  /* 1   can't count. */
@@ -745,6 +1062,7 @@ int main(int argc, char* argv[]) {
     char filename[300];
     char *basename = NULL;
     const char *copyright = NULL;
+    uprv_memset(&UCAVersion, 0, 4);
 
     U_MAIN_INIT_ARGS(argc, argv);
 
@@ -777,12 +1095,15 @@ int main(int argc, char* argv[]) {
             argv[0], u_getDataDirectory());
         return argc<0 ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
-
     if(options[3].doesOccur) {
-      fprintf(stdout, "genuca version %hu.%hu, ICU tool to read UCA text data and create UCA data tables for collation.\n",
-            ucaDataInfo.formatVersion[0], ucaDataInfo.formatVersion[1]);
-      fprintf(stdout, "Copyright (C) 2000-2001, International Business Machines\n");
-      fprintf(stdout, "Corporation and others.  All Rights Reserved.\n");
+        fprintf(stdout, "genuca version %hu.%hu, ICU tool to read UCA text data and create UCA data tables for collation.\n",
+#if UCONFIG_NO_COLLATION
+            0, 0
+#else
+            UCA_FORMAT_VERSION_0, UCA_FORMAT_VERSION_1
+#endif
+            );
+        fprintf(stdout, U_COPYRIGHT_STRING"\n");
         exit(0);
     }
 
@@ -798,6 +1119,15 @@ int main(int argc, char* argv[]) {
     if (options[7].doesOccur) {
         u_setDataDirectory(options[7].value);
     }
+    /* Initialize ICU */
+    u_init(&status);
+    if (U_FAILURE(status) && status != U_FILE_ACCESS_ERROR) {
+        fprintf(stderr, "%s: can not initialize ICU.  status = %s\n",
+            argv[0], u_errorName(status));
+        exit(1);
+    }
+    status = U_ZERO_ERROR;
+
 
     /* prepare the filename beginning with the source dir */
     uprv_strcpy(filename, srcDir);
@@ -814,7 +1144,40 @@ int main(int argc, char* argv[]) {
       uprv_strcpy(basename, getLongPathname(*argv));
     }
 
+#if 0
+    if(u_getCombiningClass(0x0053) == 0)
+    {
+        fprintf(stderr, "SEVERE ERROR: Normalization data is not functioning! Bailing out.  Was not able to load unorm.dat.\n");
+        exit(1);
+    }
+#endif 
+
+#if UCONFIG_NO_COLLATION
+
+    UNewDataMemory *pData;
+    const char *msg;
+    
+    msg = "genuca writes dummy " UCA_DATA_NAME "." UCA_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
+    fprintf(stderr, "%s\n", msg);
+    pData = udata_create(destdir, UCA_DATA_TYPE, UCA_DATA_NAME, &dummyDataInfo,
+                         NULL, &status);
+    udata_writeBlock(pData, msg, strlen(msg));
+    udata_finish(pData, &status);
+
+    msg = "genuca writes dummy " INVC_DATA_NAME "." INVC_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
+    fprintf(stderr, "%s\n", msg);
+    pData = udata_create(destdir, INVC_DATA_TYPE, INVC_DATA_NAME, &dummyDataInfo,
+                         NULL, &status);
+    udata_writeBlock(pData, msg, strlen(msg));
+    udata_finish(pData, &status);
+
+    return (int)status;
+
+#else
+
     return write_uca_table(filename, destdir, copyright, &status);
+
+#endif
 }
 
 /*
